/*
Install.java

Configure Simulcast.

Z. Yin

8/26/2009
*/

package gov.nasa.gsfc.aisb.simulcast;
import java.io.*;
import java.util.Properties; 
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;
import org.xml.sax.InputSource;
import org.w3c.dom.*;
 
public class Install
{
  public static void main (String[] args) throws Exception
  {
    Document serverDoc, routerDoc;
    XPath xpath;
    NodeList nodes;
    String serverFile = "./configs/server-config.xml";
    String routerFile = "./configs/router-config.xml";
    BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
    String input=null;

    File pathFile = new File(args[0]);
    if (pathFile.getCanonicalPath().endsWith(".bat"))
    {
      System.out.println ("Install.bat starts at: " + pathFile.getCanonicalPath());
    }
    else
    {
      System.out.println ("Install.sh starts at: " + pathFile.getCanonicalPath());
    }

    // Show mode.
    File f = new File("../site.properties");
    if (f.exists())
    {
      System.out.println ("Configuring Simulcast for IPOPP Mode.");
    }
    else
    {
      System.out.println ("Configuring Simulcast for Standalone Mode.");
    }
  
    // Read xml and build a DOM document.
    serverDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new InputSource(serverFile));
    routerDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new InputSource(routerFile));
 
    // Use XPath to find all nodes.
    xpath = XPathFactory.newInstance().newXPath();

    // Change value.
    try
    {
      System.out.print("Enter station id, for example, DRL: ");
      input = br.readLine();
    }
    catch (IOException ioe)
    {
      System.out.println("Input error!");
      System.exit(1);
    }

    nodes = (NodeList)xpath.evaluate("//groundStation/id", serverDoc, XPathConstants.NODESET);
    for (int idx = 0; idx < nodes.getLength(); idx++)
    {
      nodes.item(idx).setTextContent(input);
    }

    nodes = (NodeList)xpath.evaluate("//groundStation/id", routerDoc, XPathConstants.NODESET);
    for (int idx = 0; idx < nodes.getLength(); idx++)
    {
      nodes.item(idx).setTextContent(input);
    }

    try
    {
      System.out.print("Enter station name, for example, Direct Readout Laboratory: ");
      input = br.readLine();
    }
    catch (IOException ioe)
    {
      System.out.println("Input error!");
      System.exit(1);
    }

    nodes = (NodeList)xpath.evaluate("//groundStation/name", serverDoc, XPathConstants.NODESET);
    for (int idx = 0; idx < nodes.getLength(); idx++)
    {
      nodes.item(idx).setTextContent(input);
    }

    nodes = (NodeList)xpath.evaluate("//groundStation/name", routerDoc, XPathConstants.NODESET);
    for (int idx = 0; idx < nodes.getLength(); idx++)
    {
      nodes.item(idx).setTextContent(input);
    }

    try
    {
      System.out.print("Enter lat, for example, 39.005: ");
      input = br.readLine();
    }
    catch (IOException ioe)
    {
      System.out.println("Input error!");
      System.exit(1);
    }

    nodes = (NodeList)xpath.evaluate("//groundStation/lat", serverDoc, XPathConstants.NODESET);
    for (int idx = 0; idx < nodes.getLength(); idx++)
    {
      nodes.item(idx).setTextContent(input);
    }

    nodes = (NodeList)xpath.evaluate("//groundStation/lat", routerDoc, XPathConstants.NODESET);
    for (int idx = 0; idx < nodes.getLength(); idx++)
    {
      nodes.item(idx).setTextContent(input);
    }

    try
    {
      System.out.print("Enter lon, for example, -76.876: ");
      input = br.readLine();
    }
    catch (IOException ioe)
    {
      System.out.println("Input error!");
      System.exit(1);
    }

    nodes = (NodeList)xpath.evaluate("//groundStation/lon", serverDoc, XPathConstants.NODESET);
    for (int idx = 0; idx < nodes.getLength(); idx++)
    {
      nodes.item(idx).setTextContent(input);
    }

    nodes = (NodeList)xpath.evaluate("//groundStation/lon", routerDoc, XPathConstants.NODESET);
    for (int idx = 0; idx < nodes.getLength(); idx++)
    {
      nodes.item(idx).setTextContent(input);
    }

    // Write the DOM document to the file
    Transformer xformer = TransformerFactory.newInstance().newTransformer();
    xformer.transform(new DOMSource(serverDoc), new StreamResult(new File(serverFile)));
    xformer.transform(new DOMSource(routerDoc), new StreamResult(new File(routerFile)));

    System.out.println("Configuration complete.");
  }
}

