/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST
*
*  History:
*
*   8-Feb-06, J Love, GST	Original version.
*  23-Jan-07, J Love, GST	Use of sc.home property.  Added "stop" for
*  				use by Advanced Installer service installation
*  				on Windows.  Use "config.path" property to
*  				pass in configuration file path.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast;
import gov.nasa.gsfc.aisb.simulcast.util.*;

public class Simulcast {
  private static boolean stop = false;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    XmlParser parser = new XmlParser(Util.getFile(System.getProperty("config.path")),"servicesConfig");
    startServices(parser.getElement("services"));
    // On Windows, loop until "stop" is invoked by the executable that was
    // installed as a Windows service.  This should be the only non-daemon
    // thread running so when this thread ends the JVM should exit.
    // On Linux, not used by the JSW but doesn't hurt too bad.
    while (!stop) {
      Util.sleep(250);
    }
  }
  /****************************************************************************
  * startServices.
  ****************************************************************************/
  private static void startServices (XmlParser parser) throws Exception {
    XmlParser[] parsers = parser.getElements("service");
    for (int i = 0; i < parsers.length; i++) {
       startService(parsers[i]);
    }
  }
  /****************************************************************************
  * startService.
  ****************************************************************************/
  private static void startService (XmlParser parser) throws Exception {
    String className = parser.getElementValue("className");
    String[] args = parser.getElement("arguments").getElementValues("argument");
    Class serviceClass = Class.forName(className);
    SimulcastService service = (SimulcastService) serviceClass.newInstance();
    service.start(args);
  }
  /****************************************************************************
  * stop.
  * On Windows, invoked when the service is told to stop.
  ****************************************************************************/
  public static void stop () {
    stop = true;
  }
}
