/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  27-Nov-05, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class AutoDayNightMapViewHandler extends JPanel implements ViewHandler {
  private MapHandler mapHandler;
  private int nBlockRows;
  private int nBlockCols;
  private int nGeoDataRows;
  private int nGeoDataCols;
  private int geoBinSize;
  private ResolutionCombo resolutionCombo;
  private EnhancementAlgorithm dayAlgorithm = null;
  private JComboBox dayEnhancementCombo;
  private PictureControls dayPictureControls;
  private EnhancementAlgorithm nightAlgorithm = null;
  private JComboBox nightEnhancementCombo;
  private PictureControls nightPictureControls;
  private JComboBox solarCombo;
  private HPanel solarPanel;
  private HPanel dayPanel;
  private HPanel nightPanel;
  private Color[][] colors;
  private int[] values = new int[3];
  private Color[] grays = GUtil.getGrays();
  private static final String[] solarChoices = new String[] {"Day","Night"};
  private static final int DAY = 0;
  private static final int NIGHT = 1;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * AutoDayNightMapViewHandler.
  ****************************************************************************/
  public AutoDayNightMapViewHandler (MapHandler mapHandler) {
    this.mapHandler = mapHandler;
    setLayout(new BorderLayout());
    resolutionCombo = new ResolutionCombo(Modis.resolutionChoices,2);
    dayEnhancementCombo = new JComboBox(Modis.dayEnhancementChoices);
    dayEnhancementCombo.setToolTipText("Enhancement algorithm to be applied to day scans (before contrast and brightness adjustments).");
    dayEnhancementCombo.setMaximumSize(dayEnhancementCombo.getPreferredSize());
    dayEnhancementCombo.setSelectedIndex(Modis.DAY_ENHANCEMENT_2);
    dayPictureControls = new PictureControls(1.00,0);
    nightEnhancementCombo = new JComboBox(Modis.nightEnhancementChoices);
    nightEnhancementCombo.setToolTipText("Enhancement algorithm to be applied to night scans (before contrast and brightness adjustments).");
    nightEnhancementCombo.setMaximumSize(nightEnhancementCombo.getPreferredSize());
    nightEnhancementCombo.setSelectedIndex(Modis.NIGHT_ENHANCEMENT_NONE);
    nightPictureControls = new PictureControls(2.50,50);
    dayPanel = new HPanel(new JLabel("Enhancement:"),GUtil.HSpace(5),dayEnhancementCombo,GUtil.HSpace(10),dayPictureControls);
    nightPanel = new HPanel(new JLabel("Enhancement:"),GUtil.HSpace(5),nightEnhancementCombo,GUtil.HSpace(10),nightPictureControls);
    solarPanel = new HPanel(dayPanel);
    solarCombo = new JComboBox(solarChoices);
    solarCombo.setMaximumSize(solarCombo.getPreferredSize());
    solarCombo.addActionListener(new SolarSelected());
    HPanel controls = new HPanel(new HPanel(new CustomLabel("Resolution:"),GUtil.HSpace(5),resolutionCombo/*,GUtil.HSpace(10),portionControls*/),GUtil.HSpace(10),new HPanel(solarPanel,GUtil.HSpace(10),solarCombo));
    add(new MarginPanel(5,1,controls,5,1),BorderLayout.SOUTH);
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    GUtil.invokeAndWait(new PlayStarting());
  }
  private class PlayStarting implements Runnable {
    public void run () {
      setControls(false);
      setEnhancement();
      mapHandler.playStarting();
    }
    private void setEnhancement () {
      String dayClassName = Modis.dayEnhancementClassNames[dayEnhancementCombo.getSelectedIndex()];
      if (dayClassName == null) {
        dayAlgorithm = null;
      } else {
        try {
	  dayAlgorithm = (EnhancementAlgorithm) Class.forName(dayClassName).newInstance();
        } catch (Exception e) {
      	  dayAlgorithm = null;
        }
      }
      String nightClassName = Modis.nightEnhancementClassNames[nightEnhancementCombo.getSelectedIndex()];
      if (nightClassName == null) {
        nightAlgorithm = null;
      } else {
        try {
	  nightAlgorithm = (EnhancementAlgorithm) Class.forName(nightClassName).newInstance();
        } catch (Exception e) {
      	  nightAlgorithm = null;
        }
      }
    }
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {

  }
  /****************************************************************************
  * getRequestMessage.
  ****************************************************************************/
  public String getRequestMessage (String passName) {
    StringBuffer msg = new StringBuffer();
    msg.append(Util.XML_HEADER);
    msg.append("<simulcastMessage>");
    msg.append("<playPass>");
    msg.append("<name>" + passName + "</name>");
    msg.append("<partial>");
    msg.append(mapHandler.getAreaFilter());
    msg.append("<send><geolocated/></send>");
    msg.append("<bands>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_RED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_GREEN_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_BLUE_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.INFRARED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("</bands>");
    msg.append("</partial>");
    if (Client.optionsScreen.compressCheckBox.isSelected()) {
      msg.append("<compress>" + "gzip" + "</compress>");
    }
    msg.append("</playPass>");
    msg.append("</simulcastMessage>");
    return msg.toString();
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    if (headerBlock.getBandCount() != 4) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != Modis.TRUE_COLOR_RED_BAND_INDEX) {
      throw new Exception("unexpected red band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    if (headerBlock.getBandIndex(1) != Modis.TRUE_COLOR_GREEN_BAND_INDEX) {
      throw new Exception("unexpected green band index");
    }
    if (headerBlock.getBandResolution(1) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(1) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(1) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.getBandIndex(2) != Modis.TRUE_COLOR_BLUE_BAND_INDEX) {
      throw new Exception("unexpected blue band index");
    }
    if (headerBlock.getBandResolution(2) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(2) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(2) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.getBandIndex(3) != Modis.INFRARED_BAND_INDEX) {
      throw new Exception("unexpected infrared band index");
    }
    if (headerBlock.getBandResolution(3) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(3) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(3) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    geoBinSize = headerBlock.getGeoResolution() / resolution;
    nGeoDataRows = headerBlock.getGeoBlockRowCount();
    if ((nGeoDataRows * geoBinSize) != nBlockRows) {
      throw new Exception("inconsistent geolocation block rows");
    }
    nGeoDataCols = headerBlock.getGeoBlockColCount();
    if ((nGeoDataCols * geoBinSize) != nBlockCols) {
      throw new Exception("inconsistent geolocation block columns");
    }
    colors = new Color[nBlockRows][nBlockCols];
    mapHandler.headerBlock(headerBlock);
  }
  /****************************************************************************
  * scanGroup.
  ****************************************************************************/
  public void scanGroup (ScanGroup scanGroup) throws Exception {
    switch (scanGroup.getScanBlock().getSolarCode()) {
      case Simulcast.DAY_PASS_CODE: {
      	dayScan(scanGroup);
      	GUtil.setCombo(solarCombo,DAY);
      	break;
      }
      case Simulcast.NIGHT_PASS_CODE: {
      	nightScan(scanGroup);
      	GUtil.setCombo(solarCombo,NIGHT);
      	break;
      }
    }
  }
  private void dayScan (ScanGroup scanGroup) throws Exception {
    byte[][] redBuffer = scanGroup.getBowedBlock().getData()[0];
    byte[][] greenBuffer = scanGroup.getBowedBlock().getData()[1];
    byte[][] blueBuffer = scanGroup.getBowedBlock().getData()[2];
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          values[0] = redBuffer[row][col] & 0xFF;
          values[1] = greenBuffer[row][col] & 0xFF;
          values[2] = blueBuffer[row][col] & 0xFF;
          if (dayAlgorithm != null) {
            dayAlgorithm.enhance(values);
          }
          values[0] = dayPictureControls.adjustValue(values[0]);
          values[1] = dayPictureControls.adjustValue(values[1]);
          values[2] = dayPictureControls.adjustValue(values[2]);
          colors[row][col] = new Color(values[0],values[1],values[2]);
       }
    }
    mapHandler.dataBlock(nGeoDataRows,nGeoDataCols,geoBinSize,
    			 scanGroup.getGeoBlock().getLats(),
			 scanGroup.getGeoBlock().getLons(),colors);
  }
  private void nightScan (ScanGroup scanGroup) throws Exception {
    byte[][] buffer = scanGroup.getBowedBlock().getData()[3];
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          int value = buffer[row][col] & 0xFF;
          if (nightAlgorithm != null) {
			// Not yet supported.
          }
          value = 255 - nightPictureControls.adjustValue(value);
	  colors[row][col] = grays[value];
       }
    }
    mapHandler.dataBlock(nGeoDataRows,nGeoDataCols,geoBinSize,
    			 scanGroup.getGeoBlock().getLats(),
			 scanGroup.getGeoBlock().getLons(),colors);
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {
    GUtil.invokeAndWait(new PlayStopped());
  }
  private class PlayStopped implements Runnable {
    public void run () {
      setControls(true);
      mapHandler.playStopped();
    }
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
  	
    return null;
  }
  /****************************************************************************
  * setControls.
  ****************************************************************************/
  private void setControls (boolean enabled) {
    resolutionCombo.setEnabled(enabled);
    dayEnhancementCombo.setEnabled(enabled);
    nightEnhancementCombo.setEnabled(enabled);
    solarCombo.setEnabled(enabled);
  }
  /****************************************************************************
  * SolarSelected.
  ****************************************************************************/
  private class SolarSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      solarPanel.removeAll();
      switch (solarCombo.getSelectedIndex()) {
        case DAY: {
          solarPanel.add(dayPanel);
          break;
        }
        case NIGHT: {
          solarPanel.add(nightPanel);
          break;
        }
      }
      solarPanel.revalidate();
      solarPanel.repaint();
    }
  }
}
