/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  27-Dec-04, J Love, GST	Original version.
*  17-Mar-05, J Love, GST	Ground station argument (tmp).
*   8-Feb-06, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class FalseColorViewHandler extends JPanel implements ViewHandler {
  private int nBlockRows;
  private int nBlockCols;
  private JComboBox typeCombo;
  private ResolutionCombo resolutionCombo;
  private JComboBox colorCombo;
  private JComboBox redBandCombo;
  private JComboBox greenBandCombo;
  private JComboBox blueBandCombo;
  private PictureControls redPictureControls;
  private PictureControls greenPictureControls;
  private PictureControls bluePictureControls;
  private HPanel colorPanel;
  private HPanel[] colorPanels;
  private ImageScroller imageScroller;
  private Color[] colors;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * FalseColorViewHandler.
  ****************************************************************************/
  public FalseColorViewHandler (int platformCode) {
    setLayout(new BorderLayout());
    typeCombo = new JComboBox(new String[] {"Manual"});
    typeCombo.setToolTipText("Predefined false-color composites (currently none).");
    resolutionCombo = new ResolutionCombo(Modis.resolutionChoices,2);
    colorCombo = new JComboBox(new String[] {"Red","Green","Blue"});
    colorCombo.addActionListener(new ColorSelected());
    redBandCombo = new JComboBox(Modis.dayBandChoices);
    redBandCombo.setToolTipText("Band for the red component of the false-color composite.");
    greenBandCombo = new JComboBox(Modis.dayBandChoices);
    greenBandCombo.setToolTipText("Band for the green component of the false-color composite.");
    blueBandCombo = new JComboBox(Modis.dayBandChoices);
    blueBandCombo.setToolTipText("Band for the blue component of the false-color composite.");
    redPictureControls = new PictureControls(1.00,0);
    greenPictureControls = new PictureControls(1.00,0);
    bluePictureControls = new PictureControls(1.00,0);
    imageScroller = new ImageScroller();
    add(imageScroller,BorderLayout.CENTER);
    colorPanels = new HPanel[] {
      new HPanel(new Component[] {new JLabel("Band:"),GUtil.HSpace(5),redBandCombo,GUtil.HSpace(15),redPictureControls}),
      new HPanel(new Component[] {new JLabel("Band:"),GUtil.HSpace(5),greenBandCombo,GUtil.HSpace(15),greenPictureControls}),
      new HPanel(new Component[] {new JLabel("Band:"),GUtil.HSpace(5),blueBandCombo,GUtil.HSpace(15),bluePictureControls})
    };
    colorPanel = new HPanel(colorPanels[0]);
    add(new MarginPanel(5,1,new HPanel(new Component[] {new JLabel("Resolution:"),GUtil.HSpace(5),resolutionCombo,GUtil.HSpace(15),typeCombo,GUtil.HSpace(15),colorCombo,GUtil.HSpace(15),colorPanel}),5,1),BorderLayout.SOUTH);
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    GUtil.invokeAndWait(new SetControls(false));
    clear();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    imageScroller.clear();
  }
  /****************************************************************************
  * getRequestMessage.
  ****************************************************************************/
  public String getRequestMessage (String passName) {
    StringBuffer msg = new StringBuffer();
    msg.append(Util.XML_HEADER);
    msg.append("<simulcastMessage>");
    msg.append("<playPass>");
    msg.append("<name>" + passName + "</name>");
    msg.append("<partial>");
    msg.append("<send><debowed/></send>");
    msg.append("<bands>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(redBandCombo.getSelectedIndex()) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(greenBandCombo.getSelectedIndex()) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(blueBandCombo.getSelectedIndex()) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("</bands>");
    msg.append("</partial>");
    if (Client.optionsScreen.compressCheckBox.isSelected()) {
      msg.append("<compress>" + "gzip" + "</compress>");
    }
    msg.append("</playPass>");
    msg.append("</simulcastMessage>");
    return msg.toString();
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    // boolean isAscending = Simulcast.isAscending(Client.groundStation,headerBlock);
    boolean isAscending = (headerBlock.getDirectionCode() == Simulcast.ASCENDING_PASS_CODE);
    if (headerBlock.getBandCount() != 3) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != redBandCombo.getSelectedIndex()) {
      throw new Exception("unexpected red band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    if (headerBlock.getBandIndex(1) != greenBandCombo.getSelectedIndex()) {
      throw new Exception("unexpected green band index");
    }
    if (headerBlock.getBandResolution(1) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(1) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(1) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.getBandIndex(2) != blueBandCombo.getSelectedIndex()) {
      throw new Exception("unexpected blue band index");
    }
    if (headerBlock.getBandResolution(2) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(2) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(2) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    String tip = "False-color composite (red component from Band " +
		 Integer.toString(headerBlock.getBandIndex(0)+1) +
		 ", green component from Band " +
		 Integer.toString(headerBlock.getBandIndex(1)+1) +
		 ", and blue component from Band " +
		 Integer.toString(headerBlock.getBandIndex(2)+1) + ").";
    imageScroller.reset(nBlockCols,true,isAscending,tip,
    			Client.optionsScreen.limitViewCheckBox.isSelected());
    colors = new Color[nBlockCols];
  }
  /****************************************************************************
  * scanGroup.
  ****************************************************************************/
  public void scanGroup (ScanGroup scanGroup) throws Exception {
    byte[][] redBuffer = scanGroup.getDebowedBlock().getData()[0];
    byte[][] greenBuffer = scanGroup.getDebowedBlock().getData()[1];
    byte[][] blueBuffer = scanGroup.getDebowedBlock().getData()[2];
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          int redValue = redBuffer[row][col] & 0xFF;
          int greenValue = greenBuffer[row][col] & 0xFF;
          int blueValue = blueBuffer[row][col] & 0xFF;
          redValue = redPictureControls.adjustValue(redValue);
          greenValue = greenPictureControls.adjustValue(greenValue);
          blueValue = bluePictureControls.adjustValue(blueValue);
          colors[col] = new Color(redValue,greenValue,blueValue);
       }
       imageScroller.addRow(colors);
    }
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {
    GUtil.invokeAndWait(new SetControls(true));
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    return imageScroller.getBufferedImage();
  }
  /****************************************************************************
  * SetControls.
  ****************************************************************************/
  private class SetControls implements Runnable {
    private boolean enabled;
    public SetControls (boolean enabled) {
      this.enabled = enabled;
    }
    public void run () {
      typeCombo.setEnabled(enabled);
      resolutionCombo.setEnabled(enabled);
      redBandCombo.setEnabled(enabled);
      greenBandCombo.setEnabled(enabled);
      blueBandCombo.setEnabled(enabled);
    }
  }
  /****************************************************************************
  * ColorSelected.
  ****************************************************************************/
  private class ColorSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      colorPanel.removeAll();
      colorPanel.add(colorPanels[colorCombo.getSelectedIndex()]);
      colorPanel.revalidate();
      colorPanel.repaint();
    }
  }
}
