/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  27-Dec-04, J Love, GST	Original version.
*   8-Feb-06, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

public class ImageScroller extends JPanel {
  private ImageHandler imageHandler = null;
  private JScrollPane scrollPane;
  private JScrollBar vScrollBar;
  private JScrollBar hScrollBar;
  private boolean isAscending;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ImageScroller (constructor).
  ****************************************************************************/
  public ImageScroller () {
    setLayout(new BorderLayout());
    scrollPane = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    				 JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    vScrollBar = scrollPane.getVerticalScrollBar();
    hScrollBar = scrollPane.getHorizontalScrollBar();
    add(scrollPane,BorderLayout.CENTER);
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    if (imageHandler == null) {
      return null;
    }
    return imageHandler.getBufferedImage();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    GUtil.invokeAndWait(new DoClear());
  }
  private class DoClear implements Runnable {
    public void run () {
      imageHandler = null;
      scrollPane.setViewportView(null);
      scrollPane.revalidate();
      scrollPane.repaint();
    }
  }
  /****************************************************************************
  * reset.
  ****************************************************************************/
  public void reset (int width, boolean isColor,
  		     boolean isAscending, String tip, boolean limitView) {
    this.isAscending = isAscending;
    GUtil.invokeAndWait(new DoReset(width,isColor,tip,limitView));
    GUtil.invokeAndWait(new DoAutoCenter());
  }
  private class DoReset implements Runnable {
    private int width;
    private boolean isColor;
    private String tip;
    private boolean limitView;
    public DoReset (int width, boolean isColor, String tip, boolean limitView) {
      this.width = width;
      this.isColor = isColor;
      this.tip = tip;
      this.limitView = limitView;
    }
    public void run () {
      scrollPane.setVerticalScrollBarPolicy(limitView ?
      					    JScrollPane.VERTICAL_SCROLLBAR_NEVER :
      					    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
      imageHandler = (isAscending ?
      		      (limitView ?
      		       (ImageHandler) new AscendingLimitedImageHandler(width,isColor,tip,scrollPane.getViewport()) :
      		       (ImageHandler) new AscendingImageHandler(width,isColor,tip)) :
      		      (limitView ?
      		       (ImageHandler) new DescendingLimitedImageHandler(width,isColor,tip,scrollPane.getViewport()) :
      		       (ImageHandler) new DescendingImageHandler(width,isColor,tip)));
      scrollPane.setViewportView(new MarginPanel(0,0,imageHandler,0,0));
      scrollPane.revalidate();
      scrollPane.repaint();
    }
  }
  /****************************************************************************
  * addRow.
  ****************************************************************************/
  public void addRow (Color[] colors) {
    imageHandler.addRow(colors);
    if (Client.optionsScreen.autoScrollCheckBox.isSelected()) {
      GUtil.invokeAndWait(new DoAutoScroll());
    }
  }
  /****************************************************************************
  * DoAutoScroll.
  ****************************************************************************/
  private class DoAutoScroll implements Runnable {
    public void run () {
      if (isAscending) {
	vScrollBar.setValue(vScrollBar.getMinimum());
      } else {
	vScrollBar.setValue(vScrollBar.getMaximum());
      }
    }
  }
  /****************************************************************************
  * DoAutoCenter.
  ****************************************************************************/
  private class DoAutoCenter implements Runnable {
    public void run () {
      hScrollBar.setValue(((hScrollBar.getMaximum() - hScrollBar.getVisibleAmount()) - hScrollBar.getMinimum()) / 2);
    }
  }
}
