/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*   8-Feb-06, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;

public class InfraredMapViewHandler extends JPanel implements ViewHandler {
  private MapHandler mapHandler;
  private int nBlockRows;
  private int nBlockCols;
  private int nGeoDataRows;
  private int nGeoDataCols;
  private int geoBinSize;
  private ResolutionCombo resolutionCombo;
  private PictureControls pictureControls;
  private Color[][] colors;
  private Color[] grays = GUtil.getGrays();
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * InfraredMapViewHandler.
  ****************************************************************************/
  public InfraredMapViewHandler (MapHandler mapHandler) {
    this.mapHandler = mapHandler;
    setLayout(new BorderLayout());
    resolutionCombo = new ResolutionCombo(Modis.resolutionChoices,2);
    pictureControls = new PictureControls(2.50,50);
    HPanel controls = new HPanel(new JLabel("Resolution:"),GUtil.HSpace(5),resolutionCombo,GUtil.HSpace(15),pictureControls);
    add(new MarginPanel(5,1,controls,5,1),BorderLayout.SOUTH);
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    GUtil.invokeAndWait(new PlayStarting());
  }
  private class PlayStarting implements Runnable {
    public void run () {
      resolutionCombo.setEnabled(false);
      mapHandler.playStarting();
    }
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {

  }
  /****************************************************************************
  * getRequestMessage.
  ****************************************************************************/
  public String getRequestMessage (String passName) {
    StringBuffer msg = new StringBuffer();
    msg.append(Util.XML_HEADER);
    msg.append("<simulcastMessage>");
    msg.append("<playPass>");
    msg.append("<name>" + passName + "</name>");
    msg.append("<partial>");
    msg.append(mapHandler.getAreaFilter());
    msg.append("<send><geolocated/></send>");
    msg.append("<bands>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.INFRARED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("</bands>");
    msg.append("</partial>");
    if (Client.optionsScreen.compressCheckBox.isSelected()) {
      msg.append("<compress>" + "gzip" + "</compress>");
    }
    msg.append("</playPass>");
    msg.append("</simulcastMessage>");
    return msg.toString();
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    if (headerBlock.getBandCount() != 1) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != Modis.INFRARED_BAND_INDEX) {
      throw new Exception("unexpected band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    geoBinSize = headerBlock.getGeoResolution() / resolution;
    nGeoDataRows = headerBlock.getGeoBlockRowCount();
    if ((nGeoDataRows * geoBinSize) != nBlockRows) {
      throw new Exception("inconsistent geolocation block rows");
    }
    nGeoDataCols = headerBlock.getGeoBlockColCount();
    if ((nGeoDataCols * geoBinSize) != nBlockCols) {
      throw new Exception("inconsistent geolocation block columns");
    }
    colors = new Color[nBlockRows][nBlockCols];
    mapHandler.headerBlock(headerBlock);
  }
  /****************************************************************************
  * scanGroup.
  ****************************************************************************/
  public void scanGroup (ScanGroup scanGroup) throws Exception {
    byte[][] buffer = scanGroup.getBowedBlock().getData()[0];
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          int value = buffer[row][col] & 0xFF;
          value = 255 - pictureControls.adjustValue(value);
	  colors[row][col] = grays[value];
       }
    }
    mapHandler.dataBlock(nGeoDataRows,nGeoDataCols,geoBinSize,
    			 scanGroup.getGeoBlock().getLats(),
			 scanGroup.getGeoBlock().getLons(),colors);
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {
    GUtil.invokeAndWait(new PlayStopped());
  }
  private class PlayStopped implements Runnable {
    public void run () {
      resolutionCombo.setEnabled(true);
      mapHandler.playStopped();
    }
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
  	
    return null;
  }
}
