/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  27-Dec-04, J Love, GST	Original version.
*  24-Feb-05, J Love, GST	IR display choices.
*  17-Mar-05, J Love, GST	Ground station argument (tmp).
*   8-Feb-06, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;

public class InfraredViewHandler extends JPanel implements ViewHandler {
  private int nBlockRows;
  private int nBlockCols;
  private ResolutionCombo resolutionCombo;
  private PictureControls pictureControls;
  private ImageScroller imageScroller;
  private Color[] colors;
  private Color[] grays = GUtil.getGrays();
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * InfraredViewHandler.
  ****************************************************************************/
  public InfraredViewHandler (int platformCode) {
    setLayout(new BorderLayout());
    resolutionCombo = new ResolutionCombo(Modis.resolutionChoices,2);
    pictureControls = new PictureControls(2.50,50);
    imageScroller = new ImageScroller();
    HPanel controls = new HPanel(new JLabel("Resolution:"),GUtil.HSpace(5),resolutionCombo,GUtil.HSpace(15),pictureControls);
    add(imageScroller,BorderLayout.CENTER);
    add(new MarginPanel(5,1,controls,5,1),BorderLayout.SOUTH);
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    GUtil.invokeAndWait(new SetControls(false));
    clear();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    imageScroller.clear();
  }
  /****************************************************************************
  * getRequestMessage.
  ****************************************************************************/
  public String getRequestMessage (String passName) {
    StringBuffer msg = new StringBuffer();
    msg.append(Util.XML_HEADER);
    msg.append("<simulcastMessage>");
    msg.append("<playPass>");
    msg.append("<name>" + passName + "</name>");
    msg.append("<partial>");
    msg.append("<send><debowed/></send>");
    msg.append("<bands>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.INFRARED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("</bands>");
    msg.append("</partial>");
    if (Client.optionsScreen.compressCheckBox.isSelected()) {
      msg.append("<compress>" + "gzip" + "</compress>");
    }
    msg.append("</playPass>");
    msg.append("</simulcastMessage>");
    return msg.toString();
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    // boolean isAscending = Simulcast.isAscending(Client.groundStation,headerBlock);
    boolean isAscending = (headerBlock.getDirectionCode() == Simulcast.ASCENDING_PASS_CODE);
    if (headerBlock.getBandCount() != 1) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != Modis.INFRARED_BAND_INDEX) {
      throw new Exception("unexpected band index");
    }
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    imageScroller.reset(nBlockCols,true,isAscending,Modis.infraredTip,
    			Client.optionsScreen.limitViewCheckBox.isSelected());
    // buffer = new byte[nBlockCols];
    colors = new Color[nBlockCols];
  }
  /****************************************************************************
  * scanGroup.
  ****************************************************************************/
  public void scanGroup (ScanGroup scanGroup) throws Exception {
    byte[][] buffer = scanGroup.getDebowedBlock().getData()[0];
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols/*buffer.length*/; col++) {
          int value = buffer[row][col] & 0xFF;
          value = 255 - pictureControls.adjustValue(value);
	  colors[col] = grays[value];
       }
       imageScroller.addRow(colors);
    }
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {
    GUtil.invokeAndWait(new SetControls(true));
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    return imageScroller.getBufferedImage();
  }
  /****************************************************************************
  * SetControls.
  ****************************************************************************/
  private class SetControls implements Runnable {
    private boolean enabled;
    public SetControls (boolean enabled) {
      this.enabled = enabled;
    }
    public void run () {
      resolutionCombo.setEnabled(enabled);
    }
  }
}
