/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*   8-Feb-06, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;

public class OptionsScreen extends HPanel {
  protected JCheckBox passViewerAutoPlayCheckBox;
  protected JCheckBox mercatorViewerAutoPlayCheckBox;
  protected JCheckBox polarViewerAutoPlayCheckBox;
  protected JCheckBox autoZoomCheckBox;
  protected JCheckBox autoEraseCheckBox;
  protected JCheckBox showOrbitsCheckBox;
  protected JCheckBox showGridCheckBox;
  protected JCheckBox showFeaturesCheckBox;
  protected JCheckBox showGroundStationCheckBox;
  protected JCheckBox[] showFeatureCheckBox;
  protected JRadioButton northPoleRadioButton;
  protected JRadioButton southPoleRadioButton;
  protected JRadioButton bothPolesRadioButton;
  protected JCheckBox autoScrollCheckBox;
  protected JCheckBox autoViewSelectionCheckBox;
  protected JCheckBox limitViewCheckBox;
  protected JCheckBox compressCheckBox;
  protected JCheckBox loopCheckBox;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * OptionsScreen.
  ****************************************************************************/
  public OptionsScreen (String autoPlay, boolean mapAutoZoom) throws Exception {
    setBorder(new LineBorder(Color.black,1));
  	
    passViewerAutoPlayCheckBox = new JCheckBox("Space View",autoPlay.equals("space"));
    passViewerAutoPlayCheckBox.setToolTipText("If checked, new passes are automatically played in the Space View.");
    mercatorViewerAutoPlayCheckBox = new JCheckBox("Mercator Map View",autoPlay.equals("mercator"));
    mercatorViewerAutoPlayCheckBox.setToolTipText("If checked, new passes are automatically played in the Mercator Map Viewer.");
    polarViewerAutoPlayCheckBox = new JCheckBox("Polar Map View",autoPlay.equals("polar"));
    polarViewerAutoPlayCheckBox.setToolTipText("If checked, new passes are automatically played in the Polar Map Viewer.");
    VPanel autoPlayPanel = new VPanel(passViewerAutoPlayCheckBox,mercatorViewerAutoPlayCheckBox,polarViewerAutoPlayCheckBox,GUtil.VGlue());
    autoPlayPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Auto Play "));
    
    autoScrollCheckBox = new JCheckBox("Auto Scroll",true);
    autoScrollCheckBox.setToolTipText("If checked, the image is automatically scrolled as needed.");
    autoViewSelectionCheckBox = new JCheckBox("Auto View Selection",true);
    autoViewSelectionCheckBox.setToolTipText("If checked, the view is automatically changed to the most recently selected view that is appropriate.");
    limitViewCheckBox = new JCheckBox("Limit View",false);
    limitViewCheckBox.setToolTipText("If checked, size of view is limited (no scrolling back).");
    VPanel spaceViewPanel = new VPanel(autoScrollCheckBox,autoViewSelectionCheckBox,limitViewCheckBox,GUtil.VGlue());
    spaceViewPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Space View "));

    autoZoomCheckBox = new JCheckBox("Auto Zoom",mapAutoZoom);
    autoZoomCheckBox.setToolTipText("If checked, the map is automatically zoomed to cover the estimated extents of the pass.");
    autoEraseCheckBox = new JCheckBox("Auto Erase",false);
    autoEraseCheckBox.setToolTipText("If checked, the map is automatically erased when a new pass is played.");
    showOrbitsCheckBox = new JCheckBox("Show Orbits",true);
    showOrbitsCheckBox.setToolTipText("If checked, satellite orbits/positions are shown on the map.");
    showOrbitsCheckBox.addActionListener(new MapOptionChanged());
    showGridCheckBox = new JCheckBox("Show Grid",true);
    showGridCheckBox.setToolTipText("If checked, latitude/longitude grid lines are shown on the map.");
    showGridCheckBox.addActionListener(new MapOptionChanged());
    showGroundStationCheckBox = new JCheckBox("Show Ground Station",true);
    showGroundStationCheckBox.setToolTipText("If checked, the ground station is shown on the map.");
    showGroundStationCheckBox.addActionListener(new MapOptionChanged());
    showFeaturesCheckBox = new JCheckBox("Show Features",true);
    showFeaturesCheckBox.setToolTipText("If checked, border/coastline features are shown on the map.");
    showFeaturesCheckBox.addActionListener(new MapOptionChanged());
    showFeatureCheckBox = new JCheckBox[MapHandler.shpNames.length];
    VPanel mapViewPanel = new VPanel(autoZoomCheckBox,autoEraseCheckBox,showOrbitsCheckBox,showGridCheckBox,showGroundStationCheckBox,showFeaturesCheckBox,GUtil.VGlue());
    mapViewPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Map Views "));
    
    VPanel featuresPanel = new VPanel();
    for (int i = 0; i < MapHandler.shpNames.length; i++) {
       showFeatureCheckBox[i] = new JCheckBox("Show "+MapHandler.shpNames[i],MapHandler.shpDefaultStates[i]);
       showFeatureCheckBox[i].setToolTipText("If checked, " + MapHandler.shpNames[i] + " are shown on the map.");
       showFeatureCheckBox[i].addActionListener(new MapOptionChanged());
       featuresPanel.add(showFeatureCheckBox[i]);
    }
    featuresPanel.add(GUtil.VGlue());
    featuresPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Map Features "));
    
    northPoleRadioButton = new JRadioButton("Show North Pole",true);
    northPoleRadioButton.setToolTipText("If checked, the north pole is displayed.");
    northPoleRadioButton.addActionListener(new PolarHemisphereChanged());
    southPoleRadioButton = new JRadioButton("Show South Pole",false);
    southPoleRadioButton.setToolTipText("If checked, the south pole is displayed.");
    southPoleRadioButton.addActionListener(new PolarHemisphereChanged());
    bothPolesRadioButton = new JRadioButton("Show Both Poles",false);
    bothPolesRadioButton.setToolTipText("If checked, both poles are displayed.");
    bothPolesRadioButton.addActionListener(new PolarHemisphereChanged());
    GUtil.buttonGroup(northPoleRadioButton,southPoleRadioButton,bothPolesRadioButton);
    VPanel polarPanel = new VPanel(northPoleRadioButton,southPoleRadioButton,bothPolesRadioButton,GUtil.VGlue());
    polarPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Polar Hemisphere "));

    compressCheckBox = new JCheckBox("Compress",false);
    compressCheckBox.setToolTipText("If checked, uses a compressed data stream during playback (helps slow connections).");
    loopCheckBox = new JCheckBox("Loop",false);
    loopCheckBox.setToolTipText("If checked, plays passes in order and repeats indefinitely.");
    VPanel playerPanel = new VPanel(compressCheckBox,loopCheckBox,GUtil.VGlue());
    playerPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Pass Player "));

    add(new VPanel(GUtil.VSpace(5),new HPanel(GUtil.HSpace(5),autoPlayPanel,GUtil.HSpace(5),spaceViewPanel,GUtil.HSpace(5),mapViewPanel,GUtil.HSpace(5),featuresPanel,GUtil.HSpace(5),polarPanel,GUtil.HSpace(5),playerPanel,GUtil.HGlue()),GUtil.VSpace(5)));
  }
  /****************************************************************************
  * PolarHemisphereChanged.
  ****************************************************************************/
  private class PolarHemisphereChanged implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      if (bothPolesRadioButton.isSelected()) {
	Client.polarViewer = Client.bothPolarViewer;
      }
      if (northPoleRadioButton.isSelected()) {
	Client.polarViewer = Client.northPolarViewer;
      }
      if (southPoleRadioButton.isSelected()) {
	Client.polarViewer = Client.southPolarViewer;
      }
      Client.polarPanel.removeAll();
      Client.polarPanel.add(Client.polarViewer);
      Client.polarPanel.revalidate();
      Client.polarPanel.repaint();
    }
  }
  /****************************************************************************
  * MapOptionChanged.
  ****************************************************************************/
  private class MapOptionChanged implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      Client.mercatorViewer.redraw();
      Client.polarViewer.redraw();
    }
  }
}
