/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*   8-Feb-06, J Love, GST	Original version.
*  17-Mar-06, J Love, GST	More pass details.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

public class PassHandler extends VPanel {
  private String passName;
  private int platformCode = Simulcast.UNKNOWN;
  private int instrumentCode = Simulcast.UNKNOWN;
  private int startSolarCode = Simulcast.UNKNOWN;
  private CustomLabel groundStationField;
  private CustomLabel platformField;
  private CustomLabel instrumentField;
  private CustomLabel startedField;
  private CustomLabel endedField;
  private CustomLabel directionField;
  private CustomLabel solarField;
  private CustomLabel durationField;
  private CustomLabel scansField;
  private CustomLabel errorField;
  private VPanel detailsPanel;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PassHandler.
  ****************************************************************************/
  public PassHandler (XmlParser parser) throws Exception {
    setBorder(new EmptyBorder(5,5,0,5));
    passName = parser.getElementValue("name");
    groundStationField = new CustomLabel(" ",160);
    platformField = new CustomLabel(" ",70);
    instrumentField = new CustomLabel(" ",70);
    startedField = new CustomLabel(" ",200);
    endedField = new CustomLabel(" ",200);
    directionField = new CustomLabel(" ",100);
    solarField = new CustomLabel(" ",100);
    durationField = new CustomLabel(" ",90);
    scansField = new CustomLabel(" ",90);
    errorField = new CustomLabel("");
    detailsPanel = new VPanel();
    detailsPanel.addComponentListener(new DetailsPanelChanged());
    HPanel h0 = new HPanel();
    h0.setBorder(new LineBorder(Color.black,1));
    HPanel h1 = new HPanel();
    h1.add(GUtil.HSpace(3));
    h1.add(new VPanel(new CustomLabel(passName,160),groundStationField));
    h1.add(GUtil.HSpace(15));
    h1.add(new VPanel(platformField,instrumentField));
    h1.add(GUtil.HSpace(15));
    h1.add(new VPanel(startedField,endedField));
    h1.add(GUtil.HSpace(15));
    h1.add(new VPanel(directionField,solarField));
    h1.add(GUtil.HSpace(15));
    h1.add(new VPanel(durationField,scansField));
    h0.add(GUtil.HSpace(5));
    VPanel v1 = new VPanel(new HPanel(h1,GUtil.HGlue()),new HPanel(errorField,GUtil.HGlue()),new HPanel(new ArrowPanel(false,ArrowButton.RIGHT,"Details",18,detailsPanel),GUtil.HGlue()));
    v1.setAlignmentY(TOP_ALIGNMENT);
    h0.add(v1);
    h0.add(GUtil.HSpace(5));
    h0.add(GUtil.HGlue());
    JButton viewButton = new JButton("Space");
    viewButton.addActionListener(new ViewButtonClicked());
    VPanel v2 = new VPanel(GUtil.VSpace(5),viewButton);
    v2.setAlignmentY(TOP_ALIGNMENT);
    h0.add(v2);
    h0.add(GUtil.HSpace(10));
    JButton mercatorButton = new JButton("Mercator");
    mercatorButton.addActionListener(new MercatorButtonClicked());
    VPanel v3 = new VPanel(GUtil.VSpace(5),mercatorButton);
    v3.setAlignmentY(TOP_ALIGNMENT);
    h0.add(v3);
    h0.add(GUtil.HSpace(10));
    JButton polarButton = new JButton("Polar");
    polarButton.addActionListener(new PolarButtonClicked());
    VPanel v4 = new VPanel(GUtil.VSpace(5),polarButton);
    v4.setAlignmentY(TOP_ALIGNMENT);
    h0.add(v4);
    h0.add(GUtil.HSpace(10));
    add(h0);
    set(parser);
  }
  /****************************************************************************
  * passDetails.
  ****************************************************************************/
  public void passDetails (XmlParser parser) throws Exception {
    set(parser);
  }
  /****************************************************************************
  * passUpdated.
  ****************************************************************************/
  public void passUpdated (XmlParser parser) throws Exception {
    set(parser);
    Util.startThread(new DoAutoPlay());
  }
  private class DoAutoPlay implements Runnable {
    public void run () {
      if (Client.optionsScreen.passViewerAutoPlayCheckBox.isSelected()) {
      	Util.sleep(1.0);
        loadView(true);
      }
      if (Client.optionsScreen.mercatorViewerAutoPlayCheckBox.isSelected()) {
      	Util.sleep(1.0);
        loadMercator(true);
      }
      if (Client.optionsScreen.polarViewerAutoPlayCheckBox.isSelected()) {
      	Util.sleep(1.0);
        loadPolar(true);
      }
    }
  }
  /****************************************************************************
  * passEnded.
  ****************************************************************************/
  public void passEnded (XmlParser parser) throws Exception {
    set(parser);
  }
  /****************************************************************************
  * set.
  ****************************************************************************/
  private void set (XmlParser parser) throws Exception {
    long startedAt = 0, endedAt = 0;
    if (parser.hasElement("startedAt")) {
      startedAt = Long.parseLong(parser.getElementValue("startedAt"));
      platformCode = Integer.parseInt(parser.getElementValue("platformCode"));
      instrumentCode = Integer.parseInt(parser.getElementValue("instrumentCode"));
      startSolarCode = Integer.parseInt(parser.getElementValue("startSolarCode"));
      GroundStation groundStation = new GroundStation(parser.getElement("groundStation"));
      groundStationField.update(groundStation.getName());
      platformField.update(Simulcast.getPlatformName(platformCode));
      instrumentField.update(Simulcast.getInstrumentName(instrumentCode));
      startedField.update(encodeDateTime(startedAt));
      solarField.update(getPassSolar(startSolarCode,Simulcast.UNKNOWN));
      directionField.update(Simulcast.getPassDirection(Integer.parseInt(parser.getElementValue("directionCode"))));
    }
    if (parser.hasElement("endedAt")) {
      endedAt = Long.parseLong(parser.getElementValue("endedAt"));
      endedField.update(encodeDateTime(endedAt));
      solarField.update(getPassSolar(startSolarCode,Integer.parseInt(parser.getElementValue("endSolarCode"))));
      durationField.update(encodeDuration(startedAt,endedAt));
      scansField.update(encodeScanCounts(startSolarCode,Integer.parseInt(parser.getElementValue("dayScanCount")),Integer.parseInt(parser.getElementValue("nightScanCount"))));
    }
    if (parser.hasElement("error")) {
      errorField.update("ERROR..."+parser.getElementValue("error"),Color.red);
    }
    if (parser.hasElement("more")) {
      setMore(parser.getElement("more"));
    }
  }
  private void setMore (XmlParser parser) throws Exception {
    detailsPanel.removeAll();
    if (parser.hasElement("geolocated")) {
      detailsPanel.add(new HPanel(new CustomLabel("Geolocation: "),GUtil.HGlue()));
      detailsPanel.add(new HPanel(GUtil.HSpace(10),new CustomLabel(parser.getElementValue("geoResolution")+"m, "+parser.getElementValue("geoBlockRowCount")+"x"+parser.getElementValue("geoBlockColCount")),GUtil.HGlue()));
    } else {
      detailsPanel.add(new HPanel(new CustomLabel("Not geolocated."),GUtil.HGlue()));
    }
    if (!parser.hasElement("debowed")) {
      detailsPanel.add(new HPanel(new CustomLabel("Not debowed."),GUtil.HGlue()));
    }
    detailsPanel.add(new HPanel(new CustomLabel("Coarsest: "),GUtil.HGlue()));
    detailsPanel.add(new HPanel(GUtil.HSpace(10),new CustomLabel(parser.getElementValue("coarsestResolution")+"m, "+parser.getElementValue("coarsestBlockRowCount")+"x"+parser.getElementValue("coarsestBlockColCount")),GUtil.HGlue()));
    detailsPanel.add(new HPanel(new CustomLabel("Bands:"),GUtil.HGlue()));
    XmlParser[] parsers = parser.getElement("bands").getElements("band");
    for (int b = 0; b < parsers.length; b++) {
       detailsPanel.add(new HPanel(GUtil.HSpace(10),new CustomLabel(Modis.dayBandChoices[Integer.parseInt(parsers[b].getElementValue("bandIndex"))]+", "+parsers[b].getElementValue("bandResolution")+"m, "+parsers[b].getElementValue("blockRowCount")+"x"+parsers[b].getElementValue("blockColCount")),GUtil.HGlue()));
    }
  }
  /****************************************************************************
  * getPassName.
  ****************************************************************************/
  public String getPassName () {
    return passName;
  }
  /****************************************************************************
  * getPassSolar.
  ****************************************************************************/
  private String getPassSolar (int startSolarCode, int endSolarCode) {
    StringBuffer passSolar = new StringBuffer(Simulcast.getPassSolar(startSolarCode));
    if (endSolarCode != Simulcast.UNKNOWN && endSolarCode != startSolarCode) {
      passSolar.append("/" + Simulcast.getPassSolar(endSolarCode));
    }
    return passSolar.toString();
  }
  /****************************************************************************
  * encodeDateTime.
  ****************************************************************************/
  private String encodeDateTime (long at) {
    if (at != Simulcast.UNKNOWN) {
      return Util.encodeDateTime(at);
    }
    return " ";
  }
  /****************************************************************************
  * encodeDuration.
  ****************************************************************************/
  private String encodeDuration (long startedAt, long endedAt) {
    if (endedAt != Simulcast.UNKNOWN) {
      int duration = (int) (endedAt - startedAt);
      return Util.encodeDuration(duration);
    }
    return " ";
  }
  /****************************************************************************
  * encodeScanCounts
  ****************************************************************************/
  private String encodeScanCounts (int startSolarCode, int nDayScans, int nNightScans) {
    if (nDayScans > 0) {
      if (nNightScans > 0) {
	if (startSolarCode == Simulcast.DAY_PASS_CODE) {
	  return Integer.toString(nDayScans) + "/" + Integer.toString(nNightScans);
	} else {
	  return Integer.toString(nNightScans) + "/" + Integer.toString(nDayScans);
	}
      } else {
        return Integer.toString(nDayScans);
      }
    } else {
      if (nNightScans > 0) {
        return Integer.toString(nNightScans);
      } else {
        return " ";
      }
    } 
  }
  /****************************************************************************
  * DetailsPanelChanged.
  ****************************************************************************/
  private class DetailsPanelChanged extends ComponentAdapter {
    public void componentShown (ComponentEvent ce) {
      Client.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
         "<sendPassDetails>" + "<passName>" + passName + "</passName>" +
         "</sendPassDetails>" + "</simulcastMessage>");
    }
  }
  /****************************************************************************
  * ViewButtonClicked.
  ****************************************************************************/
  private class ViewButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      loadView(false);
    }
  }
  /****************************************************************************
  * MercatorButtonClicked.
  ****************************************************************************/
  private class MercatorButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      loadMercator(false);
    }
  }
  /****************************************************************************
  * PolarButtonClicked.
  ****************************************************************************/
  private class PolarButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      loadPolar(false);
    }
  }
  /****************************************************************************
  * loadView.
  ****************************************************************************/
  public boolean loadView (boolean play) {
    if (platformCode != Simulcast.UNKNOWN && instrumentCode != Simulcast.UNKNOWN && startSolarCode != Simulcast.UNKNOWN) {
      Client.passViewer.loadPass(passName,platformCode,instrumentCode,startSolarCode,play);
      Client.switchToPassViewer();
      return true;
    }
    return false;
  }
  /****************************************************************************
  * loadMercator.
  ****************************************************************************/
  public boolean loadMercator (boolean play) {
    if (platformCode != Simulcast.UNKNOWN && instrumentCode != Simulcast.UNKNOWN && startSolarCode != Simulcast.UNKNOWN) {
      Client.switchToMercatorViewer();
      Client.mercatorViewer.loadPass(passName,platformCode,instrumentCode,startSolarCode,play);
      return true;
    }
    return false;
  }
  /****************************************************************************
  * loadPolar.
  ****************************************************************************/
  public boolean loadPolar (boolean play) {
    if (platformCode != Simulcast.UNKNOWN && instrumentCode != Simulcast.UNKNOWN && startSolarCode != Simulcast.UNKNOWN) {
      Client.polarViewer.loadPass(passName,platformCode,instrumentCode,startSolarCode,play);
      Client.switchToPolarViewer();
      return true;
    }
    return false;
  }
}
