/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  17-Dec-04, J Love, GST	Original version.
*  17-Mar-05, J Love, GST	Ground station argument (tmp).
*   8-Feb-06, J Love, GST	New style (no more table).
*  12-Mar-06, J Love, GST	More pass details.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import javax.swing.*;

public class PassManager extends JPanel {
  private StringMap passMap = new StringMap();
  private VPanel handlersPanel;
  private JScrollBar vScrollBar;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PassManager.
  ****************************************************************************/
  public PassManager () {
    setLayout(new BorderLayout());
    handlersPanel = new VPanel();
    JScrollPane scrollPane = new JScrollPane(new MarginPanel(0,0,handlersPanel,5,0));
    vScrollBar = scrollPane.getVerticalScrollBar();
    add(scrollPane,BorderLayout.CENTER);
    HPanel foot = new HPanel(Client.serverHandler,GUtil.HGlue());
    add(new MarginPanel(3,2,foot,3,2),BorderLayout.SOUTH);
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    passMap.clear();
    handlersPanel.removeAll();
    revalidate();
    repaint();
  }
  /****************************************************************************
  * handlePassListing.
  ****************************************************************************/
  public void handlePassListing (XmlParser parser) throws Exception {
    passMap.clear();
    handlersPanel.removeAll();
    XmlParser[] parsers = parser.getElements("passDetails");
    for (int i = 0; i < parsers.length; i++) {
       PassHandler handler = new PassHandler(parsers[i]);
       passMap.put(handler.getPassName(),handler);
       handlersPanel.add(handler);
    }
    revalidate();
    repaint();
    SwingUtilities.invokeLater(new DoAutoScroll());
  }
  /****************************************************************************
  * handlePassDetails.
  ****************************************************************************/
  public void handlePassDetails (XmlParser parser) throws Exception {
    String passName = parser.getElementValue("name");
    PassHandler handler = (PassHandler) passMap.get(passName);
    handler.passDetails(parser);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * handlePassStarted.
  ****************************************************************************/
  public void handlePassStarted (XmlParser parser) throws Exception {
    PassHandler handler = new PassHandler(parser);
    passMap.put(handler.getPassName(),handler);
    handlersPanel.add(handler);
    revalidate();
    repaint();
    SwingUtilities.invokeLater(new DoAutoScroll());
  }
  /****************************************************************************
  * handlePassUpdated.
  ****************************************************************************/
  public void handlePassUpdated (XmlParser parser) throws Exception {
    String passName = parser.getElementValue("name");
    PassHandler handler = (PassHandler) passMap.get(passName);
    handler.passUpdated(parser);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * handlePassEnded.
  ****************************************************************************/
  public void handlePassEnded (XmlParser parser) throws Exception {
    String passName = parser.getElementValue("name");
    PassHandler handler = (PassHandler) passMap.get(passName);
    handler.passEnded(parser);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * handlePassRemoved.
  ****************************************************************************/
  public void handlePassRemoved (XmlParser parser) throws Exception {
    String name = parser.getElementValue("name");
    PassHandler handler = (PassHandler) passMap.remove(name);
    handlersPanel.remove(handler);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * getNextPassHandler.
  ****************************************************************************/
  public PassHandler getNextPassHandler (String passName) {
    int index = passMap.indexOf(passName);
    if (index < passMap.size() - 1) {
      return (PassHandler) passMap.get(index+1);
    }
    return (PassHandler) passMap.get(0);
  }
  /****************************************************************************
  * DoAutoScroll.
  ****************************************************************************/
  private class DoAutoScroll implements Runnable {
    public void run () {
      vScrollBar.setValue(vScrollBar.getMaximum());
    }
  }
}
