/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  17-Dec-04, J Love, GST	Original version.
*   8-Feb-06, J Love, GST	Appearance tweaking.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.event.*;

public class PortionControls extends JPanel {
  private JComboBox widthCombo;
  private JComboBox offCenterCombo;
  private static final int WIDTH_100_PCT = 0;
  // private static final int WIDTH_75_PCT = 1;
  private static final int WIDTH_50_PCT = 2;
  private static final int WIDTH_25_PCT = 3;
  // private static final int WIDTH_10_PCT = 4;
  // private static final int WIDTH_5_PCT = 5;
  protected static final String[] widthChoices = new String[] {
    "100%","75%","50%","25%","10%","5%"
  };
  protected static final String[] widthValues = new String[] {
    "100","75","50","25","10","5"
  };
  private static final int CENTER = 4;
  protected static final String[] offCenterChoices = new String[] {
    "100%","75%","50%","25%","0%","-25%","-50%","-75%","-100%"
  };
  protected static final String[] offCenterValues = new String[] {
    "100","75","50","25","0","-25","-50","-75","-100"
  };
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PortionControls.
  ****************************************************************************/
  public PortionControls () {
    setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
    widthCombo = new JComboBox(widthChoices);
    widthCombo.setToolTipText("Percentage of the instrument swath to be displayed.");
    widthCombo.setSelectedIndex(WIDTH_100_PCT);
    widthCombo.addActionListener(new WidthSelected());
    offCenterCombo = new JComboBox(offCenterChoices);
    offCenterCombo.setToolTipText("Portion of the instrument swath to be displayed (relative to the satellite's direction of travel).  100% is far left.  -100% is far right");
    offCenterCombo.setSelectedIndex(CENTER);
    offCenterCombo.setEnabled(false);
    add(new HPanel(new JLabel("Width:"),GUtil.HSpace(5),widthCombo,GUtil.HSpace(10),new JLabel("Position:"),GUtil.HSpace(5),offCenterCombo));
  }
  /****************************************************************************
  * getWidthPct.
  ****************************************************************************/
  public String getWidthPct () {
    return widthValues[widthCombo.getSelectedIndex()];
  }
  /****************************************************************************
  * getOffCenterPct.
  ****************************************************************************/
  public String getOffCenterPct () {
    return offCenterValues[offCenterCombo.getSelectedIndex()];
  }
  /****************************************************************************
  * setEnabled.
  * Assumed to be invoked from the event thread.
  ****************************************************************************/
  public void setEnabled (boolean enabled) {
    if (enabled) {
      widthCombo.setEnabled(true);
      adjustOffCenterCombo();
    } else {
      widthCombo.setEnabled(false);
      offCenterCombo.setEnabled(false);
    }
  }
  /****************************************************************************
  * WidthSelected.
  * This can only be invoked when the controls are enabled.
  ****************************************************************************/
  private class WidthSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      adjustOffCenterCombo();
    }
  }
  /****************************************************************************
  * adjustOffCenterCombo.
  ****************************************************************************/
  private void adjustOffCenterCombo () {
    switch (widthCombo.getSelectedIndex()) {
      case WIDTH_100_PCT: {
      	offCenterCombo.setSelectedIndex(CENTER);
        offCenterCombo.setEnabled(false);
        break;
      }
      default: {
        offCenterCombo.setEnabled(true);
        break;
      }
    }
  }
  /****************************************************************************
  * adjustWidthForResolution.
  ****************************************************************************/
  public void adjustWidthForResolution (int resolution) {
    if (resolution <= 250) {
      if (widthCombo.getSelectedIndex() < WIDTH_25_PCT) {
      	widthCombo.setSelectedIndex(WIDTH_25_PCT);
      }
      return;
    }
    if (resolution <= 500) {
      if (widthCombo.getSelectedIndex() < WIDTH_50_PCT) {
      	widthCombo.setSelectedIndex(WIDTH_50_PCT);
      }
      return;
    }
    return;
  }
}
