/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*   22-Jul-04, J Love, GST	Original version.
*    8-Feb-06, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.util.*;

public class RateHandler {
  private LinkedList<Date> satTimes = new LinkedList<Date>();
  private LinkedList<Date> wallTimes = new LinkedList<Date>();
  private LinkedList<Integer> byteCounts = new LinkedList<Integer>();
  private static final int SLIDING_SIZE = 5;
  /****************************************************************************
  * RateHandler.
  ****************************************************************************/
  public RateHandler () {
  	
  }
  /****************************************************************************
  * scanReceived.
  ****************************************************************************/
  public void scanReceived (ScanGroup scanGroup) {
    Date wallTime = new Date();
    Date satTime = new Date(scanGroup.getScanBlock().getScanTime());
    wallTimes.addLast(wallTime);
    satTimes.addLast(satTime);
    int byteCount = scanGroup.getScanBlock().size() +
    		    (scanGroup.getGeoBlock() != null ? scanGroup.getGeoBlock().size() : 0) +
    		    (scanGroup.getBowedBlock() != null ? scanGroup.getBowedBlock().size() : 0) +
    		    (scanGroup.getDebowedBlock() != null ? scanGroup.getDebowedBlock().size() : 0);
    byteCounts.addLast(new Integer(byteCount));
  }
  /****************************************************************************
  * toText.
  ****************************************************************************/
  public String toText (boolean ended) {
    if (ended) {
      if (wallTimes.size() > 1) {
        long elapsedMS = wallTimes.getLast().getTime() - wallTimes.getFirst().getTime();
        int pct = (int) Math.round((100.0 * (satTimes.getLast().getTime() - satTimes.getFirst().getTime())) / elapsedMS);
        long totalCount = getTotal(byteCounts.size()-1);
        int dataRate = (int) Math.round(totalCount / (elapsedMS/1000.0));
        return "Average: " + Util.encodeByteCount(dataRate) + "/sec, " + Integer.toString(pct) + "%";
      }
      return null;
    }
    if (wallTimes.size() < SLIDING_SIZE) {
      return null;
    }
    long elapsedMS = ((Date)wallTimes.getLast()).getTime() - ((Date)wallTimes.get(wallTimes.size()-SLIDING_SIZE)).getTime();
    int pct = (int) Math.round((100.0 * (((Date)satTimes.getLast()).getTime() - ((Date)satTimes.get(satTimes.size()-SLIDING_SIZE)).getTime())) / elapsedMS);
    long slidingCount = getTotal(SLIDING_SIZE-1);
    int dataRate = (int) Math.round(slidingCount / (elapsedMS/1000.0));
    return Util.encodeByteCount(dataRate) + "/sec, " + Integer.toString(pct) + "%";
  }
  /****************************************************************************
  * getTotal.
  ****************************************************************************/
  private long getTotal (int nCounts) {
    long total = 0;
    for (int i = byteCounts.size()-nCounts; i < byteCounts.size(); i++) {
       total += byteCounts.get(i).intValue();
    }
    return total;
  }
}
