/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  19-Dec-04, J Love, GST		Original version.
*  10-Feb-06, J Love, GST		Java 1.5.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.*;
import javax.media.jai.*;
import com.sun.media.jai.codec.*;

public class SaveAs extends JDialog {
  private JDialog thisDialog;
  // private String passName;
  private BufferedImage bi;
  private int fullWidth;
  private int fullHeight;
  private int halfWidth;
  private int halfHeight;
  private int quarterWidth;
  private int quarterHeight;
  private JRadioButton fullResolutionButton;
  private JRadioButton halfResolutionButton;
  private JRadioButton quarterResolutionButton;
  private JRadioButton customResolutionButton;
  private JTextField widthField;
  private JTextField heightField;
  private JCheckBox maintainCheckBox;
  private JRadioButton highQualityButton;
  private JRadioButton mediumQualityButton;
  private JRadioButton lowQualityButton;
  private JRadioButton customQualityButton;
  private JTextField qualityField;
  private JTextField pathField;
  private JLabel statusLabel;
  private JPanel containerPanel;
  private JPanel mainPanel;
  private static String lastBrowsedDir = null;
  private static final int HIGH_QUALITY = 100;
  private static final int MEDIUM_QUALITY = 75;
  private static final int LOW_QUALITY = 50;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * SaveAs.
  ****************************************************************************/
  public SaveAs (String passName, BufferedImage bi) {
    super(Client.frame,"Save As",false);
    thisDialog = this;
    // this.passName = passName;
    this.bi = bi;
    fullWidth = bi.getWidth();
    fullHeight = bi.getHeight();
    containerPanel = new JPanel(new BorderLayout());
    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(containerPanel,BorderLayout.CENTER);
    mainPanel = new JPanel(new BorderLayout());
    // Resolution...
    fullResolutionButton = new JRadioButton("  " + Integer.toString(fullHeight) + " x " + Integer.toString(fullWidth) + " (full resolution)",true);
    fullResolutionButton.setToolTipText("Full resolution in pixels (rows x columns).");
    halfHeight = Math.round(((float)fullHeight)/2);
    halfWidth = calcWidth(halfHeight);
    halfResolutionButton = new JRadioButton("  " + Integer.toString(halfHeight) + " x " + Integer.toString(halfWidth) + " (half resolution)",false);
    halfResolutionButton.setToolTipText("Half resolution in pixels (rows x columns).");
    quarterHeight = Math.round(((float)fullHeight)/4);
    quarterWidth = calcWidth(quarterHeight);
    quarterResolutionButton = new JRadioButton("  " + Integer.toString(quarterHeight) + " x " + Integer.toString(quarterWidth) + " (quarter resolution)",false);
    quarterResolutionButton.setToolTipText("Quarter resolution in pixels (rows x columns).");
    customResolutionButton = new JRadioButton("",false);
    customResolutionButton.setToolTipText("Custom resolution in pixels (rows x columns).");
    ButtonGroup resGroup = new ButtonGroup();
    resGroup.add(fullResolutionButton);
    resGroup.add(halfResolutionButton);
    resGroup.add(quarterResolutionButton);
    resGroup.add(customResolutionButton);
    heightField = new JTextField(5);
    heightField.setToolTipText("Number of rows (height in pixels).");
    heightField.setMaximumSize(heightField.getPreferredSize());
    heightField.addKeyListener(new HeightFieldChanged());
    widthField = new JTextField(5);
    widthField.setToolTipText("Number of columns (width in pixels).");
    widthField.setMaximumSize(widthField.getPreferredSize());
    widthField.addKeyListener(new WidthFieldChanged());
    maintainCheckBox = new JCheckBox("Maintain Aspect Ratio",true);
    maintainCheckBox.setToolTipText("If checked, as the width or height is modfied, the other will be adjusted to maintain the image's aspect ratio.");
    VPanel sizePanel = new VPanel(new HPanel(fullResolutionButton,GUtil.HGlue()),new HPanel(halfResolutionButton,GUtil.HGlue()),new HPanel(quarterResolutionButton,GUtil.HGlue()),new HPanel(customResolutionButton,GUtil.HSpace(5),heightField,new JLabel(" x "),widthField,GUtil.HSpace(10),maintainCheckBox,GUtil.HGlue()),GUtil.VGlue());
    sizePanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Size "));
    // Quality...
    highQualityButton = new JRadioButton("  High (" + Integer.toString(HIGH_QUALITY) + "%)",true);
    highQualityButton.setToolTipText("High quality (less compression).");
    mediumQualityButton = new JRadioButton("  Medium (" + Integer.toString(MEDIUM_QUALITY) + "%)",false);
    mediumQualityButton.setToolTipText("Medium quality (moderate compression).");
    lowQualityButton = new JRadioButton("  Low (" + Integer.toString(LOW_QUALITY) + "%)",false);
    lowQualityButton.setToolTipText("Low quality (more compression).");
    customQualityButton = new JRadioButton("",false);
    customQualityButton.setToolTipText("Custom quality.");
    ButtonGroup qualityGroup = new ButtonGroup();
    qualityGroup.add(highQualityButton);
    qualityGroup.add(mediumQualityButton);
    qualityGroup.add(lowQualityButton);
    qualityGroup.add(customQualityButton);
    qualityField = new JTextField(5);
    qualityField.setToolTipText("Quality percentage (lower values produce smaller image files but more compression artifacts.");
    qualityField.addKeyListener(new QualityFieldChanged());
    qualityField.setMaximumSize(qualityField.getPreferredSize());
    VPanel qualityPanel = new VPanel(new HPanel(highQualityButton,GUtil.HGlue()),new HPanel(mediumQualityButton,GUtil.HGlue()),new HPanel(lowQualityButton,GUtil.HGlue()),new HPanel(customQualityButton,GUtil.HSpace(5),qualityField,new JLabel(" % "),GUtil.HGlue()),GUtil.VGlue());
    qualityPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Quality "));
    // Format...
    JRadioButton jpegButton = new JRadioButton("JPEG",true);
    jpegButton.setToolTipText("JPEG format (currently the only format supported).");
    ButtonGroup formatGroup = new ButtonGroup();
    formatGroup.add(jpegButton);
    VPanel formatPanel = new VPanel(new HPanel(jpegButton,GUtil.HGlue()),GUtil.VGlue());
    formatPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Format "));
    // Path...
    String path = passName + ".jpg";
    if (lastBrowsedDir != null) {
      path = lastBrowsedDir + File.separator + path;
    } else {
      File file = new File(path);
      path = file.getAbsolutePath();
    }
    pathField = new JTextField(path,50);
    pathField.setToolTipText("Location and name of the image file to create.");
    pathField.setMaximumSize(new Dimension(pathField.getMaximumSize().width,pathField.getPreferredSize().height));
    JButton browseButton = new JButton("Browse");
    browseButton.setToolTipText("Displays a file chooser dialog.");
    browseButton.addActionListener(new BrowseButtonClicked());
    browseButton.setMargin(new Insets(0,5,0,5));
    HPanel pathPanel = new HPanel(pathField,GUtil.HSpace(10),browseButton);
    pathPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Save As "));
    VPanel previewPanel = new VPanel(new MarginPanel(5,5,new ImagePanel(bi.getScaledInstance(100,-1,Image.SCALE_SMOOTH)),5,5),GUtil.VGlue());
    previewPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Preview "));
    mainPanel.add(new VPanel(new HPanel(new VPanel(sizePanel,qualityPanel,formatPanel),previewPanel),pathPanel),BorderLayout.CENTER);
    statusLabel = new JLabel(" ");
    JButton saveButton = new JButton("Create");
    saveButton.setToolTipText("Creates the image file.");
    saveButton.addActionListener(new SaveButtonClicked());
    JButton cancelButton = new JButton("Cancel");
    cancelButton.setToolTipText("Cancels without creating an image file.");
    cancelButton.addActionListener(new CancelButtonClicked());
    mainPanel.add(new MarginPanel(5,5,new HPanel(saveButton,GUtil.HGlue(),statusLabel,GUtil.HGlue(),cancelButton),5,5),BorderLayout.SOUTH);
    containerPanel.add(mainPanel,BorderLayout.CENTER);
  }
  /****************************************************************************
  * calcHeight.
  ****************************************************************************/
  private int calcHeight (int width) {
    return Math.round(((float) (width * fullHeight)) / fullWidth);
  }
  /****************************************************************************
  * calcWidth.
  ****************************************************************************/
  private int calcWidth (int height) {
    return Math.round(((float) (height * fullWidth)) / fullHeight);
  }
  /****************************************************************************
  * WidthFieldChanged.
  ****************************************************************************/
  private class WidthFieldChanged extends KeyAdapter {
    public void keyTyped (KeyEvent ke) {
      SwingUtilities.invokeLater(new WidthChanged());
    }
  }
  private class WidthChanged implements Runnable {
    public void run () {
      if (maintainCheckBox.isSelected()) {
      	try {
          int width = Integer.parseInt(widthField.getText());
          int height = calcHeight(width);
          heightField.setText(Integer.toString(height));
      	} catch (Exception e) {
      	  heightField.setText("");
      	}
      }
      customResolutionButton.setSelected(true);
    }
  }
  /****************************************************************************
  * HeightFieldChanged.
  ****************************************************************************/
  private class HeightFieldChanged extends KeyAdapter {
    public void keyTyped (KeyEvent ke) {
      SwingUtilities.invokeLater(new HeightChanged());
    }
  }
  private class HeightChanged implements Runnable {
    public void run () {
      if (maintainCheckBox.isSelected()) {
      	try {
          int height = Integer.parseInt(heightField.getText());
          int width = calcWidth(height);
          widthField.setText(Integer.toString(width));
      	} catch (Exception e) {
      	  widthField.setText("");
      	}
      }
      customResolutionButton.setSelected(true);
    }
  }
  /****************************************************************************
  * QualityFieldChanged.
  ****************************************************************************/
  private class QualityFieldChanged extends KeyAdapter {
    public void keyTyped (KeyEvent ke) {
      customQualityButton.setSelected(true);
    }
  }
  /****************************************************************************
  * BrowseButtonClicked.
  ****************************************************************************/
  private class BrowseButtonClicked implements ActionListener {
    JFileChooser chooser;
    public void actionPerformed (ActionEvent ae) {
      containerPanel.removeAll();
      chooser = new JFileChooser();
      File file = new File(pathField.getText());
      chooser.setCurrentDirectory(file.exists() ? file : new File(file.getParent()));
      chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
      chooser.setApproveButtonText("Select");
      chooser.setSelectedFile(file);
      chooser.addActionListener(new ChooserAction());
      containerPanel.add(chooser,BorderLayout.CENTER);
      containerPanel.revalidate();
      containerPanel.repaint();
    }
    private class ChooserAction implements ActionListener {
      public void actionPerformed (ActionEvent ae) {
        if (ae.getActionCommand().equals(JFileChooser.APPROVE_SELECTION)) {
	  File file = chooser.getSelectedFile();
	  pathField.setText(file.getAbsolutePath());
	  lastBrowsedDir = file.getParent();
        }
        if (ae.getActionCommand().equals(JFileChooser.CANCEL_SELECTION)) {
        }
        containerPanel.removeAll();
        containerPanel.add(mainPanel,BorderLayout.CENTER);
        containerPanel.revalidate();
        containerPanel.repaint();
      }
    }
  }
  /****************************************************************************
  * SaveButtonClicked.
  ****************************************************************************/
  private class SaveButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      Util.startThread(new DoSave());
    }
  }
  private class DoSave implements Runnable {
    private FileOutputStream out;
    public void run () {
      GUtil.invokeAndWait(new UpdateLabel(statusLabel,"Saving..."));
      try {
      	save();
        GUtil.invokeAndWait(new UpdateLabel(statusLabel,"Saving...done"));
        Util.sleep(1.0);
        GUtil.invokeAndWait(new UpdateLabel(statusLabel," "));
      } catch (Exception e) {
        GUtil.invokeAndWait(new UpdateLabel(statusLabel," "));
      	JOptionPane.showMessageDialog(thisDialog,e.toString(),"ERROR",
      				      JOptionPane.ERROR_MESSAGE);
	try {out.close();} catch (Exception ex) {}
      }
    }
    private void save () throws Exception {
      File file = new File(pathField.getText());
      JPEGEncodeParam param = new JPEGEncodeParam();
      if (highQualityButton.isSelected()) {
	param.setQuality((float)(HIGH_QUALITY / 100.0));
      }
      if (mediumQualityButton.isSelected()) {
	param.setQuality((float)(MEDIUM_QUALITY / 100.0));
      }
      if (lowQualityButton.isSelected()) {
	param.setQuality((float)(LOW_QUALITY / 100.0));
      }
      if (customQualityButton.isSelected()) {
	param.setQuality((float)(Integer.parseInt(qualityField.getText()) / 100.0));
      }
      out = new FileOutputStream(file);
      if (fullResolutionButton.isSelected()) {
        JAI.create("encode",bi,out,"JPEG",param);
      } else {
      	Image i = null;
        if (halfResolutionButton.isSelected()) {
          i = bi.getScaledInstance(halfWidth,halfHeight,Image.SCALE_SMOOTH);
        }
        if (quarterResolutionButton.isSelected()) {
          i = bi.getScaledInstance(quarterWidth,quarterHeight,Image.SCALE_SMOOTH);
        }
        if (customResolutionButton.isSelected()) {
          int customHeight = Integer.parseInt(heightField.getText());
          int customWidth = Integer.parseInt(widthField.getText());
          i = bi.getScaledInstance(customWidth,customHeight,Image.SCALE_SMOOTH);
        }
	BufferedImage biOut = new BufferedImage(i.getWidth(null),
						i.getHeight(null),
			    			bi.getType());
	Graphics g = biOut.getGraphics();
	g.drawImage(i,0,0,null);
	JAI.create("encode",biOut,out,"JPEG",param);
      }
      out.close();
    }
  }
  /****************************************************************************
  * CancelButtonClicked.
  ****************************************************************************/
  private class CancelButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      dispose();
    }
  }
  /****************************************************************************
  * ImagePanel.
  ****************************************************************************/
  private class ImagePanel extends JPanel {
    private Image image;
    private static final long serialVersionUID = 1L;
    public ImagePanel (Image image) {
      this.image = image;
      Dimension dims = new Dimension(image.getWidth(null),image.getHeight(null));
      setMinimumSize(dims);
      setPreferredSize(dims);
      setMaximumSize(dims);
    }
    public void paint (Graphics g) {
      g.drawImage(image,0,0,null);
    }
  }
}
