/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CONSOLE
*
*  History:
*
*   9-Dec-04, J Love, GST	Original version.
*  10-Feb-06, J Love, GST	Clean up, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.console;
import gov.nasa.gsfc.aisb.simulcast.util.*;

public class ClientHandler extends HPanel {
  private String id;
  private boolean downstreamServer;
  private ArrowPanel arrowPanel;
  private StringMap handlers = new StringMap();
  private VPanel handlersPanel;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ClientHandler.
  ****************************************************************************/
  public ClientHandler (XmlParser parser) throws Exception {
    id = parser.getAttributeValue("id");
    downstreamServer = parser.hasElement("downstreamServer");
    long connectedAt = Long.parseLong(parser.getElementValue("connectedAt"));
    String remoteHostPort = parser.getElementValue("remoteHostPort");
    String text = id + ", " + Util.encodeHostPort(remoteHostPort) +
		  ", at " + Util.encodeDateTime(connectedAt);
    handlersPanel = new VPanel();
    arrowPanel = new ArrowPanel(false,ArrowButton.DOWN,text,18,handlersPanel);
    add(new HPanel(arrowPanel,GUtil.HGlue()));
  }
  /****************************************************************************
  * getId.
  ****************************************************************************/
  public String getId () {
    return id;
  }
  /****************************************************************************
  * isDownstreamServer.
  ****************************************************************************/
  public boolean isDownstreamServer () {
    return downstreamServer;
  }
  /****************************************************************************
  * addPlayer.
  ****************************************************************************/
  public void addPlayer (PlayerHandler handler) {
    handlers.put(handler.getId(),handler);
    handlersPanel.add(handler);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * updatePlayer.
  ****************************************************************************/
  public void updatePlayer (XmlParser parser) throws Exception {
    String id = parser.getAttributeValue("id");
    PlayerHandler handler = (PlayerHandler) handlers.get(id);
    if (handler != null) {
      handler.update(parser);
    }
    revalidate();
    repaint();
  }
  /****************************************************************************
  * removePlayer.
  ****************************************************************************/
  public void removePlayer (String id) {
    PlayerHandler handler = (PlayerHandler) handlers.get(id);
    if (handler != null) {
      handlers.remove(id);
      handlersPanel.remove(handler);
    }
    revalidate();
    repaint();
  }
}
