/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CONSOLE
*
*  History:
*
*   9-Dec-04, J Love, GST	Original version.
*  10-Feb-06, J Love, GST	Clean up.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.console;
import gov.nasa.gsfc.aisb.simulcast.util.*;

public class ClientManager extends VPanel {
  private StringMap clientHandlers = new StringMap();
  private StringMap downstreamServerHandlers = new StringMap();
  private VPanel clientsPanel;
  private VPanel downstreamServersPanel;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ClientManager.
  ****************************************************************************/
  public ClientManager () {
    clientsPanel = new VPanel();
    downstreamServersPanel = new VPanel();
    add(new HPanel(new ArrowPanel(false,ArrowButton.DOWN,"Clients",18,clientsPanel),GUtil.HGlue()));
    add(new HPanel(new ArrowPanel(false,ArrowButton.DOWN,"Downstream Servers",18,downstreamServersPanel),GUtil.HGlue()));
  }
  /****************************************************************************
  * clientListing.
  ****************************************************************************/
  public void clientListing (XmlParser[] parsers) throws Exception {
    for (int i = 0; i < parsers.length; i++) {
       ClientHandler handler = new ClientHandler(parsers[i]);
       if (handler.isDownstreamServer()) {
       	 downstreamServerHandlers.put(handler.getId(),handler);
       	 downstreamServersPanel.add(handler);
       } else {
	 clientHandlers.put(handler.getId(),handler);
	 clientsPanel.add(handler);
       }
    }
    revalidate();
    repaint();
  }
  /****************************************************************************
  * clientAdded.
  ****************************************************************************/
  public void clientAdded (XmlParser parser) throws Exception {
    ClientHandler handler = new ClientHandler(parser);
    if (handler.isDownstreamServer()) {
      downstreamServerHandlers.put(handler.getId(),handler);
      downstreamServersPanel.add(handler);
    } else {
      clientHandlers.put(handler.getId(),handler);
      clientsPanel.add(handler);
    }
    revalidate();
    repaint();
  }
  /****************************************************************************
  * clientRemoved.
  ****************************************************************************/
  public void clientRemoved (XmlParser parser) throws Exception {
    String id = parser.getAttributeValue("id");
    if (clientHandlers.contains(id)) {
      ClientHandler handler = (ClientHandler) clientHandlers.get(id);
      clientHandlers.remove(id);
      clientsPanel.remove(handler);
      revalidate();
      repaint();
      return;
    }
    if (downstreamServerHandlers.contains(id)) {
      ClientHandler handler = (ClientHandler) downstreamServerHandlers.get(id);
      downstreamServerHandlers.remove(id);
      downstreamServersPanel.remove(handler);
      revalidate();
      repaint();
      return;
    }
  }
  /****************************************************************************
  * playerListing.
  ****************************************************************************/
  public void playerListing (XmlParser[] parsers) throws Exception {
    for (int i = 0; i < parsers.length; i++) {
       PlayerHandler player = new PlayerHandler(parsers[i]);
       if (clientHandlers.contains(player.getClientId())) {
	 ClientHandler client = (ClientHandler) clientHandlers.get(player.getClientId());
	 client.addPlayer(player);
	 continue;
       }
       if (downstreamServerHandlers.contains(player.getClientId())) {
	 ClientHandler client = (ClientHandler) downstreamServerHandlers.get(player.getClientId());
	 client.addPlayer(player);
	 continue;
       }
    }
  }
  /****************************************************************************
  * playerAdded.
  ****************************************************************************/
  public void playerAdded (XmlParser parser) throws Exception {
    PlayerHandler player = new PlayerHandler(parser);
    if (clientHandlers.contains(player.getClientId())) {
      ClientHandler client = (ClientHandler) clientHandlers.get(player.getClientId());
      client.addPlayer(player);
      return;
    }
    if (downstreamServerHandlers.contains(player.getClientId())) {
      ClientHandler client = (ClientHandler) downstreamServerHandlers.get(player.getClientId());
      client.addPlayer(player);
      return;
    }
  }
  /****************************************************************************
  * playerUpdate.
  ****************************************************************************/
  public void playerUpdate (XmlParser parser) throws Exception {
    String clientId = parser.getElementValue("clientId");
    if (clientHandlers.contains(clientId)) {
      ClientHandler client = (ClientHandler) clientHandlers.get(clientId);
      client.updatePlayer(parser);
      return;
    }
    if (downstreamServerHandlers.contains(clientId)) {
      ClientHandler client = (ClientHandler) downstreamServerHandlers.get(clientId);
      client.updatePlayer(parser);
      return;
    }
  }
  /****************************************************************************
  * playerRemoved.
  ****************************************************************************/
  public void playerRemoved (XmlParser parser) throws Exception {
    String playerId = parser.getAttributeValue("id");
    String clientId = parser.getElementValue("clientId");
    if (clientHandlers.contains(clientId)) {
      ClientHandler client = (ClientHandler) clientHandlers.get(clientId);
      client.removePlayer(playerId);
      return;
    }
    if (downstreamServerHandlers.contains(clientId)) {
      ClientHandler client = (ClientHandler) downstreamServerHandlers.get(clientId);
      client.removePlayer(playerId);
      return;
    }
  }
}
