/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CONSOLE
*
*  History:
*
*  12-Nov-04, J Love, GST	Original version.
*  10-Feb-06, J Love, GST	Clean up.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.console;
import gov.nasa.gsfc.aisb.simulcast.util.*;

public class ConsoleManager extends HPanel {
  private ArrowPanel arrowPanel;
  private StringMap handlers = new StringMap();
  private VPanel handlersPanel;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ConsoleManager.
  ****************************************************************************/
  public ConsoleManager () {
    handlersPanel = new VPanel();
    arrowPanel = new ArrowPanel(false,ArrowButton.DOWN,"Consoles",18,handlersPanel);
    add(new HPanel(arrowPanel,GUtil.HGlue()));
  }
  /****************************************************************************
  * consoleListing.
  ****************************************************************************/
  public void consoleListing (XmlParser[] parsers) throws Exception {
    for (int i = 0; i < parsers.length; i++) {
       ConsoleHandler handler = new ConsoleHandler(parsers[i]);
       handlers.put(handler.getId(),handler);
       handlersPanel.add(handler);
    }
    revalidate();
    repaint();
  }
  /****************************************************************************
  * consoleAdded.
  ****************************************************************************/
  public void consoleAdded (XmlParser parser) throws Exception {
    ConsoleHandler handler = new ConsoleHandler(parser);
    handlers.put(handler.getId(),handler);
    handlersPanel.add(handler);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * consoleRemoved.
  ****************************************************************************/
  public void consoleRemoved (XmlParser parser) throws Exception {
    String id = parser.getAttributeValue("id");
    ConsoleHandler handler = (ConsoleHandler) handlers.get(id);
    if (handler != null) {
      handlers.remove(id);
      handlersPanel.remove(handler);
    }
    revalidate();
    repaint();
  }
}
