/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CONSOLE
*
*  History:
*
*   2-Jan-07, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.console;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class PassArchiveHandler extends VPanel {
  private ServerHandler serverHandler;
  private String name;
  private ArrowPanel arrowPanel;
  private CustomLabel startedField;
  private CustomLabel endedField;
  private CustomLabel errorField;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PassArchiveHandler.
  ****************************************************************************/
  public PassArchiveHandler (ServerHandler serverHandler, XmlParser parser) throws Exception {
    this.serverHandler = serverHandler;
    name = parser.getElementValue("name");
    startedField = new CustomLabel("");
    endedField = new CustomLabel("");
    errorField = new CustomLabel("");
    JButton deleteButton = new JButton("Delete");
    deleteButton.addActionListener(new DeleteButtonClicked());
    VPanel v1 = new VPanel(new HPanel(new CustomLabel("Started:"),GUtil.HSpace(5),startedField,GUtil.HGlue()),
			   new HPanel(new CustomLabel("Ended:"),GUtil.HSpace(5),endedField,GUtil.HGlue()),
			   new HPanel(errorField,GUtil.HGlue()),
		       	   new HPanel(deleteButton,GUtil.HGlue()));
    arrowPanel = new ArrowPanel(false,ArrowButton.RIGHT,name,18,v1); 
    update(parser);
    add(arrowPanel);
  }
  /****************************************************************************
  * getName.
  ****************************************************************************/
  public String getName () {
    return name;
  }
  /****************************************************************************
  * update.
  ****************************************************************************/
  public void update (XmlParser parser) throws Exception {
    if (parser.hasElement("startedAt")) {
      startedField.update(Util.encodeDateTime(Long.parseLong(parser.getElementValue("startedAt"))));
    }
    if (parser.hasElement("endedAt")) {
      endedField.update(Util.encodeDateTime(Long.parseLong(parser.getElementValue("endedAt"))));
    }
    if (parser.hasElement("error")) {
      /*
      arrowPanel.setColor(Color.RED);
      errorField.update("ERROR: "+parser.getElementValue("error"),Color.RED);
      */
      if (parser.getElementValue("error").equals("no scans"))
      {
        arrowPanel.setColor(Color.BLACK);
        errorField.update("ERROR: "+"No scans were acquired during this pass.",Color.BLACK);
      }
      else if (parser.getElementValue("error").startsWith("timeout while spooling pass"))
      {
        arrowPanel.setColor(Color.BLACK);
        errorField.update("ERROR: "+"Network or upstream server problem occured during acquisition of this pass.",Color.BLACK);
        
      }
      else if (parser.getElementValue("error").startsWith("java.net.SocketException"))
      {
        arrowPanel.setColor(Color.BLACK);
        errorField.update("ERROR: "+"Network or upstream server problem occured during acquisition of this pass.",Color.BLACK);
        
      }
      else if (parser.getElementValue("error").startsWith("java.io.EOFException"))
      {
        arrowPanel.setColor(Color.BLACK);
        errorField.update("ERROR: "+"Data or Network problem occured during acquisition of this pass.",Color.BLACK);
        
      }
      else
      {
        arrowPanel.setColor(Color.BLACK);
        errorField.update("ERROR: "+parser.getElementValue("error"),Color.BLACK);
      }

    }
  }
  /****************************************************************************
  * DeleteButtonClicked.
  ****************************************************************************/
  private class DeleteButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
	  		 "<deletePass>" + "<name>" + name + "</name>" +
	  		 "</deletePass>" + "</simulcastMessage>");
    }
  }
}
