/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CONSOLE
*
*  History:
*
*   2-Jan-07, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.console;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.event.*;
import javax.swing.*;

public class PassArchiveManager extends HPanel {
  private ServerHandler serverHandler;
  private ArrowPanel arrowPanel;
  private StringMap handlers = new StringMap();
  private VPanel handlersPanel;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PassArchiveManager.
  ****************************************************************************/
  public PassArchiveManager (ServerHandler serverHandler) {
    this.serverHandler = serverHandler;
    handlersPanel = new VPanel();
    JButton purgeButton = new JButton("Purge Passes");
    purgeButton.addActionListener(new PurgeButtonClicked());
    arrowPanel = new ArrowPanel(false,ArrowButton.DOWN,"Pass Archive",18,new VPanel(handlersPanel,GUtil.VSpace(5),purgeButton));
    add(new HPanel(arrowPanel,GUtil.HGlue()));
  }
  /****************************************************************************
  * passListing.
  ****************************************************************************/
  public void passListing (XmlParser[] parsers) throws Exception {
    handlers.clear();
    handlersPanel.removeAll();
    for (XmlParser parser : parsers) {
       PassArchiveHandler handler = new PassArchiveHandler(serverHandler,parser);
       handlers.put(handler.getName(),handler);
       handlersPanel.add(handler);
    }
    revalidate();
    repaint();
  }
  /****************************************************************************
  * passDetails.
  ****************************************************************************/
  public void passDetails (XmlParser parser) throws Exception {
    PassArchiveHandler handler = (PassArchiveHandler) handlers.get(parser.getElementValue("name"));
    handler.update(parser);
  }
  /****************************************************************************
  * passStarted.
  ****************************************************************************/
  public void passStarted (XmlParser parser) throws Exception {
    PassArchiveHandler handler = new PassArchiveHandler(serverHandler,parser);
    handlers.put(handler.getName(),handler);
    handlersPanel.add(handler);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * passUpdated.
  ****************************************************************************/
  public void passUpdated (XmlParser parser) throws Exception {
    PassArchiveHandler handler = (PassArchiveHandler) handlers.get(parser.getElementValue("name"));
    handler.update(parser);
  }
  /****************************************************************************
  * passEnded.
  ****************************************************************************/
  public void passEnded (XmlParser parser) throws Exception {
    PassArchiveHandler handler = (PassArchiveHandler) handlers.get(parser.getElementValue("name"));
    handler.update(parser);
  }
  /****************************************************************************
  * passRemoved.
  ****************************************************************************/
  public void passRemoved (XmlParser parser) throws Exception {
    PassArchiveHandler handler = (PassArchiveHandler) handlers.remove(parser.getElementValue("name"));
    handlersPanel.remove(handler);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * PurgeButtonClicked.
  ****************************************************************************/
  private class PurgeButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
	  		 "<purgePasses/>" + "</simulcastMessage>");
    }
  }
}
