/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CONSOLE
*
*  History:
*
*   6-Dec-04, J Love, GST	Original version.
*  10-Feb-06, J Love, GST	Clean up.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.console;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;

public class PassHistoryHandler extends VPanel {
  private String name;
  private ArrowPanel arrowPanel;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PassHandler.
  ****************************************************************************/
  public PassHistoryHandler (XmlParser parser) throws Exception {
    name = parser.getAttributeValue("name");
    PassParameters passParameters = (parser.hasElement("stationPassParameters") ?
    				     (PassParameters) new StationPassParameters(parser.getElement("stationPassParameters")) :
    				     (PassParameters) new UpstreamPassParameters(parser.getElement("upstreamPassParameters")));
    String[] lines = passParameters.toLines();
    VPanel panel = new VPanel();
    for (int i = 1; i < lines.length; i++) {
       panel.add(new HPanel(new JLabel(lines[i]),GUtil.HGlue()));
    }
    String text = lines[0] + ", " + Util.encodeDateTime(System.currentTimeMillis()) + ", " + name;
    arrowPanel = new ArrowPanel(false,ArrowButton.RIGHT,text,18,panel);
    add(arrowPanel);
  }
  /****************************************************************************
  * getName.
  ****************************************************************************/
  public String getName () {
    return name;
  }
  /****************************************************************************
  * endOfPass.
  ****************************************************************************/
  public void endOfPass () {
    String text = arrowPanel.getTitle();
    arrowPanel.setTitle(text + ", COMPLETED",Color.black);
  }
  /****************************************************************************
  * errorOnPass.
  ****************************************************************************/
  public void errorOnPass (String error) {
    String text = arrowPanel.getTitle();
    arrowPanel.setTitle(text + ", ERROR: " + error,Color.red);
  }
}
