/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CONSOLE
*
*  History:
*
*  19-Nov-04, J Love, GST	Original version.
*  10-Feb-06, J Love, GST	Clean up.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.console;
import gov.nasa.gsfc.aisb.simulcast.util.*;

public class PassHistoryManager extends HPanel {
  private ArrowPanel arrowPanel;
  private StringMap handlers = new StringMap();
  private VPanel handlersPanel;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PassManager.
  ****************************************************************************/
  public PassHistoryManager () {
    handlersPanel = new VPanel();
    arrowPanel = new ArrowPanel(false,ArrowButton.DOWN,"Pass History",18,handlersPanel);
    add(new HPanel(arrowPanel,GUtil.HGlue()));
  }
  /****************************************************************************
  * startOfPass.
  ****************************************************************************/
  public void startOfPass (XmlParser parser) throws Exception {
    PassHistoryHandler handler = new PassHistoryHandler(parser);
    handlers.put(handler.getName(),handler);
    handlersPanel.add(handler);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * endOfPass.
  ****************************************************************************/
  public void endOfPass (XmlParser parser) throws Exception {
    String passName = parser.getAttributeValue("name");
    PassHistoryHandler handler = (PassHistoryHandler) handlers.get(passName);
    if (handler != null) {
      handler.endOfPass();
      revalidate();
      repaint();
    }
  }
  /****************************************************************************
  * errorOnPass.
  ****************************************************************************/
  public void errorOnPass (XmlParser parser) throws Exception {
    String passName = parser.getAttributeValue("name");
    PassHistoryHandler handler = (PassHistoryHandler) handlers.get(passName);
    if (handler != null) {
      handler.errorOnPass(parser.getElementValue());
      revalidate();
      repaint();
    }
  }
}
