/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/IMAGES
*
*  History:
*
*   1-Oct-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*  28-Nov-05, J Love, GST	Better log messages.
*   4-Jan-07, J Love, GST	Log environment at startup.
*   1-Feb-07, J Love, GST	Use of sc.home property.  Daemon thread for
*   				Windows service.  Added "stop" for use by
*   				Advanced Installer service installation on
*   				Windows.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.images;
import gov.nasa.gsfc.aisb.simulcast.*;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.nisgs.nsls.*;
import java.io.*;

public class Images implements SimulcastService {
  protected static String configFile;
  protected static ServerHandler serverHandler;
  protected static HostPort serverHostPort;
  protected static String imageExistingPasses;
  protected static boolean imageNewPasses;
  protected static File handlerScriptFile;
  protected static Log log;
  private static boolean stop = false;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    if (args.length != 1) {
      System.out.println("Usage: Images <config-file>");
      return;
    }
    Images images = new Images();
    images.start(args);
    // On Windows, loop until "stop" is invoked by the executable that was
    // installed as a Windows service.  This should be the only non-daemon
    // thread running so when this thread ends the JVM should exit.
    // On Linux, not used by the JSW but doesn't hurt too bad.
    while (!stop) {
      Util.sleep(250);
    }
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start (String[] args) throws Exception {
    configFile = args[0];
    XmlParser parser = new XmlParser(Util.getFile(configFile),"imagesConfig");
    log = Util.initLog(parser.getElement("log"));
    log.setDefaultSource(new NSLS.SC("Images"));
    log.info("Images: STARTING"+", "+Util.getEnv(),"v");
    imageExistingPasses = parser.getElementValue("imageExistingPasses");
    imageNewPasses = parser.getElementValue("imageNewPasses").equals("yes");
    serverHostPort = new HostPort(parser.getElementValue("serverHostPort"));
    handlerScriptFile = Util.getFile(parser.getElementValue("handlerScript"));
    emptyScratchDir(Util.getFile(parser.getElementValue("scratchDirPath")));
    serverHandler = new ServerHandler();
    Thread thread = Util.startThread(serverHandler,true);
    log.info("Images: created/started a ServerHandler, " + thread.getName(),"v");
  }
  /****************************************************************************
  * emptyScratchDir.
  ****************************************************************************/
  private void emptyScratchDir (File scratchDir) {
    // File dir = new File(scratchDir);
    File[] files = scratchDir.listFiles();
    for (int i = 0; i < files.length; i++) {
       files[i].delete();
    }
  }
  /****************************************************************************
  * stop.
  * On Windows, invoked when the service is told to stop.
  ****************************************************************************/
  public static void stop () {
    stop = true;
  }
}
