/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*   8-Feb-06, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.images.handler;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.drl.geo.*;
import java.awt.*;
import java.awt.image.*;

public class MapHandler {
  private int width;
  private int height;
  private ShpOverlay features = null;
  private MapImage image = null;
  private MapParms mapParms = null;
  // private GroundStation[] groundStations = null;
  private static final String[] shpFiles = new String[] {
    "shapes/states/s_12my03.shp",
    "shapes/canada/PROVINCE.SHP",
    "shapes/world-esri/world.shp"
  };
  private static final Color[] shpColors = new Color[] {
    new Color(0,100,0),
    new Color(0,100,0),
    new Color(0,120,0)
  };
  /****************************************************************************
  * MapHandler.
  ****************************************************************************/
  public MapHandler (int width, int height) {
    this.width = width;
    this.height = height;
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    mapParms = new MercatorMapParms(width,height);
    Geo.Extents extents = headerBlock.getEstExtents();
    mapParms.zoom(extents.getBeginRight().getLat(),extents.getBeginRight().getLon(),
      		  extents.getBeginLeft().getLat(),extents.getBeginLeft().getLon(),
		  extents.getEndRight().getLat(),extents.getEndRight().getLon(),
		  extents.getEndLeft().getLat(),extents.getEndLeft().getLon());
    image = new MapImage(mapParms);
    features = new ShpOverlay(mapParms);
    for (int i = 0; i < shpFiles.length; i++) {
       features.addShpFile(shpFiles[i],shpColors[i]);
    }
  }
  /****************************************************************************
  * dataBlock.
  ****************************************************************************/
  public void dataBlock (int nGeoDataRows, int nGeoDataCols, int geoBinSize,
  			 double[][] geoLats, double[][] geoLons, Color[][] colors) {
    image.dataBlock(nGeoDataRows,nGeoDataCols,geoBinSize,geoLats,geoLons,colors);
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    BufferedImage bi = new BufferedImage(mapParms.getWidth(),
            				 mapParms.getHeight(),
            				 BufferedImage.TYPE_INT_RGB);
    Graphics g = bi.getGraphics();
    paint(g);
    return bi;
  }
  /****************************************************************************
  * paint.
  ****************************************************************************/
  public void paint (Graphics g) {
    g.setColor(Color.black);
    g.fillRect(0,0,mapParms.getWidth(),mapParms.getHeight());
    mapParms.drawImage(g,image);
    mapParms.drawImage(g,features);
    mapParms.drawGrid(g);
    mapParms.drawBorder(g);
  }
}
