/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/PROCESSOR
* 
*  History:
* 
*   30-Jun-04, J Love, GST	Original version.
*    8-Feb-06, J Love, GST	Java 1.5.
*   19-Jan-07, J Love, GST	Use of sc.home property.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.processor;
import java.io.*;
import java.util.zip.*;

public class LUTs {
  private float[][][][] m1LUT;		// [BAND][FOV.DETECTOR][SAMPLE][SIDE]
  private float[][][][] K_instLUT;	// [BAND][FOV.DETECTOR][SAMPLE][SIDE]
  private float[][][][] RVSrefLUT;	// [BAND][FOV.DETECTOR][SIDE][FRAME]
  private float[][][][] RVStebLUT;	// [BAND][FOV.DETECTOR][SIDE][FRAME]
  /****************************************************************************
  * LUTs.
  ****************************************************************************/
  public LUTs (File binFile) throws Exception {
    DataInputStream in = null;
    try {
      in = new DataInputStream(new BufferedInputStream(new GZIPInputStream(new FileInputStream(binFile))));
      load(in);
      in.close();
    } finally {
      try {in.close();} catch (Exception ex) {}
    }
  }
  /****************************************************************************
  * load.
  ****************************************************************************/
  private void load (DataInputStream in) throws Exception {
    // m1.
    m1LUT = new float[22][][][];
    for (int band = 0; band <= 1; band++) {
       m1LUT[band] = new float[40][][];
       for (int detector = 0; detector < 40; detector++) {
       	  m1LUT[band][detector] = new float[4][2];
	  for (int sample = 0; sample < 4; sample++) {
	     for (int side = 0; side < 2; side++) {
	      	m1LUT[band][detector][sample][side] = in.readFloat();
	     }
	  }
       }
    }
    for (int band = 2; band <= 6; band++) {
       m1LUT[band] = new float[20][][];
       for (int detector = 0; detector < 20; detector++) {
       	  m1LUT[band][detector] = new float[2][2];
	  for (int sample = 0; sample < 2; sample++) {
	     for (int side = 0; side < 2; side++) {
	      	m1LUT[band][detector][sample][side] = in.readFloat();
	     }
	  }
       }
    }
    for (int band = 7; band <= 21; band++) {
       m1LUT[band] = new float[10][][];
       for (int detector = 0; detector < 10; detector++) {
       	  m1LUT[band][detector] = new float[1][2];
	  for (int sample = 0; sample < 1; sample++) {
	     for (int side = 0; side < 2; side++) {
	      	m1LUT[band][detector][sample][side] = in.readFloat();
	     }
	  }
       }
    }
    // K_inst.
    K_instLUT = new float[22][][][];
    for (int band = 0; band <= 1; band++) {
       K_instLUT[band] = new float[40][][];
       for (int detector = 0; detector < 40; detector++) {
          K_instLUT[band][detector] = new float[4][2];
	  for (int sample = 0; sample < 4; sample++) {
	     for (int side = 0; side < 2; side++) {
	      	K_instLUT[band][detector][sample][side] = in.readFloat();
	     }
	  }
       }
    }
    for (int band = 2; band <= 6; band++) {
       K_instLUT[band] = new float[20][][];
       for (int detector = 0; detector < 20; detector++) {
          K_instLUT[band][detector] = new float[2][2];
	  for (int sample = 0; sample < 2; sample++) {
	     for (int side = 0; side < 2; side++) {
	      	K_instLUT[band][detector][sample][side] = in.readFloat();
	     }
	  }
       }
    }
    for (int band = 7; band <= 21; band++) {
       K_instLUT[band] = new float[10][][];
       for (int detector = 0; detector < 10; detector++) {
          K_instLUT[band][detector] = new float[1][2];
	  for (int sample = 0; sample < 1; sample++) {
	     for (int side = 0; side < 2; side++) {
	      	K_instLUT[band][detector][sample][side] = in.readFloat();
	     }
	  }
       }
    }
    // RVSrefLUT.
    RVSrefLUT = new float[22][][][];
    for (int band = 0; band <= 1; band++) {
       RVSrefLUT[band] = new float[40][2][1354];
       for (int detector = 0; detector < 40; detector++) {
	  for (int side = 0; side < 2; side++) {
	     for (int frame = 0; frame < 1354; frame++) {
	      	RVSrefLUT[band][detector][side][frame] = in.readFloat();
	     }
	  }
       }
    }
    for (int band = 2; band <= 6; band++) {
       RVSrefLUT[band] = new float[20][2][1354];
       for (int detector = 0; detector < 20; detector++) {
	  for (int side = 0; side < 2; side++) {
	     for (int frame = 0; frame < 1354; frame++) {
	      	RVSrefLUT[band][detector][side][frame] = in.readFloat();
	     }
	  }
       }
    }
    for (int band = 7; band <= 21; band++) {
       RVSrefLUT[band] = new float[10][2][1354];
       for (int detector = 0; detector < 10; detector++) {
	  for (int side = 0; side < 2; side++) {
	     for (int frame = 0; frame < 1354; frame++) {
	      	RVSrefLUT[band][detector][side][frame] = in.readFloat();
	     }
	  }
       }
    }
    // RVStebLUT.
    RVStebLUT = new float[16][][][];
    for (int band = 0; band <= 15; band++) {
       RVStebLUT[band] = new float[10][2][1354];
       for (int detector = 0; detector < 10; detector++) {
	  for (int side = 0; side < 2; side++) {
	     for (int frame = 0; frame < 1354; frame++) {
	      	RVStebLUT[band][detector][side][frame] = in.readFloat();
	     }
	  }
       }
    }
  }
  /****************************************************************************
  * getM1LUT.
  ****************************************************************************/
  public float[][][] getM1LUT (int bandIndex) {
    return m1LUT[bandIndex];
  }
  /****************************************************************************
  * getK_instLUT.
  ****************************************************************************/
  public float[][][] getK_instLUT (int bandIndex) {
    return K_instLUT[bandIndex];
  }
  /****************************************************************************
  * getRVSrefLUT.
  ****************************************************************************/
  public float[][][] getRVSrefLUT (int bandIndex) {
    return RVSrefLUT[bandIndex];
  }
  /****************************************************************************
  * getRVStebLUT.
  ****************************************************************************/
  public float[][][] getRVStebLUT (int bandIndex) {
    return RVSrefLUT[bandIndex];
  }
}
