/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*  18-Nov-05, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.drl.geo.*;

public class AreaFilter {
  private double north;
  private double south;
  private double east;
  private double west;
  /****************************************************************************
  * AreaFilter.
  ****************************************************************************/
  public AreaFilter (XmlParser parser) throws Exception {
    north = Double.parseDouble(parser.getElement("mercator").getElement("lat").getElementValue("north"));
    south = Double.parseDouble(parser.getElement("mercator").getElement("lat").getElementValue("south"));
    east = Double.parseDouble(parser.getElement("mercator").getElement("lon").getElementValue("east"));
    west = Double.parseDouble(parser.getElement("mercator").getElement("lon").getElementValue("west"));
  }
  /****************************************************************************
  * passes.
  ****************************************************************************/
  public boolean passes (Geo.Extents extents) {
    if (extents.getMinLat() > north) {
      return false;
    }
    if (extents.getMaxLat() < south) {
      return false;
    }
    if (extents.getMinLon() > east) {
      return false;
    }
    if (extents.getMaxLon() < west) {
      return false;
    }
    return true;
  }
  public boolean passes (GeoBlock geoBlock) {
    int nRows = geoBlock.getRowCount();
    int nCols = geoBlock.getColCount();
    double[][] lats = geoBlock.getLats();
    double[][] lons = geoBlock.getLons();
    for (int i = 0; i <= nCols; i++) {
       if (Util.inRange(south,lats[0][i],north) && Util.inRange(west,lons[0][i],east)) {
       	 return true;
       }
       if (Util.inRange(south,lats[nRows][i],north) && Util.inRange(west,lons[nRows][i],east)) {
       	 return true;
       }
    }
    return false;
  }
  /****************************************************************************
  * toString.
  ****************************************************************************/
  public String toString () {
    return "AreaFilter, north: " + north + ", south: " + south + ", east: " + east + ", west: " + west;
  }
}
