/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*   9-Dec-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*   8-Feb-06, J Love, GST	Better log messages, etc.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.util.*;
import java.net.*;
import java.io.*;

public class ClientManager {
  private LinkedList<ClientHandler> handlers = new LinkedList<ClientHandler>();
  private int nextClientId = 1;
  /****************************************************************************
  * ClientManager (constructor).
  ****************************************************************************/
  public ClientManager () {

  }
  /****************************************************************************
  * addClient.
  ****************************************************************************/
  public synchronized void addClient (Socket s, DataInputStream in,
  				      boolean downstreamServer, boolean loopMode, boolean imageMode) throws Exception {
    String clientId = "CLIENT." + Integer.toString(nextClientId++);
    ClientHandler handler = new ClientHandler(s,in,clientId,downstreamServer,loopMode,imageMode);
    handlers.add(handler);
    Thread thread = Util.startThread(handler,true);
    String text = "ClientManager: created/started a ClientHandler" +
    		  (downstreamServer ? " (downstream server)" : "") +
		  ", " + clientId + ", " + thread.getName();
    Server.log.info(text,"v");
    Server.consoleManager.broadcast(Util.XML_HEADER + "<simulcastMessage>" +
				    "<clientAdded>" + handler.toXml() +
				    "</clientAdded>" + "</simulcastMessage>");
  }
  /****************************************************************************
  * removeClient.
  ****************************************************************************/
  public synchronized void removeClient (ClientHandler handler) {
    handlers.remove(handler);
    Server.consoleManager.broadcast(Util.XML_HEADER + "<simulcastMessage>" +
				    "<clientRemoved>" + handler.toXml() +
				    "</clientRemoved>" + "</simulcastMessage>");
  }
  /****************************************************************************
  * broadcast.
  ****************************************************************************/
  public synchronized void broadcast (String msg) {
	for (ClientHandler handler : handlers) {
      handler.send(msg);
    }
  }
  /****************************************************************************
  * getClientListing.
  ****************************************************************************/
  public synchronized String getClientListing () {
    StringBuffer xml = new StringBuffer();
    xml.append(Util.XML_HEADER);
    xml.append("<simulcastMessage>");
    xml.append("<clientListing>");
	for (ClientHandler handler : handlers) {
      xml.append(handler.toXml());
    }
    xml.append("</clientListing>");
    xml.append("</simulcastMessage>");
    return xml.toString();
  }
}
