/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*  16-Nov-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*   8-Feb-06, J Love, GST	Better log messages.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.util.*;
import java.net.*;
import java.io.*;

public class ConsoleManager {
  private LinkedList<ConsoleHandler> handlers = new LinkedList<ConsoleHandler>();
  private int nextConsoleId = 1;
  /****************************************************************************
  * ConsoleManager.
  ****************************************************************************/
  public ConsoleManager () {

  }
  /****************************************************************************
  * addConsole.
  ****************************************************************************/
  public synchronized void addConsole (Socket s, DataInputStream in) throws Exception {
    String consoleId = "CONSOLE." + Integer.toString(nextConsoleId++);
    ConsoleHandler handler = new ConsoleHandler(s,in,consoleId);
    handlers.add(handler);
    Thread thread = Util.startThread(handler,true);
    Server.log.info("ConsoleManager: created/started a ConsoleHandler, " + consoleId + ", " + thread.getName(),"v");
    Server.consoleManager.broadcast(Util.XML_HEADER + "<simulcastMessage>" +
				    "<consoleAdded>" + handler.toXml() +
				    "</consoleAdded>" + "</simulcastMessage>");
  }
  /****************************************************************************
  * removeConsole.
  ****************************************************************************/
  public synchronized void removeConsole (ConsoleHandler handler) {
    handlers.remove(handler);
    Server.consoleManager.broadcast(Util.XML_HEADER + "<simulcastMessage>" +
				    "<consoleRemoved>" + handler.toXml() +
				    "</consoleRemoved>" + "</simulcastMessage>");
  }
  /****************************************************************************
  * broadcast.
  ****************************************************************************/
  public synchronized void broadcast (String msg) {
	for (ConsoleHandler handler : handlers) {
      handler.send(msg);
    }
  }
  /****************************************************************************
  * getConsoleListing.
  ****************************************************************************/
  public synchronized String getConsoleListing () {
    StringBuffer xml = new StringBuffer();
    xml.append(Util.XML_HEADER);
    xml.append("<simulcastMessage>");
    xml.append("<consoleListing>");
	for (ConsoleHandler handler : handlers) {
      xml.append(handler.toXml());
    }
    xml.append("</consoleListing>");
    xml.append("</simulcastMessage>");
    return xml.toString();
  }
}
