/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*   9-Dec-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*  19-Dec-05, J Love, GST	Better log messages.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service.
*  15-Mar-07, Z Yin,  GST	Display image resolution choices equal or larger 
*                               then data resolution.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.net.*;
import java.io.*;

public class ListenForClients implements Runnable {
  private int clientPort;
  private ServerSocket ss = null;
  /****************************************************************************
  * ListenForClients.
  ****************************************************************************/
  public ListenForClients (int clientPort) {
    this.clientPort = clientPort;
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    try {
      listen();
    } catch (Exception e) {
      Server.log.warning("Error when server listens for client","n");
      Server.log.warning("ListenForClients: listening for client",e,"v");
    } finally {
      try {ss.close();} catch (Exception e) {}
    }
  }
  /****************************************************************************
  * listen.
  ****************************************************************************/
  private void listen () throws Exception {
    ss = new ServerSocket(clientPort);
    Server.log.info("ListenForClients: listening on port " + Integer.toString(ss.getLocalPort()),"v");
    while (true) {
      Socket s = ss.accept();
      Server.log.info("ListenForClients: " + " " + Util.getAcceptedLogEntry(s),"v");
      Thread thread = Util.startThread(new ModeReader(s),true);
      Server.log.info("ListenForClients: created/started a ModeReader, " + thread.getName(),"v");
    }
  }
  /****************************************************************************
  * ModeReader.
  ****************************************************************************/
  private static class ModeReader implements Runnable {
    private Socket s;
    private DataInputStream in = null;
    public ModeReader (Socket s) {
      this.s = s;
    }
    public void run () {
      try {
	in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
	String msg = in.readUTF();

	XmlParser parser = new XmlParser(msg,"simulcastMessage");
	if (parser.hasElement("consoleMode")) {
	  Server.log.info("ListenForClients.ModeReader: console mode requested","v");
	  Server.consoleManager.addConsole(s,in);
	  return;
	}
	if (parser.hasElement("clientMode")) {
	  boolean downstreamServer = parser.getElement("clientMode").hasElement("downstreamServer");
	  boolean loopMode = parser.hasElement("loopMode");
	  boolean imageMode = parser.hasElement("imageMode");
	  Server.log.info("ListenForClients.ModeReader: client mode requested" + (downstreamServer ? " (downstream server)" : ""),"v");
	  Server.clientManager.addClient(s,in,downstreamServer,loopMode,imageMode);
	  return;
	}
	if (parser.hasElement("playMode")) {
          boolean imagePlayMode = parser.hasElement("imagePlayMode");
	  String clientId = parser.getElement("playMode").getAttributeValue("clientId");
	  Server.log.info("ListenForClients.ModeReader: play mode requested, clientId: " + clientId,"v");
	  Server.playerManager.addPlayer(s,in,clientId,imagePlayMode);
	  return;
	}
	throw new Exception("unknown mode");
      } catch (Exception e) {
        Server.log.warning("Error when listen to clint mode.","n");
        Server.log.warning("ListenForClients.ModeReader: handling mode",e,"v");
      	try {in.close();} catch (Exception ex) {}
      	try {s.close();} catch (Exception ex) {}
      }
    }
  }
}
