/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*  30-Jan-07, J Love, GST	Original version.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.io.*;
import java.net.*;

public class LoopbackTest implements Runnable {
  private HostPort address;
  private long interval;
  private long timeout;
  private int maxFailures;
  private String onMaxFailures;
  private int failureCount = 0;
  /****************************************************************************
  * LoopbackTest.
  ****************************************************************************/
  public LoopbackTest (XmlParser parser) throws Exception {
    address = new HostPort(parser.getElementValue("hostPort"));
    interval = Util.decodeTimeInterval(parser.getElementValue("interval"));
    timeout = Util.decodeTimeInterval(parser.getElementValue("timeout"));
    maxFailures = Util.max(1,Integer.parseInt(parser.getElementValue("maxFailures")));
    onMaxFailures = parser.getElementValue("onMaxFailures");
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    Util.sleep(interval);	// Give ListenForClients a chance to start.
    while (true) {
      ObjectQueue q = new ObjectQueue();
      Server.log.info("LoopbackTest.run: starting test","v");
      Util.startThread(new Test(q),true);
      Object o = q.get();
      if (o == null) {
	Server.log.info("LoopbackTest.run: success","v");
	failureCount = 0;
      } else {
        if (o instanceof Exception) {
          Exception e = (Exception) o;
	  Server.log.warning("Not able to connect Simulcast server.","n");
	  Server.log.warning("LoopbackTest.run/Exception: " + e.toString(),"v");
	  if (++failureCount == maxFailures) {
	    if (onMaxFailures.equals("halt")) {
	      Server.log.warning("Simulcast server cannot be connected and start another JVM.", "n");
	      Server.log.warning("LoopbackTest.run: max failures reached...halt","v");
	      Runtime.getRuntime().halt(1);
	    }
	    if (onMaxFailures.equals("exit")) {
	      Server.log.warning("Simulcast server cannot be connected and exit.", "n");
	      Server.log.warning("LoopbackTest.run: max failures reached...exit","v");
	      System.exit(1);
	    }
	    Server.log.warning("Simulcast server cannot be connected and try again", "n");
	    Server.log.warning("LoopbackTest.run: max failures reached...continue", "v");
	    failureCount = 0;
	  }
        }
      }
      Util.sleep(interval);
    }
  }
  /****************************************************************************
  * Test.
  ****************************************************************************/
  private class Test implements Runnable, TimeoutMonitorListener {
    private ObjectQueue q;
    private Socket s = null;
    private DataInputStream in = null;
    private DataOutputStream out = null;
    public Test (ObjectQueue q) {
      this.q = q;
    }
    public void run () {
      try {
	test();
        q.put(null);
      } catch (Exception e) {
	q.put(e);
      } finally {
        try {in.close();} catch (Exception e) {}
        try {out.close();} catch (Exception e) {}
        try {s.close();} catch (Exception e) {}
      }
    }
    private void test () throws Exception {
      TimeoutMonitor monitor = new TimeoutMonitor(timeout,this);
      monitor.start();
      s = new Socket();
      s.setSoTimeout((int)timeout);
      s.connect(new InetSocketAddress(address.getHost(),address.getPort()),(int)timeout);
      in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
      out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
      out.writeUTF(Util.XML_HEADER + "<simulcastMessage>" + "<clientMode/>"+ "<loopMode/>" + "</simulcastMessage>");
      out.flush();
      while (true) {
        String msg = in.readUTF();
	XmlParser parser = new XmlParser(msg,"simulcastMessage");
	if (parser.hasElement("passListing")) {
          break;
        }
      }
      monitor.stop();
      in.close();
      out.close();
      s.close();
    }
    public void timedOut () {
      q.put(new Exception("timed out"));
    }
  }
}
