/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*   8-Feb-06, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;

class OutputGeo {
  private int iBlockRowCount;
  // private int iBlockColCount;
  private int resolution;
  private int oBlockRowCount;
  private int oBlockColCount;
  private int jump;
  // private int iColStart;
  private int oRow = 0;
  /****************************************************************************
  * OutputGeo.
  ****************************************************************************/
  public OutputGeo (HeaderBlock inputHeader, HeaderBlock outputHeader/*,
		    double outputRelativeLeftEdgePct, double outputRelativeRightEdgePct*/) {
    iBlockRowCount = inputHeader.getGeoBlockRowCount();
    // iBlockColCount = inputHeader.getGeoBlockColCount();
    resolution = Util.max(outputHeader.getCoarsestResolution(),inputHeader.getGeoResolution());
    jump = resolution / inputHeader.getGeoResolution();
    // double outputRelativeWidthPct = outputRelativeRightEdgePct - outputRelativeLeftEdgePct;
    oBlockRowCount = Util.min(outputHeader.getCoarsestBlockRowCount(),
            		      inputHeader.getGeoBlockRowCount());
    oBlockColCount = Util.min(outputHeader.getCoarsestBlockColCount(),
           		      (int)/*Math.round*/(/*outputRelativeWidthPct* */ inputHeader.getGeoBlockColCount()));
    // iColStart = (int) Math.round(outputRelativeLeftEdgePct * inputHeader.getGeoBlockColCount());
    outputHeader.setGeoResolution(resolution);
    outputHeader.setGeoBlockRowCount(oBlockRowCount);
    outputHeader.setGeoBlockColCount(oBlockColCount);
  }
  /****************************************************************************
  * getBlockRowCount.
  ****************************************************************************/
  public int getBlockRowCount () {
    return oBlockRowCount;
  }
  /****************************************************************************
  * getBlockColCount.
  ****************************************************************************/
  public int getBlockColCount () {
    return oBlockColCount;
  }
  /****************************************************************************
  * loadGeoData.
  ****************************************************************************/
  public void loadGeoData (boolean firstBlock, boolean lastBlock,
			   GeoBlock inputGeoBlock, GeoBlock outputGeoBlock) {
    if (firstBlock) {
      oRow = 0;
    }
    if (oRow < oBlockRowCount) {
      for (int iRow = 0; iRow < iBlockRowCount; iRow += jump) {
      	 double[] oLats = outputGeoBlock.getLats()[oRow];
      	 double[] oLons = outputGeoBlock.getLons()[oRow];
      	 double[] iLats = inputGeoBlock.getLats()[iRow];
      	 double[] iLons = inputGeoBlock.getLons()[iRow];
      	 loadRow(oLats,oLons,iLats,iLons);
      	 oRow++;
      }
    }
    if (lastBlock) {
      int iRow = Util.min(oBlockRowCount*jump,iBlockRowCount);
      double[] oLats = outputGeoBlock.getLats()[oBlockRowCount];
      double[] oLons = outputGeoBlock.getLons()[oBlockRowCount];
      double[] iLats = inputGeoBlock.getLats()[iRow];
      double[] iLons = inputGeoBlock.getLons()[iRow];
      loadRow(oLats,oLons,iLats,iLons);
    }
  }
  private void loadRow (double[] oLats, double[] oLons,
			double[] iLats, double[] iLons) {
    for (int oCol = 0, iCol = /*iColStart*/0; oCol <= oBlockColCount; oCol++, iCol += jump) {
       oLats[oCol] = iLats[iCol];
       oLons[oCol] = iLons[iCol];
    }
  }
}
