/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*  23-Nov-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*   8-Feb-06, J Love, GST	Better log messages.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.util.*;
import java.net.*;
import java.io.*;

public class PlayerManager {
  private LinkedList<PassPlayer> players = new LinkedList<PassPlayer>();
  private int nextPlayerId = 1;
  /****************************************************************************
  * PlayerManager (constructor).
  ****************************************************************************/
  public PlayerManager () {

  }
  /****************************************************************************
  * addPlayer.
  ****************************************************************************/
  public synchronized void addPlayer (Socket s, DataInputStream in, String clientId, boolean imagePlayMode) throws Exception {
    String playerId = "PLAYER." + Integer.toString(nextPlayerId++);
    PassPlayer player = new PassPlayer(s,in,playerId,clientId,imagePlayMode);
    players.add(player);
    Server.consoleManager.broadcast(Util.XML_HEADER + "<simulcastMessage>" +
				    "<playerAdded>" + player.toXml() +
				    "</playerAdded>" + "</simulcastMessage>");
    Thread thread = Util.startThread(player,true);
    Server.log.info("PlayerManager: created/started a PassPlayer, " + playerId + ", " + thread.getName(),"v");
  }
  /****************************************************************************
  * removePlayer.
  ****************************************************************************/
  public synchronized void removePlayer (PassPlayer player) {
    players.remove(player);
    Server.consoleManager.broadcast(Util.XML_HEADER + "<simulcastMessage>" +
				    "<playerRemoved>" + player.toXml() +
				    "</playerRemoved>" + "</simulcastMessage>");
  }
  /****************************************************************************
  * getPlayerListing.
  ****************************************************************************/
  public synchronized String getPlayerListing () {
    StringBuffer xml = new StringBuffer();
    xml.append(Util.XML_HEADER);
    xml.append("<simulcastMessage>");
    xml.append("<playerListing>");
    for (PassPlayer player : players) {
      xml.append(player.toXml());
    }
    xml.append("</playerListing>");
    xml.append("</simulcastMessage>");
    return xml.toString();
  }
}
