/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*   6-Dec-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*  28-Nov-05, J Love, GST	Geolocation, etc.
*  10-Jan-07, J Love, GST	Added timeout for PassSpooler.  Added loopback
*  				test.
*   1-Feb-07, J Love, GST	Use of sc.home property.  Advanced Installer
*  				tweaking.  Daemon threads and timers for
*  				Windows service.  Added "stop" for use by
*  				Advanced Installer service installation on
*				Windows.
*  15-May-07, Z Yin, GST        Add no firewall client port for some client 
                                cannot access current port 3502.	
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.aisb.simulcast.*;
import gov.nasa.gsfc.nisgs.nsls.*;
import java.io.*;
import java.util.*;
import java.io.*;
import java.util.Properties; 
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;
import org.xml.sax.InputSource;
import org.w3c.dom.*;

public class Server implements SimulcastService {
  protected static Log log;
  protected static PassManager passManager;
  protected static ConsoleManager consoleManager;
  protected static ClientManager clientManager;
  protected static PlayerManager playerManager;
  protected static TLEManager tleManager;
  protected static GroundStations groundStations;
  protected static File spoolDir;
  protected static int maxSavedPasses;
  protected static long inputTimeOut;
  private static final long READY_INTERVAL_MS = Util.SERVER_READY_INTERVAL_SEC * 1000;
  private static boolean stop = false;
  static String Fix = "yes";
  public static int logDisplayTime = 1;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    if (args.length != 1) {
      System.out.println("Usage: Server <config-file>");
      return;
    }
    Server server = new Server();
    server.start(args);
    // On Windows, loop until "stop" is invoked by the executable that was
    // installed as a Windows service.  This should be the only non-daemon
    // thread running so when this thread ends the JVM should exit.
    // On Linux, not used by the JSW but doesn't hurt too bad.
    while (!stop) {
      Util.sleep(250);
    }
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start (String[] args) throws Exception {
    Properties props = new Properties () ; 
    String key;
    String Host, Port;

    Document doc;
    XPath xpath;
    NodeList nodes;
    Element e1, e2; 
    //String configFile = "./configs/server-config.xml";
    String configFile = args[0];
    File f = new File("../site.properties");
    if (f.exists())
    {
      try     
      {       
        props.load (new FileInputStream (new File ( "../site.properties"))); 
      }
      catch (IOException ie)
      {
        System.out.println ( "Error reading file" ) ;
      }
      key = "NSLS_SERVER_HOST";
      Host = props.getProperty (key) ;
      key = "NSLS_SERVER_PORT";
      Port = props.getProperty (key) ;

      // Read xml and build a DOM document.
      doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new InputSource(configFile));

      // Use XPath to find all nodes.
      xpath = XPathFactory.newInstance().newXPath();
      nodes = (NodeList)xpath.evaluate("//server", doc, XPathConstants.NODESET);
      if (nodes.getLength()<1)
      {
        // If no server, add node.
        e1 = (Element)doc.getElementsByTagName("log").item(0);
        e1.insertBefore(doc.createElement("server"), e1.getFirstChild());
        e2 = (Element)doc.getElementsByTagName("server").item(0);
        e2.insertBefore(doc.createElement("tmpDir"), e2.getFirstChild());
        e2.insertBefore(doc.createElement("hostPort"), e2.getFirstChild());
      }

      // Change value.
      nodes = (NodeList)xpath.evaluate("//server/hostPort", doc, XPathConstants.NODESET);
      for (int idx = 0; idx < nodes.getLength(); idx++)
      {
        nodes.item(idx).setTextContent(Host+":"+Port);
      }
      nodes = (NodeList)xpath.evaluate("//server/tmpDir", doc, XPathConstants.NODESET);
      for (int idx = 0; idx < nodes.getLength(); idx++)
      {
        if (configFile.equals("./configs/server-config.xml"))
        {
          nodes.item(idx).setTextContent("./logs/tmp/server");
        }
        else 
        {
          nodes.item(idx).setTextContent("./logs/tmp/downstream-server");
        }
      }

      // Write the DOM document to the file
      Transformer xformer = TransformerFactory.newInstance().newTransformer();
      xformer.transform(new DOMSource(doc), new StreamResult(new File(configFile)));
    }
    else
    {
      // Do nothing.
      // System.out.println ("No file ../site.properties" ) ; 
    }

    XmlParser parser = new XmlParser(Util.getFile(args[0]),"serverConfig");
    log = Util.initLog(parser.getElement("log"));
    log.setDefaultSource(new NSLS.SC("Server"));
    log.info("Simulcast Server is starting"+", "+Util.getEnv());
    if (parser.hasElement("fix")) 
    {
      Fix = parser.getElementValue("fix");
    }
    if (parser.hasElement("logDisplayTime")) 
    {
      logDisplayTime = Integer.valueOf(parser.getElementValue("logDisplayTime"));
    }
    spoolDir = Util.getFile(parser.getElementValue("spoolDirPath"));
    maxSavedPasses = Integer.parseInt(parser.getElementValue("maxSavedPasses"));
    inputTimeOut = Util.decodeTimeInterval(parser.getElementValue("inputTimeOut"));
    passManager = new PassManager();
    consoleManager = new ConsoleManager();
    clientManager = new ClientManager();
    playerManager = new PlayerManager();
    groundStations = new GroundStations(parser.getElement("groundStations"));
    tleManager = new TLEManager(parser.getElement("tle"));
    tleManager.start();
    if (parser.hasElement("clientPort")) {
      Util.sleep(2.0);
      Util.startThread(new ListenForClients(Integer.parseInt(parser.getElementValue("clientPort"))),true);
    }
    if (parser.hasElement("noFirewallClientPort")) {
      Util.sleep(2.0);
      Util.startThread(new ListenForClients(Integer.parseInt(parser.getElementValue("noFirewallClientPort"))),true);
    }
    if (parser.hasElement("inputPort")) {
      Util.sleep(2.0);
      Util.startThread(new ListenForProcessors(Integer.parseInt(parser.getElementValue("inputPort"))),true);
    }
    if (parser.hasElement("upstreamServers")) {
      parseUpstreamServers(parser.getElement("upstreamServers").getElements("upstreamServer"));
    }
    if (parser.hasElement("loopbackTest")) {
      Util.sleep(2.0);
      Util.startThread(new LoopbackTest(parser.getElement("loopbackTest")),true);
    }
    if (parser.hasElement("noFirewallLoopbackTest")) {
      Util.sleep(2.0);
      Util.startThread(new LoopbackTest(parser.getElement("noFirewallLoopbackTest")),true);
    }
    // Start timer for ServerReady messages.
    // Run as daemon thread so it won't prevent the JVM from exiting.
    Timer timer = new Timer(true);
    timer.schedule(new SendReady(),READY_INTERVAL_MS,READY_INTERVAL_MS);
  }
  private void parseUpstreamServers (XmlParser[] parsers) throws Exception {
    for (int i = 0; i < parsers.length; i++) {
       Util.sleep(2.0);
       parseUpstreamServer(parsers[i], i);
    }
  }
  private void parseUpstreamServer (XmlParser parser, int stationI) throws Exception {
    HostPort upstreamServerAddr = new HostPort(parser.getElementValue("hostPort"));
    ServerHandler upstreamServerHandler = new ServerHandler(upstreamServerAddr);
    upstreamServerHandler.setListener(new UpstreamServerListener(upstreamServerHandler,parsePortion(parser),stationI));
    upstreamServerHandler.start();
    log.info("Server: created/started Upstream ServerHandler/Listener: " + upstreamServerAddr.toString(),"v");
  }
  private String parsePortion (XmlParser parser) throws Exception {
    if (parser.hasElement("full")) {
      return "<full/>";
    }
    if (parser.hasElement("partial")) {
      StringBuffer xml = new StringBuffer();
      xml.append("<partial>");
      xml.append("<send><geolocated/><debowed/></send>");
      xml.append("<bands>");
      XmlParser[] parsers = parser.getElement("partial").getElement("bands").getElements("band");
      for (int i = 0; i < parsers.length; i++) {
	 xml.append("<band><bandIndex>"+parsers[i].getElementValue("bandIndex")+"</bandIndex><resolution>"+parsers[i].getElementValue("resolution")+"</resolution></band>");
      }
      xml.append("</bands>");
      xml.append("</partial>");
      return xml.toString();
    }
    throw new Exception("unknown portion");
  }
  /****************************************************************************
  * SendReady.
  ****************************************************************************/
  private class SendReady extends TimerTask {
    public void run () {
      clientManager.broadcast(Util.XML_HEADER + "<simulcastMessage>" +
			      "<serverReady/>" + "</simulcastMessage>");
      consoleManager.broadcast(Util.XML_HEADER + "<simulcastMessage>" +
			       "<serverReady/>" + "</simulcastMessage>");
    }
  }
  /****************************************************************************
  * stop.
  * On Windows, invoked when the service is told to stop.
  ****************************************************************************/
  public static void stop () {
    stop = true;
  }
}
