/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*  29-Jun-05, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import java.util.*;

public class TLE {
  private String[] lines;
  private Date date;
  /****************************************************************************
  * TLE.
  ****************************************************************************/
  public TLE (String[] lines) {
    this.lines = lines;
    date = parseDate(lines[0].substring(18,32));
  }
  /****************************************************************************
  * getLines.
  ****************************************************************************/
  public String[] getLines () {
    return lines;
  }
  /****************************************************************************
  * getDate.
  ****************************************************************************/
  public Date getDate () {
    return date;
  }
  /****************************************************************************
  * parseDate.
  ****************************************************************************/
  private Date parseDate (String field) {
    Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
    int year = Integer.parseInt(field.substring(0,2));
    year += (year < 57 ? 2000 : 1900);
    int doy = Integer.parseInt(field.substring(2,5));
    int msOfDay = (int) (Double.parseDouble(field.substring(5)) * (24 * 60 * 60 * 1000));
    int ms = msOfDay % 1000;
    int secOfDay = msOfDay / 1000;
    int sec = secOfDay % 60;
    int minOfDay = secOfDay / 60;
    int min = minOfDay % 60;
    int hour = minOfDay / 60;
    calendar.set(Calendar.YEAR,year);
    calendar.set(Calendar.DAY_OF_YEAR,doy);
    calendar.set(Calendar.HOUR_OF_DAY,hour);
    calendar.set(Calendar.MINUTE,min);
    calendar.set(Calendar.SECOND,sec);
    calendar.set(Calendar.MILLISECOND,ms);
    return calendar.getTime();
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public String toXml () {
    return "<tle>" + "<line1>" + lines[0] + "</line1>" +
    	             "<line2>" + lines[1] + "</line2>" + "</tle>";
  }
}
