/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*  15-Jul-05, J Love, GST	Original version.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.util.*;

public class TLEManager {
  private TLEArchive aquaArchive;
  private TLEArchive terraArchive;
  /****************************************************************************
  * TLEManager.
  ****************************************************************************/
  public TLEManager (XmlParser parser) throws Exception {
    aquaArchive = new TLEArchive("AQUA",parser.getElement("aqua"));
    terraArchive = new TLEArchive("TERRA",parser.getElement("terra"));
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start () {
    Util.startThread(aquaArchive,true);
    Util.startThread(terraArchive,true);
  }
  /****************************************************************************
  * getClosestTLE.
  ****************************************************************************/
  public String getClosestTLE (String satellite, Date date) {
    StringBuffer xml = new StringBuffer(Util.XML_HEADER);
    xml.append("<simulcastMessage>");
    xml.append("<closestTLE>");
    xml.append("<satellite>" + satellite + "</satellite>");
    xml.append("<dateTime>" + Long.toString(date.getTime()) + "</dateTime>");
    if (satellite.equals("AQUA")) {
      xml.append(aquaArchive.closestTLEtoXml(date));
    }
    if (satellite.equals("TERRA")) {
      xml.append(terraArchive.closestTLEtoXml(date));
    }
    xml.append("</closestTLE>");
    xml.append("</simulcastMessage>");
    return xml.toString();
  }
  /****************************************************************************
  * getMostRecentTLEs.
  ****************************************************************************/
  public String getMostRecentTLEs () {
    StringBuffer xml = new StringBuffer(Util.XML_HEADER);
    xml.append("<simulcastMessage>");
    xml.append("<mostRecentTLEs>");
    xml.append("<aqua>" + aquaArchive.mostRecentTLEtoXml() + "</aqua>");
    xml.append("<terra>" + terraArchive.mostRecentTLEtoXml() + "</terra>");
    xml.append("</mostRecentTLEs>");
    xml.append("</simulcastMessage>");
    return xml.toString();
  }
}
