/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
package gov.nasa.gsfc.aisb.simulcast.test;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.drl.geo.sgp4.*;
import java.awt.*;
// import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.text.*;

public class GeoTest1 extends JFrame {
  // private JFrame frame;
  private PlanePanel xyPlane;
  private PlanePanel xzPlane;
  private PlanePanel yzPlane;
  private PlanePanel oxyPlane;
  private PlanePanel oxzPlane;
  private PlanePanel oyzPlane;
  private CustomLabel timeField;
  private CustomLabel xField;
  private CustomLabel yField;
  private CustomLabel zField;
  private CustomLabel r1Field;
  private CustomLabel a1Field;
  private CustomLabel h1Field;
  private CustomLabel xxField;
  private CustomLabel yyField;
  private CustomLabel zzField;
  private CustomLabel altField;
  private CustomLabel latField;
  private CustomLabel lonField;
  private static final DecimalFormat fp3 = new DecimalFormat("0.00");
  private static final String[] tle = new String[] {
    "TERRA",
    "1 25994U 99068A   05084.11431949  .00000159  00000-0  45270-4 0   909",
    "2 25994  98.2107 159.9954 0001087 104.1319 256.0007 14.57101985280152"
  };
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    GeoTest1 geoTest = new GeoTest1();
    geoTest.start(args);
  }
  /****************************************************************************
  * GeoTest1.
  ****************************************************************************/
  public GeoTest1 () throws Exception {
    // frame = this;
    setTitle("GeoTest1");
    getContentPane().setLayout(new BorderLayout());
    xyPlane = new PlanePanel(8000);
    xzPlane = new PlanePanel(8000);
    yzPlane = new PlanePanel(8000);
    oxyPlane = new PlanePanel(2000);
    oxzPlane = new PlanePanel(2000);
    oyzPlane = new PlanePanel(2000);
    timeField = new CustomLabel(" ");
    xField = new CustomLabel(" ",70);
    yField = new CustomLabel(" ",70);
    zField = new CustomLabel(" ",70);
    r1Field = new CustomLabel(" ",70);
    a1Field = new CustomLabel(" ",70);
    h1Field = new CustomLabel(" ",70);
    xxField = new CustomLabel(" ",70);
    yyField = new CustomLabel(" ",70);
    zzField = new CustomLabel(" ",70);
    altField = new CustomLabel(" ",70);
    latField = new CustomLabel(" ",70);
    lonField = new CustomLabel(" ",70);
    HPanel hp1 = new HPanel(xyPlane,GUtil.HSpace(10),xzPlane,GUtil.HSpace(10),yzPlane);
    HPanel hp2 = new HPanel(oxyPlane,GUtil.HSpace(10),oxzPlane,GUtil.HSpace(10),oyzPlane);
    VPanel vp1 = new VPanel(new HPanel(new CustomLabel("Date/Time:"),GUtil.HSpace(5),timeField),GUtil.VGlue());
    VPanel vp2 = new VPanel(new LabelAndValue(new CustomLabel("x:"),xField),new LabelAndValue(new CustomLabel("y:"),yField),new LabelAndValue(new CustomLabel("z:"),zField),GUtil.VGlue());
    VPanel vp3 = new VPanel(new LabelAndValue(new CustomLabel("r1:"),r1Field),new LabelAndValue(new CustomLabel("a1:"),a1Field),new LabelAndValue(new CustomLabel("h1:"),h1Field),GUtil.VGlue());
    VPanel vp4 = new VPanel(new LabelAndValue(new CustomLabel("xx:"),xxField),new LabelAndValue(new CustomLabel("yy:"),yyField),new LabelAndValue(new CustomLabel("zz:"),zzField),GUtil.VGlue());
    VPanel vp5 = new VPanel(new LabelAndValue(new CustomLabel("alt:"),altField),new LabelAndValue(new CustomLabel("lat:"),latField),new LabelAndValue(new CustomLabel("lon:"),lonField),GUtil.VGlue());
    HPanel hp3 = new HPanel(GUtil.HGlue(),vp1,GUtil.HSpace(50),vp2,GUtil.HSpace(50),vp3,GUtil.HSpace(50),vp4,GUtil.HSpace(50),vp5,GUtil.HGlue());
    VPanel vp = new VPanel(GUtil.VSpace(20),hp1,GUtil.VSpace(10),hp2,GUtil.VSpace(10),hp3,GUtil.VGlue());
    getContentPane().add(vp,BorderLayout.CENTER);
    pack();
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    GUtil.configureWindow(this,0.95,0.85);
    setVisible(true);
  }
  private class LabelAndValue extends HPanel {
    private static final long serialVersionUID = 1L;
    public LabelAndValue (CustomLabel label, CustomLabel value) {
      // label.setAlignmentX(LEFT_ALIGNMENT);
      add(label);
      add(GUtil.HSpace(10));
      // value.setAlignmentX(RIGHT_ALIGNMENT);
      add(value);
    }
  }
  private class CustomLabel extends JLabel {
    private static final long serialVersionUID = 1L;
    public CustomLabel (String text) {
      super(text);
    }
    public CustomLabel (String text, int width) {
      super(text);
      setHorizontalAlignment(SwingConstants.RIGHT);
      Dimension dims = new Dimension(width,getPreferredSize().height);
      setMinimumSize(dims);
      setPreferredSize(dims);
      setMaximumSize(dims);
    }
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start (String[] args) throws Exception {
    Util.startThread(new FlySatellite());
  }
  /****************************************************************************
  * FlySatellite.
  ****************************************************************************/
  private class FlySatellite implements Runnable {
    public void run () {
      try {
      	fly();
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    private void fly () throws Exception {
      SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss zzz");
      sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
      // XYZ prevOppXYZ = new XYZ(0,0,0);
      double scanInterval = 1; // 1.477 / 60;
      Orbit orbit = new Orbit(new TLE(tle[0],tle[1],tle[2]));
      // System.out.println("orbit tle time: " + orbit.getTleTime());
      long startMs = orbit.getTleTime();
      for (double tsince = 0; true; tsince += scanInterval) {
      	 long ms = startMs + Math.round(tsince * (60 * 1000));
	 GUtil.invokeAndWait(new UpdateLabel(timeField,sdf.format(new Date(ms))));
         // double tsince = (((scanMs - orbit.getTleTime()) / 1000.0) / 60.0);
         ECI eci = orbit.getPosition(tsince);
         // System.out.println(tsince + " " + eci.getPos().m_x + " " + eci.getPos().m_y + " " + eci.getPos().m_z);
         XYZ scXYZ = new XYZ(eci.getPos().m_x,eci.getPos().m_y,eci.getPos().m_z);
         scXYZ.update(xyPlane,xzPlane,yzPlane);
         scXYZ.update(xField,yField,zField);
         RAH scRAH = scXYZ.getRAH();
         scRAH.update(r1Field,a1Field,h1Field);
         XYZ scXXYYZZ = scRAH.getXYZ();
         scXXYYZZ.update(xxField,yyField,zzField);
         CoordGeo geo = eci.toGeo();
         // System.out.println(pt7.format(tsince) + " " + pt7.format(geo.m_Lat) +  " " + pt7.format(geo.m_Lon) + " " + pt7.format(geo.m_Alt));
         // System.out.println(pt7.format(tsince) + " " + pt7.format(Globals.rad2deg(geo.m_Lat)) + " " + pt7.format(Globals.rad2deg(geo.m_Lon)) + " " + geo.m_Alt);
         double alt = geo.m_Alt;
	 GUtil.invokeAndWait(new UpdateLabel(altField,fp3.format(alt)));
	 // GUtil.invokeAndWait(new UpdateLabel(latField,fp3.format(Math.toDegrees(geo.m_Lat))));
	 // GUtil.invokeAndWait(new UpdateLabel(lonField,fp3.format(Math.toDegrees(geo.m_Lon))));
	 XYZ oppXYZ = scXYZ.getOpp(alt);
         oppXYZ.update(oxyPlane,oxzPlane,oyzPlane);
         
         // RAH oppRAH = oppXYZ.getRAH();
         // RAH prevOppRAH = prevOppXYZ.getRAH();
         

         // prevOppXYZ = oppXYZ;
         Util.sleep(100);
      }
    }
    /*
    private String pad (String s, int width) {
      StringBuffer b = new StringBuffer();
      for (int i = s.length(); i < width; i++) {
      	 b.append(" ");
      }
      b.append(s);
      return b.toString();
    }
    */
  }
  /****************************************************************************
  * XYZ.
  ****************************************************************************/
  private class XYZ {
    public double x;
    public double y;
    public double z;
    public XYZ (double x, double y, double z) {
      this.x = x;
      this.y = y;
      this.z = z;
    }
    public void update (PlanePanel xyP, PlanePanel xzP, PlanePanel yzP) {
      xyP.update(x,y);
      xzP.update(x,z);
      yzP.update(y,z);
    }
    public void update (CustomLabel xF, CustomLabel yF, CustomLabel zF) {
      GUtil.invokeAndWait(new UpdateLabel(xF,fp3.format(x)));
      GUtil.invokeAndWait(new UpdateLabel(yF,fp3.format(y)));
      GUtil.invokeAndWait(new UpdateLabel(zF,fp3.format(z)));
    }
    public RAH getRAH () {
      double r = Math.sqrt((x*x) + (y*y) + (z*z));
      double a = Math.toDegrees(Math.asin(z/r));
      double h = Math.toDegrees(Math.atan(y/x));
      if (x < 0) {
        if (y < 0) {
          h = h - 180.0;
        } else {
	  h = 180.0 + h;
        }
      }
      return new RAH(r,a,h);
    }
    public XYZ getOpp (double alt) {
      double r = Math.sqrt((x*x) + (y*y) + (z*z));
      double scale = alt / r;
      double ox = -x * scale;
      double oy = -y * scale;
      double oz = -z * scale;
      return new XYZ(ox,oy,oz);
    }
  }
  /****************************************************************************
  * RAH.
  ****************************************************************************/
  private class RAH {
    public double r;
    public double a;
    public double h;
    public RAH (double r, double a, double h) {
      this.r = r;
      this.a = a;
      this.h = h;
    }
    public void update (CustomLabel rF, CustomLabel aF, CustomLabel hF) {
      GUtil.invokeAndWait(new UpdateLabel(rF,fp3.format(r)));
      GUtil.invokeAndWait(new UpdateLabel(aF,fp3.format(a)));
      GUtil.invokeAndWait(new UpdateLabel(hF,fp3.format(h)));
    }
    public XYZ getXYZ () {
      double r1 = r * Math.cos(Math.toRadians(Math.abs(a)));
      double z = r * Math.sin(Math.toRadians(a));
      double x = r1 * Math.cos(Math.toRadians(h));
      double y = r1 * Math.sin(Math.toRadians(h));
      return new XYZ(x,y,z);
    }
  }
  /****************************************************************************
  * PlanePanel.
  ****************************************************************************/
  private class PlanePanel extends JPanel {
    private static final int PIXEL_WIDTH = 300;
    private static final int TRACK_SIZE = 80;
    private double max;
    private double twoMax;
    private int col = 0, row = 0;
    private LinkedList<int[]> track = new LinkedList<int[]>();
    private static final long serialVersionUID = 1L;
    public PlanePanel (double max) {
      this.max = max;
      twoMax = 2 * max;
      setMinimumSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
      setPreferredSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
      setMaximumSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
    }
    public void update (double x, double y) {
      col = calcCol(x);
      row = calcRow(y);
      if (track.size() > TRACK_SIZE) {
      	track.removeLast();
      }
      track.addFirst(new int[] {col,row});
      repaint();
    }
    public void paint (Graphics g) {
      g.setColor(Color.black);
      g.fillRect(0,0,PIXEL_WIDTH,PIXEL_WIDTH);
      // g.setColor(Color.gray);
      // g.drawLine(0,0,PIXEL_WIDTH,PIXEL_WIDTH);
      // g.drawLine(0,PIXEL_WIDTH-1,PIXEL_WIDTH-1,0);
      g.setColor(Color.white);
      g.drawLine(calcCol(0),0,calcCol(0),PIXEL_WIDTH-1);
      g.drawLine(0,calcRow(0),PIXEL_WIDTH-1,calcRow(0));
      g.setColor(Color.red);
      for (int[] pos : track) {
      // Iterator it = track.iterator();
      // while (it.hasNext()) {
        // int[] pos = (int[]) it.next();
        g.fillRect(pos[0],pos[1],1,1);
      }
      g.fillRect(col-1,row-1,3,3);
    }
    private int calcCol (double x) {
      return (int) Math.round(PIXEL_WIDTH * ((x + max) / (twoMax)));
    }
    private int calcRow (double y) {
      return PIXEL_WIDTH - (int) Math.round(PIXEL_WIDTH * ((y + max) / (twoMax)));
    }
  }
}
