/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
package gov.nasa.gsfc.aisb.simulcast.test;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.drl.geo.sgp4.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.text.*;

public class GeoTest2 extends JFrame {
  // private JFrame frame;
  private PlanePanel xyPlane;
  private PlanePanel xzPlane;
  private PlanePanel yzPlane;
  private PlanePanel2 oxyPlane;
  private PlanePanel2 oxzPlane;
  private PlanePanel2 oyzPlane;
  private PlanePanel2 rotPlane;
  private WorldMap map;
  private CustomLabel timeField;
  private CustomLabel xField;
  private CustomLabel yField;
  private CustomLabel zField;
  /*
  private CustomLabel hField;
  private CustomLabel eField;
  private CustomLabel dField;
  private CustomLabel xxField;
  private CustomLabel yyField;
  private CustomLabel zzField;
  */
  private CustomLabel altField;
  // private CustomLabel latField;
  // private CustomLabel lonField;
  // private CustomLabel rotField;
  private boolean pause = false;
  private static final DecimalFormat fp3 = new DecimalFormat("0.00");
  private static final String[] tle = new String[] {
    "TERRA",
    "1 25994U 99068A   05084.11431949  .00000159  00000-0  45270-4 0   909",
    "2 25994  98.2107 159.9954 0001087 104.1319 256.0007 14.57101985280152"
  };
  private static final int X = 0;
  private static final int Y = 1;
  private static final int Z = 2;
  //
  /*
  private static final int H = 0;
  private static final int E = 1;
  private static final int D = 2;
  */
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    GeoTest2 geoTest = new GeoTest2();
    geoTest.start(args);
  }
  /****************************************************************************
  * GeoTest2.
  ****************************************************************************/
  public GeoTest2 () throws Exception {
    // frame = this;
    setTitle("GeoTest2");
    getContentPane().setLayout(new BorderLayout());
    xyPlane = new PlanePanel(8000);
    xzPlane = new PlanePanel(8000);
    yzPlane = new PlanePanel(8000);
    oxyPlane = new PlanePanel2(2000);
    oxzPlane = new PlanePanel2(2000);
    oyzPlane = new PlanePanel2(2000);
    rotPlane = new PlanePanel2(2000);
    map = new WorldMap();
    timeField = new CustomLabel(" ");
    xField = new CustomLabel(" ",70);
    yField = new CustomLabel(" ",70);
    zField = new CustomLabel(" ",70);
    /*
    hField = new CustomLabel(" ",70);
    eField = new CustomLabel(" ",70);
    dField = new CustomLabel(" ",70);
    xxField = new CustomLabel(" ",70);
    yyField = new CustomLabel(" ",70);
    zzField = new CustomLabel(" ",70);
    */
    altField = new CustomLabel(" ",70);
    // latField = new CustomLabel(" ",70);
    // lonField = new CustomLabel(" ",70);
    // rotField = new CustomLabel(" ",70);
    HPanel hp1 = new HPanel(xyPlane,GUtil.HSpace(10),xzPlane,GUtil.HSpace(10),yzPlane);
    HPanel hp2 = new HPanel(oxyPlane,GUtil.HSpace(10),oxzPlane,GUtil.HSpace(10),oyzPlane);
    HPanel hp5 = new HPanel(rotPlane,GUtil.HSpace(10),map,GUtil.HSpace(10));
    /*
    VPanel vp1 = new VPanel(new HPanel(new CustomLabel("Date/Time:"),GUtil.HSpace(5),timeField),GUtil.VGlue());
    VPanel vp2 = new VPanel(new LabelAndValue(new CustomLabel("x:"),xField),new LabelAndValue(new CustomLabel("y:"),yField),new LabelAndValue(new CustomLabel("z:"),zField),GUtil.VGlue());
    VPanel vp3 = new VPanel(new LabelAndValue(new CustomLabel("h:"),hField),new LabelAndValue(new CustomLabel("e:"),eField),new LabelAndValue(new CustomLabel("d:"),dField),GUtil.VGlue());
    VPanel vp4 = new VPanel(new LabelAndValue(new CustomLabel("xx:"),xxField),new LabelAndValue(new CustomLabel("yy:"),yyField),new LabelAndValue(new CustomLabel("zz:"),zzField),GUtil.VGlue());
    VPanel vp5 = new VPanel(new LabelAndValue(new CustomLabel("alt:"),altField),new LabelAndValue(new CustomLabel("rot:"),rotField),GUtil.VGlue());
    HPanel hp3 = new HPanel(GUtil.HGlue(),vp1,GUtil.HSpace(50),vp2,GUtil.HSpace(50),vp3,GUtil.HSpace(50),vp4,GUtil.HSpace(50),vp5,GUtil.HGlue());
    */
    JButton pauseButton = new JButton("Pause");
    pauseButton.addActionListener(new PauseButtonClicked());
    HPanel hp4 = new HPanel(GUtil.HGlue(),pauseButton,GUtil.HSpace(20));
    VPanel vp = new VPanel(new Component[] {GUtil.VSpace(5),hp1,GUtil.VSpace(10),hp2,GUtil.VSpace(10),hp5,GUtil.VGlue(),GUtil.VSpace(10),hp4,GUtil.VSpace(5)});
    getContentPane().add(vp,BorderLayout.CENTER);
    pack();
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    // GUtil.configureWindow(this,0.95,0.85);
    setVisible(true);
  }
  private class PauseButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      pause = !pause;
    }
  }
  /*
  private class LabelAndValue extends HPanel {
    public LabelAndValue (CustomLabel label, CustomLabel value) {
      // label.setAlignmentX(LEFT_ALIGNMENT);
      add(label);
      add(GUtil.HSpace(10));
      // value.setAlignmentX(RIGHT_ALIGNMENT);
      add(value);
    }
  }
  */
  private class CustomLabel extends JLabel {
    private static final long serialVersionUID = 1L;
    public CustomLabel (String text) {
      super(text);
    }
    public CustomLabel (String text, int width) {
      super(text);
      setHorizontalAlignment(SwingConstants.RIGHT);
      Dimension dims = new Dimension(width,getPreferredSize().height);
      setMinimumSize(dims);
      setPreferredSize(dims);
      setMaximumSize(dims);
    }
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start (String[] args) throws Exception {
    Util.startThread(new FlySatellite());
  }
  /****************************************************************************
  * FlySatellite.
  ****************************************************************************/
  private class FlySatellite implements Runnable {
    public void run () {
      try {
      	fly();
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    private void fly () throws Exception {
      double[] prev_scOpp = new double[] {0.0,0.0,0.0};
      SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss zzz");
      sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
      double scanInterval = 0.5; // 1.477 / 60;
      Orbit orbit = new Orbit(new TLE(tle[0],tle[1],tle[2]));
      // System.out.println("orbit tle time: " + orbit.getTleTime());
      long startMs = orbit.getTleTime();
      for (double tsince = 0; true; tsince += scanInterval) {
      	 long ms = startMs + Math.round(tsince * (60 * 1000));
	 GUtil.invokeAndWait(new UpdateLabel(timeField,sdf.format(new Date(ms))));
	 // Calculate spacecraft location.
         ECI eci = orbit.getPosition(tsince);
         double scx = eci.getPos().m_x;
         double scy = eci.getPos().m_y;
         double scz = eci.getPos().m_z;
         /*
	 xyPlane.update(scx,scy);
	 xzPlane.update(scx,scz);
	 yzPlane.update(scy,scz);
	 */
	 GUtil.invokeAndWait(new UpdateLabel(xField,fp3.format(scx)));
	 GUtil.invokeAndWait(new UpdateLabel(yField,fp3.format(scy)));
	 GUtil.invokeAndWait(new UpdateLabel(zField,fp3.format(scz)));
	 /*
	 double[] sc = calcHED(new double[] {scx,scy,scz});
	 GUtil.invokeAndWait(new UpdateLabel(hField,fp3.format(sc[H])));
	 GUtil.invokeAndWait(new UpdateLabel(eField,fp3.format(sc[E])));
	 GUtil.invokeAndWait(new UpdateLabel(dField,fp3.format(sc[D])));
	 */
         CoordGeo geo = eci.toGeo();
         double alt = geo.m_Alt;
         double scLat = geo.m_Lat;
         double scLon = geo.m_Lon;
	 GUtil.invokeAndWait(new UpdateLabel(altField,fp3.format(alt)));
	 // Calculate...
	 double scD = Math.sqrt((scx*scx) + (scy*scy) + (scz*scz));
         double scale = alt / scD;
         double[] scOpp = new double[] {-scx*scale,-scy*scale,-scz*scale};
         // Calculate...
	 double d1, angle = 55.0;
	 if (angle > 0.0) {
           double aB = 180.0 - asin((scD * sin(angle)) / (scD - alt));
           double aA = 180.0 - aB - angle;
           d1 = (scD * sin(aA)) / sin(aB);
	 } else {
	   d1 = alt;
	 }
         // System.out.println("d1: " + d1);
	 /*
         double[] ray1 = calcXYZ(new double[] {angle,0.0,d1});
         double[] ray2 = calcXYZ(new double[] {360.0-angle,0.0,d1});
         */
	 double[] ray1 = new double[] {d1*cos(angle),d1*sin(angle),0.0};
	 double[] ray2 = new double[] {d1*cos(-angle),d1*sin(-angle),0.0};
	 // System.out.println("ray1: " + ray1[X] + " " + ray1[Y] + " " + ray1[Z]);
	 // System.out.println("ray2: " + ray2[X] + " " + ray2[Y] + " " + ray2[Z]);

	 /*
         double rotationYZ = calcRotationYZ(scOpp,prev_scOpp);
         // System.out.println("rotationYZ: " + rotationYZ);
	 GUtil.invokeAndWait(new UpdateLabel(rotField,fp3.format(rotationYZ)));
         // applyRotationYZ(ray1,rotationYZ);
         // applyRotationYZ(ray2,rotationYZ);
         */
	 double R = calcRotation(scOpp,prev_scOpp);
	 applyRotation(ray1,R);
	 applyRotation(ray2,R);
         rotPlane.update(ray1[Y],ray1[Z],ray2[Y],ray2[Z]);

         /*
         applyRotationXZ(ray1,scOpp);
         applyRotationXZ(ray2,scOpp);
         */
         applyElevation(ray1,scOpp);
         applyElevation(ray2,scOpp);
         
         /*
         applyRotationXY(ray1,scOpp);
         applyRotationXY(ray2,scOpp);
         */
         applyHeading(ray1,scOpp);
         applyHeading(ray2,scOpp);
         
         // System.out.println("opp:  " + scOpp[X] + " " + scOpp[Y] + " " + scOpp[Z]);
         // System.out.println("ray1: " + ray1[X] + " " + ray1[Y] + " " + ray1[Z]);
         
	 /*
         applyElevation(scOpp[E],ray1);
         applyElevation(scOpp[E],ray2);
	 applyHeading(scOpp[H],ray1);
	 applyHeading(scOpp[H],ray2);
	 */
         
	 xyPlane.update(scx,scy,scx+ray1[X],scy+ray1[Y],scx+ray2[X],scy+ray2[Y]);
	 xzPlane.update(scx,scz,scx+ray1[X],scz+ray1[Z],scx+ray2[X],scz+ray2[Z]);
	 yzPlane.update(scy,scz,scy+ray1[Y],scz+ray1[Z],scy+ray2[Y],scz+ray2[Z]);

	 /*         
         ray1[H] = (ray1[H] + scOpp[H]) % 360.0;
         ray1[E] = ray1[E] + scOpp[E];
         if (ray1[E] > 90.0) {
           // System.out.println("adjusting 1 +");
           ray1[E] = 90.0 - (ray1[E] - 90.0);
           ray1[H] = (ray1[H] + 180.0) % 360.0;
         } else {
           if (ray1[E] < -90.0) {
             // System.out.println("adjusting 1 -");
             ray1[E] = -90.0 - (ray1[E] + 90.0);
             ray1[H] = (ray1[H] + 180.0) % 360.0;
           }
         }
         // System.out.println("H: " + ray1[H] + ", E: " + ray1[E] + ", D: " + ray1[D]);
         ray2[H] = (ray2[H] + scOpp[H]) % 360.0;
         ray2[E] = ray2[E] + scOpp[E];
         if (ray2[E] > 90.0) {
           // System.out.println("adjusting 2 +");
           ray2[E] = 90.0 - (ray2[E] - 90.0);
           ray2[H] = (ray2[H] + 180.0) % 360.0;
         } else {
           if (ray2[E] < -90.0) {
             // System.out.println("adjusting 2 -");
             ray2[E] = -90.0 - (ray2[E] + 90.0);
             ray2[H] = (ray2[H] + 180.0) % 360.0;
           }
         }
         double[] ray11 = calcXYZ(ray1);
         double[] ray22 = calcXYZ(ray2);
         */
	 
	 
         oxyPlane.update(ray1[X],ray1[Y],ray2[X],ray2[Y]);
         oxzPlane.update(ray1[X],ray1[Z],ray2[X],ray2[Z]);
         oyzPlane.update(ray1[Y],ray1[Z],ray2[Y],ray2[Z]);
         /*
         double[] opp1 = calcXYZ(scOpp);
         double[] opp2 = calcXYZ(prev_scOpp);
         oxyPlane.update(opp1[X],opp1[Y],opp2[X],opp2[Y]);
         oxzPlane.update(opp1[X],opp1[Z],opp2[X],opp2[Z]);
         oyzPlane.update(opp1[Y],opp1[Z],opp2[Y],opp2[Z]);
         */

         ECI ray1eci = new ECI(new gov.nasa.gsfc.drl.geo.sgp4.Vector(scx+ray1[X],scy+ray1[Y],scz+ray1[Z]),eci.getVel(),eci.getDate());
         CoordGeo ray1geo = ray1eci.toGeo();
         ECI ray2eci = new ECI(new gov.nasa.gsfc.drl.geo.sgp4.Vector(scx+ray2[X],scy+ray2[Y],scz+ray2[Z]),eci.getVel(),eci.getDate());
         CoordGeo ray2geo = ray2eci.toGeo();
         // System.out.println(ray1geo.m_Lat + " " + scLat + " " + ray2geo.m_Lat);
         // System.out.println(ray1geo.m_Lon + " " + scLon + " " + ray2geo.m_Lon);
         map.update(Math.toDegrees(ray1geo.m_Lat),Math.toDegrees(ray1geo.m_Lon),Math.toDegrees(scLat),Math.toDegrees(scLon),Math.toDegrees(ray2geo.m_Lat),Math.toDegrees(ray2geo.m_Lon));
         
	 // Next time...
         prev_scOpp = scOpp;
         while (pause) {
           Util.sleep(100);
         }
         Util.sleep(5);
      }
    }
    /*
    private void applyHeading (double heading, double[] ray) {
      if (ray[X] == 0.0 && ray[Y] == 0.0) {
      	return;
      }
      double d = Math.sqrt((ray[X]*ray[X]) + (ray[Y]*ray[Y]));
      double h = calcHeading(ray[X],ray[Y]);
      h = (h + heading) % 360.0;
      ray[X] = d * cos(h);
      ray[Y] = d * sin(h);
      return;
    }
    */
    /*
    private void applyElevation (double elevation, double[] ray) {
      if (ray[X] == 0.0 && ray[Z] == 0.0) {
      	return;
      }
      double d = Math.sqrt((ray[X]*ray[X]) + (ray[Z]*ray[Z]));
      double e = asin(ray[Z]/d);
      e += elevation;
      ray[X] = d * cos(e);
      ray[Z] = d * sin(e);
      return;
    }
    */
    private double calcRotation (double[] opp, double[] prevOpp) {
      double[] cur = new double[] {opp[X],opp[Y],opp[Z]};
      double[] prev = new double[] {prevOpp[X],prevOpp[Y],prevOpp[Z]};
      double curH = calcAngle(cur[X],cur[Y]);
      double curD = Math.sqrt((cur[X]*cur[X]) + (cur[Y]*cur[Y]));
      cur[X] = curD * cos(0.0);
      cur[Y] = curD * sin(0.0);
      double prevH = calcAngle(prev[X],prev[Y]);
      // System.out.println("curH: " + curH + ", prevH: " + prevH);
      prevH = (360.0 + (prevH - curH)) % 360.0;
      double prevD = Math.sqrt((prev[X]*prev[X]) + (prev[Y]*prev[Y]));
      prev[X] = prevD * cos(prevH);
      prev[Y] = prevD * sin(prevH);

      // rotPlane.update(cur[X],cur[Y],prev[X],prev[Y]);
      
      double curE = calcAngle(cur[X],cur[Z]);
      curD = Math.sqrt((cur[X]*cur[X]) + (cur[Z]*cur[Z]));
      cur[X] = curD * cos(0.0);
      cur[Z] = curD * sin(0.0);
      double prevE = calcAngle(prev[X],prev[Z]);
      // System.out.println("curE: " + curE + ", prevE: " + prevE);
      prevE = (360.0 + (prevE - curE)) % 360.0;
      prevD = Math.sqrt((prev[X]*prev[X]) + (prev[Z]*prev[Z]));
      prev[X] = prevD * cos(prevE);
      prev[Z] = prevD * sin(prevE);

      // rotPlane.update(cur[X],cur[Z],prev[X],prev[Z]);
      
      double R = calcAngle(prev[Y],prev[Z]);
      R = (R + 270.0) % 360.0;
      // System.out.println("R: " + R);
      return R;
    }
    private void applyRotation (double[] ray, double rotation) {
      if (ray[Y] == 0.0 && ray[Z] == 0.0) {
      	return;
      }
      double d = Math.sqrt((ray[Y]*ray[Y]) + (ray[Z]*ray[Z]));
      double a = calcAngle(ray[Y],ray[Z]);
      a = (a + rotation) % 360.0;
      ray[Y] = d * cos(a);
      ray[Z] = d * sin(a);
      return;
    }
    private void applyElevation (double[] ray, double[] opp) {
      if (ray[X] == 0.0 && ray[Z] == 0.0) {
      	return;
      }

      // double oppAngle = calcAngle(opp[X],opp[Z]);
      double oppXYd = Math.sqrt((opp[X]*opp[X])+(opp[Y]*opp[Y]));
      double oppAngle = (360.0 + atan(opp[Z]/oppXYd)) % 360.0;
      
      double rayAngle = calcAngle(ray[X],ray[Z]);
      rayAngle = (rayAngle + oppAngle) % 360.0;
      double rayD = Math.sqrt((ray[X]*ray[X]) + (ray[Z]*ray[Z]));
      ray[X] = rayD * cos(rayAngle);
      ray[Z] = rayD * sin(rayAngle);
      return;
    }
    private void applyHeading (double[] ray, double[] opp) {
      if (ray[X] == 0.0 && ray[Y] == 0.0) {
      	return;
      }
      double oppAngle = calcAngle(opp[X],opp[Y]);
      double rayAngle = calcAngle(ray[X],ray[Y]);
      rayAngle = (rayAngle + oppAngle) % 360.0;
      double rayD = Math.sqrt((ray[X]*ray[X]) + (ray[Y]*ray[Y]));
      ray[X] = rayD * cos(rayAngle);
      ray[Y] = rayD * sin(rayAngle);
      return;
    }
    /*
    private double[] calcXYZ (double[] pt) {
      double xyd = pt[D] * cos(Math.abs(pt[E]));
      double z = pt[D] * sin(pt[E]);
      double x = xyd * cos(pt[H]);
      double y = xyd * sin(pt[H]);
      return new double[] {x,y,z};
    }
    */
    /*
    private double[] calcHED (double[] pt) {
      double d = Math.sqrt((pt[X]*pt[X]) + (pt[Y]*pt[Y]) + (pt[Z]*pt[Z]));
      double e = asin(pt[Z]/d);
      double h = calcHeading(pt[X],pt[Y]);
      return new double[] {h,e,d};
    }
    */
    private double calcAngle (double x, double y) {
      double d = Math.sqrt((x*x) + (y*y));
      double angle = asin(y/d);
      if (x < 0) {
      	angle = 180.0 - angle;
      } else {
        if (y < 0) {
          angle = 360.0 + angle;
        }
      }
      return angle;
    }
    /*
    private double calcHeading (double x, double y) {
      double d = Math.sqrt((x*x) + (y*y));
      double h = asin(y/d);
      if (x < 0) {
      	h = 180.0 - h;
      } else {
        if (y < 0) {
          h = 360.0 + h;
        }
      }
      return h;
    }
    */
    /*
    private String pad (String s, int width) {
      StringBuffer b = new StringBuffer();
      for (int i = s.length(); i < width; i++) {
      	 b.append(" ");
      }
      b.append(s);
      return b.toString();
    }
    */
    private double sin (double degrees) {
      return Math.sin(Math.toRadians(degrees));
    }
    private double cos (double degrees) {
      return Math.cos(Math.toRadians(degrees));
    }
    /*
    private double tan (double degrees) {
      return Math.tan(Math.toRadians(degrees));
    }
    */
    private double asin (double value) {
      return Math.toDegrees(Math.asin(value));
    }
    /*
    private double acos (double value) {
      return Math.toDegrees(Math.acos(value));
    }
    */
    private double atan (double value) {
      return Math.toDegrees(Math.atan(value));
    }
  }
  /****************************************************************************
  * PlanePanel.
  ****************************************************************************/
  private class PlanePanel extends JPanel {
    private static final int PIXEL_WIDTH = 200;
    private static final int TRACK_SIZE = 80;
    private double max;
    private double twoMax;
    private int col = 0, row = 0;
    private int col1 = 0, row1 = 0;
    private int col2 = 0, row2 = 0;
    private LinkedList<int[]> track = new LinkedList<int[]>();
    private static final long serialVersionUID = 1L;
    public PlanePanel (double max) {
      this.max = max;
      twoMax = 2 * max;
      setMinimumSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
      setPreferredSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
      setMaximumSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
    }
    public void update (double x, double y, double x1, double y1, double x2, double y2) {
      col = calcCol(x);
      row = calcRow(y);
      col1 = calcCol(x1);
      row1 = calcRow(y1);
      col2 = calcCol(x2);
      row2 = calcRow(y2);
      if (track.size() > TRACK_SIZE) {
      	track.removeLast();
      }
      track.addFirst(new int[] {col,row});
      repaint();
    }
    public void paint (Graphics g) {
      g.setColor(Color.black);
      g.fillRect(0,0,PIXEL_WIDTH,PIXEL_WIDTH);
      // g.setColor(Color.gray);
      // g.drawLine(0,0,PIXEL_WIDTH,PIXEL_WIDTH);
      // g.drawLine(0,PIXEL_WIDTH-1,PIXEL_WIDTH-1,0);
      g.setColor(Color.gray);
      g.drawLine(calcCol(0),0,calcCol(0),PIXEL_WIDTH-1);
      g.drawLine(0,calcRow(0),PIXEL_WIDTH-1,calcRow(0));
      g.setColor(Color.white);
      for (int[] pos : track) {
        g.fillRect(pos[0],pos[1],1,1);
      }
      g.fillRect(col-1,row-1,3,3);
      g.setColor(Color.red);
      g.drawLine(col,row,col1,row1);
      g.fillRect(col1-1,row1-1,3,3);
      g.setColor(Color.green);
      g.drawLine(col,row,col2,row2);
      g.fillRect(col2-1,row2-1,3,3);
    }
    private int calcCol (double x) {
      return (int) Math.round(PIXEL_WIDTH * ((x + max) / (twoMax)));
    }
    private int calcRow (double y) {
      return PIXEL_WIDTH - (int) Math.round(PIXEL_WIDTH * ((y + max) / (twoMax)));
    }
  }
  /****************************************************************************
  * PlanePanel2.
  ****************************************************************************/
  private class PlanePanel2 extends JPanel {
    private static final int PIXEL_WIDTH = 200;
    private double max;
    private double twoMax;
    private int zeroRow, zeroCol;
    private int col1 = 0, row1 = 0;
    private int col2 = 0, row2 = 0;
    private static final long serialVersionUID = 1L;
    public PlanePanel2 (double max) {
      this.max = max;
      twoMax = 2 * max;
      zeroCol = calcCol(0);
      zeroRow = calcRow(0);
      setMinimumSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
      setPreferredSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
      setMaximumSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
    }
    public void update (double x1, double y1, double x2, double y2) {
      col1 = calcCol(x1);
      row1 = calcRow(y1);
      col2 = calcCol(x2);
      row2 = calcRow(y2);
      repaint();
    }
    public void paint (Graphics g) {
      g.setColor(Color.black);
      g.fillRect(0,0,PIXEL_WIDTH,PIXEL_WIDTH);
      g.setColor(Color.white);
      g.drawLine(zeroCol,0,zeroCol,PIXEL_WIDTH-1);
      g.drawLine(0,zeroRow,PIXEL_WIDTH-1,zeroRow);
      g.setColor(Color.red);
      g.drawLine(zeroCol,zeroRow,col1,row1);
      g.fillRect(col1-1,row1-1,3,3);
      g.setColor(Color.green);
      g.drawLine(zeroCol,zeroRow,col2,row2);
      g.fillRect(col2-1,row2-1,3,3);
    }
    private int calcCol (double x) {
      return (int) Math.round(PIXEL_WIDTH * ((x + max) / (twoMax)));
    }
    private int calcRow (double y) {
      return PIXEL_WIDTH - (int) Math.round(PIXEL_WIDTH * ((y + max) / (twoMax)));
    }
  }
  /****************************************************************************
  * WorldMap.
  ****************************************************************************/
  private class WorldMap extends JPanel {
    private static final int PIXEL_HEIGHT = 200;
    private static final int PIXEL_WIDTH = 400;
    private int colLeft = 0, rowLeft = 0;
    private int col = 0, row = 0;
    private int colRight = 0, rowRight = 0;
    private static final long serialVersionUID = 1L;
    public WorldMap () {
      setMinimumSize(new Dimension(PIXEL_WIDTH,PIXEL_HEIGHT));
      setPreferredSize(new Dimension(PIXEL_WIDTH,PIXEL_HEIGHT));
      setMaximumSize(new Dimension(PIXEL_WIDTH,PIXEL_HEIGHT));
    }
    public void update (double latLeft, double lonLeft, double lat, double lon, double latRight, double lonRight) {
      // System.out.println(latLeft + " " + lat + " " + latRight);
      // System.out.println(lonLeft + " " + lon + " " + lonRight);
      colLeft = calcCol(lonLeft);
      rowLeft = calcRow(latLeft);
      col = calcCol(lon);
      row = calcRow(lat);
      colRight = calcCol(lonRight);
      rowRight = calcRow(latRight);
      // System.out.println(colLeft + " " + col + " " + colRight);
      // System.out.println(rowLeft + " " + row + " " + rowRight);
      repaint();
    }
    public void paint (Graphics g) {
      g.setColor(Color.black);
      g.fillRect(0,0,PIXEL_WIDTH,PIXEL_HEIGHT);
      g.setColor(Color.red);
      g.fillRect(colLeft-1,rowLeft-1,3,3);
      g.setColor(Color.white);
      g.fillRect(col-1,row-1,3,3);
      g.setColor(Color.green);
      g.fillRect(colRight-1,rowRight-1,3,3);
    }
    private int calcCol (double lon) {
      return (int) Math.round(PIXEL_WIDTH * (lon / 360.0));
    }
    private int calcRow (double lat) {
      return (int) Math.round(PIXEL_HEIGHT * ((90.0 - lat) / 180.0));
    }
  }
}
