/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
package gov.nasa.gsfc.aisb.simulcast.test;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.drl.geo.sgp4.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.text.*;

public class GeoTest2b extends JFrame {
  // private JFrame frame;
  private PlanePanel xyPlane;
  private PlanePanel xzPlane;
  private PlanePanel yzPlane;
  private PlanePanel2 oxyPlane;
  private PlanePanel2 oxzPlane;
  private PlanePanel2 oyzPlane;
  private PlanePanel2 rotPlane;
  private WorldMap map;
  private boolean pause = false;
  private static final String[] tle = new String[] {
    // "TERRA",
    // "1 25994U 99068A   05084.11431949  .00000159  00000-0  45270-4 0   909",
    // "2 25994  98.2107 159.9954 0001087 104.1319 256.0007 14.57101985280152"
    // "TERRA",
    // "1 25994U 99068A   05138.15722649  .00000266  00000-0  68949-4 0  2586",
    // "2 25994  98.2042 213.2582 0001275 100.5029 259.6313 14.57124553288027"
    "AQUA",
    "1 27424U 02022A   05138.00000000  .00000272  00000-0  60414-4 0    16",
    "2 27424 098.2026 078.7358 0001712 090.8268 074.7938 14.57095054000019"
  };
  private static final int X = 0;
  private static final int Y = 1;
  private static final int Z = 2;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    GeoTest2b geoTest = new GeoTest2b();
    geoTest.start(args);
  }
  /****************************************************************************
  * GeoTest2.
  ****************************************************************************/
  public GeoTest2b () throws Exception {
    // frame = this;
    setTitle("GeoTest2b");
    getContentPane().setLayout(new BorderLayout());
    xyPlane = new PlanePanel(8000);
    xzPlane = new PlanePanel(8000);
    yzPlane = new PlanePanel(8000);
    oxyPlane = new PlanePanel2(2000);
    oxzPlane = new PlanePanel2(2000);
    oyzPlane = new PlanePanel2(2000);
    rotPlane = new PlanePanel2(2000);
    map = new WorldMap();
    HPanel hp1 = new HPanel(xyPlane,GUtil.HSpace(10),xzPlane,GUtil.HSpace(10),yzPlane);
    HPanel hp2 = new HPanel(oxyPlane,GUtil.HSpace(10),oxzPlane,GUtil.HSpace(10),oyzPlane);
    HPanel hp5 = new HPanel(rotPlane,GUtil.HSpace(10),map,GUtil.HSpace(10));
    JButton pauseButton = new JButton("Pause");
    pauseButton.addActionListener(new PauseButtonClicked());
    HPanel hp4 = new HPanel(GUtil.HGlue(),pauseButton,GUtil.HSpace(20));
    VPanel vp = new VPanel(new Component[] {GUtil.VSpace(5),hp1,GUtil.VSpace(10),hp2,GUtil.VSpace(10),hp5,GUtil.VGlue(),GUtil.VSpace(10),hp4,GUtil.VSpace(5)});
    getContentPane().add(vp,BorderLayout.CENTER);
    pack();
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setVisible(true);
  }
  private class PauseButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      pause = !pause;
    }
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start (String[] args) throws Exception {
    Util.startThread(new FlySatellite());
  }
  /****************************************************************************
  * FlySatellite.
  ****************************************************************************/
  private class FlySatellite implements Runnable {
    /**************************************************************************
    * run.
    **************************************************************************/
    public void run () {
      try {
      	fly();
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    /**************************************************************************
    * fly.
    **************************************************************************/
    private void fly () throws Exception {
      double[] prev_scOpp = new double[] {0.0,0.0,0.0};
      SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss zzz");
      sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
      double scanInterval = 0.5; // 1.477 / 60;
      Orbit orbit = new Orbit(new TLE(tle[0],tle[1],tle[2]));
      // System.out.println("orbit tle time: " + orbit.getTleTime());
      // long startMs = orbit.getTleTime();
      for (double tsince = 0; true; tsince += scanInterval) {
      	 // long ms = startMs + Math.round(tsince * (60 * 1000));
	 // Calculate spacecraft location.
         ECI eci = orbit.getPosition(tsince);
         double scx = eci.getPos().m_x;
         double scy = eci.getPos().m_y;
         double scz = eci.getPos().m_z;
         CoordGeo geo = eci.toGeo();
         double alt = geo.m_Alt;
         double scLat = geo.m_Lat;
         double scLon = geo.m_Lon;
	 // Calculate...
	 double scD = Math.sqrt((scx*scx) + (scy*scy) + (scz*scz));
         double scale = alt / scD;
         double[] scOpp = new double[] {-scx*scale,-scy*scale,-scz*scale};
         // Calculate...
         double[] ray1 = calcRay(55.0,0.4,scD,alt);
         double[] ray2 = calcRay(-55.0,0.4,scD,alt);
	 // System.out.println("ray1: " + ray1[X] + " " + ray1[Y] + " " + ray1[Z]);
	 // System.out.println("ray2: " + ray2[X] + " " + ray2[Y] + " " + ray2[Z]);
	 double R = calcRotation(scOpp,prev_scOpp);
	 applyRotation(ray1,R);
	 applyRotation(ray2,R);
         rotPlane.update(ray1[Y],ray1[Z],ray2[Y],ray2[Z]);
         applyElevation(ray1,scOpp);
         applyElevation(ray2,scOpp);
         applyHeading(ray1,scOpp);
         applyHeading(ray2,scOpp);
	 xyPlane.update(scx,scy,scx+ray1[X],scy+ray1[Y],scx+ray2[X],scy+ray2[Y]);
	 xzPlane.update(scx,scz,scx+ray1[X],scz+ray1[Z],scx+ray2[X],scz+ray2[Z]);
	 yzPlane.update(scy,scz,scy+ray1[Y],scz+ray1[Z],scy+ray2[Y],scz+ray2[Z]);
         oxyPlane.update(ray1[X],ray1[Y],ray2[X],ray2[Y]);
         oxzPlane.update(ray1[X],ray1[Z],ray2[X],ray2[Z]);
         oyzPlane.update(ray1[Y],ray1[Z],ray2[Y],ray2[Z]);
         ECI ray1eci = new ECI(new gov.nasa.gsfc.drl.geo.sgp4.Vector(scx+ray1[X],scy+ray1[Y],scz+ray1[Z]),eci.getVel(),eci.getDate());
         CoordGeo ray1geo = ray1eci.toGeo();
         ECI ray2eci = new ECI(new gov.nasa.gsfc.drl.geo.sgp4.Vector(scx+ray2[X],scy+ray2[Y],scz+ray2[Z]),eci.getVel(),eci.getDate());
         CoordGeo ray2geo = ray2eci.toGeo();
         // System.out.println(ray1geo.m_Lat + " " + scLat + " " + ray2geo.m_Lat);
         // System.out.println(ray1geo.m_Lon + " " + scLon + " " + ray2geo.m_Lon);
         map.update(Math.toDegrees(ray1geo.m_Lat),Math.toDegrees(ray1geo.m_Lon),Math.toDegrees(scLat),Math.toDegrees(scLon),Math.toDegrees(ray2geo.m_Lat),Math.toDegrees(ray2geo.m_Lon));
	 // Next time...
         prev_scOpp = scOpp;
         while (pause) {
           Util.sleep(100);
         }
         Util.sleep(50);
      }
    }
    /**************************************************************************
    * calcRay.
    * This assumes the earth is a cylinder rather than a sphere.
    **************************************************************************/
    private double[] calcRay (double scanAngle, double fovAngle, double scD, double alt) {
      double d1;
      if (Math.abs(scanAngle) > 0.0) {
	double aB = 180.0 - asin((scD * sin(Math.abs(scanAngle))) / (scD - alt));
	double aA = 180.0 - aB - Math.abs(scanAngle);
	d1 = (scD * sin(aA)) / sin(aB);
      } else {
	d1 = alt;
      }
      return new double[] {d1*cos(scanAngle),d1*sin(scanAngle),d1*sin(fovAngle)};
    }
    /**************************************************************************
    * calcRotation.
    **************************************************************************/
    private double calcRotation (double[] opp, double[] prevOpp) {
      double[] cur = new double[] {opp[X],opp[Y],opp[Z]};
      double[] prev = new double[] {prevOpp[X],prevOpp[Y],prevOpp[Z]};
      double curH = calcAngle(cur[X],cur[Y]);
      double curD = Math.sqrt((cur[X]*cur[X]) + (cur[Y]*cur[Y]));
      cur[X] = curD * cos(0.0);
      cur[Y] = curD * sin(0.0);
      double prevH = calcAngle(prev[X],prev[Y]);
      // System.out.println("curH: " + curH + ", prevH: " + prevH);
      prevH = (360.0 + (prevH - curH)) % 360.0;
      double prevD = Math.sqrt((prev[X]*prev[X]) + (prev[Y]*prev[Y]));
      prev[X] = prevD * cos(prevH);
      prev[Y] = prevD * sin(prevH);
      double curE = calcAngle(cur[X],cur[Z]);
      curD = Math.sqrt((cur[X]*cur[X]) + (cur[Z]*cur[Z]));
      cur[X] = curD * cos(0.0);
      cur[Z] = curD * sin(0.0);
      double prevE = calcAngle(prev[X],prev[Z]);
      // System.out.println("curE: " + curE + ", prevE: " + prevE);
      prevE = (360.0 + (prevE - curE)) % 360.0;
      prevD = Math.sqrt((prev[X]*prev[X]) + (prev[Z]*prev[Z]));
      prev[X] = prevD * cos(prevE);
      prev[Z] = prevD * sin(prevE);
      double R = calcAngle(prev[Y],prev[Z]);
      R = (R + 270.0) % 360.0;
      // System.out.println("R: " + R);
      return R;
    }
    /**************************************************************************
    * applyRotation.
    **************************************************************************/
    private void applyRotation (double[] ray, double rotation) {
      if (ray[Y] == 0.0 && ray[Z] == 0.0) {
      	return;
      }
      double d = Math.sqrt((ray[Y]*ray[Y]) + (ray[Z]*ray[Z]));
      double a = calcAngle(ray[Y],ray[Z]);
      a = (a + rotation) % 360.0;
      ray[Y] = d * cos(a);
      ray[Z] = d * sin(a);
      return;
    }
    /**************************************************************************
    * applyElevation.
    **************************************************************************/
    private void applyElevation (double[] ray, double[] opp) {
      if (ray[X] == 0.0 && ray[Z] == 0.0) {
      	return;
      }
      double oppXYd = Math.sqrt((opp[X]*opp[X])+(opp[Y]*opp[Y]));
      double oppAngle = (360.0 + atan(opp[Z]/oppXYd)) % 360.0;
      double rayAngle = calcAngle(ray[X],ray[Z]);
      rayAngle = (rayAngle + oppAngle) % 360.0;
      double rayD = Math.sqrt((ray[X]*ray[X]) + (ray[Z]*ray[Z]));
      ray[X] = rayD * cos(rayAngle);
      ray[Z] = rayD * sin(rayAngle);
      return;
    }
    /**************************************************************************
    * applyHeading.
    **************************************************************************/
    private void applyHeading (double[] ray, double[] opp) {
      if (ray[X] == 0.0 && ray[Y] == 0.0) {
      	return;
      }
      double oppAngle = calcAngle(opp[X],opp[Y]);
      double rayAngle = calcAngle(ray[X],ray[Y]);
      rayAngle = (rayAngle + oppAngle) % 360.0;
      double rayD = Math.sqrt((ray[X]*ray[X]) + (ray[Y]*ray[Y]));
      ray[X] = rayD * cos(rayAngle);
      ray[Y] = rayD * sin(rayAngle);
      return;
    }
    /**************************************************************************
    * calcAngle.
    **************************************************************************/
    private double calcAngle (double x, double y) {
      double d = Math.sqrt((x*x) + (y*y));
      double angle = asin(y/d);
      if (x < 0) {
      	angle = 180.0 - angle;
      } else {
        if (y < 0) {
          angle = 360.0 + angle;
        }
      }
      return angle;
    }
    /**************************************************************************
    * sin, cos, etc...
    **************************************************************************/
    private double sin (double degrees) {
      return Math.sin(Math.toRadians(degrees));
    }
    private double cos (double degrees) {
      return Math.cos(Math.toRadians(degrees));
    }
    /*
    private double tan (double degrees) {
      return Math.tan(Math.toRadians(degrees));
    }
    */
    private double asin (double value) {
      return Math.toDegrees(Math.asin(value));
    }
    /*
    private double acos (double value) {
      return Math.toDegrees(Math.acos(value));
    }
    */
    private double atan (double value) {
      return Math.toDegrees(Math.atan(value));
    }
  }
  /****************************************************************************
  * PlanePanel.
  ****************************************************************************/
  private class PlanePanel extends JPanel {
    private static final int PIXEL_WIDTH = 200;
    private static final int TRACK_SIZE = 80;
    private double max;
    private double twoMax;
    private int col = 0, row = 0;
    private int col1 = 0, row1 = 0;
    private int col2 = 0, row2 = 0;
    private LinkedList<int[]> track = new LinkedList<int[]>();
    private static final long serialVersionUID = 1L;
    public PlanePanel (double max) {
      this.max = max;
      twoMax = 2 * max;
      setMinimumSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
      setPreferredSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
      setMaximumSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
    }
    public void update (double x, double y, double x1, double y1, double x2, double y2) {
      col = calcCol(x);
      row = calcRow(y);
      col1 = calcCol(x1);
      row1 = calcRow(y1);
      col2 = calcCol(x2);
      row2 = calcRow(y2);
      if (track.size() > TRACK_SIZE) {
      	track.removeLast();
      }
      track.addFirst(new int[] {col,row});
      repaint();
    }
    public void paint (Graphics g) {
      g.setColor(Color.black);
      g.fillRect(0,0,PIXEL_WIDTH,PIXEL_WIDTH);
      // g.setColor(Color.gray);
      // g.drawLine(0,0,PIXEL_WIDTH,PIXEL_WIDTH);
      // g.drawLine(0,PIXEL_WIDTH-1,PIXEL_WIDTH-1,0);
      g.setColor(Color.gray);
      g.drawLine(calcCol(0),0,calcCol(0),PIXEL_WIDTH-1);
      g.drawLine(0,calcRow(0),PIXEL_WIDTH-1,calcRow(0));
      g.setColor(Color.white);
      for (int[] pos : track) {
        g.fillRect(pos[0],pos[1],1,1);
      }
      g.fillRect(col-1,row-1,3,3);
      g.setColor(Color.red);
      g.drawLine(col,row,col1,row1);
      g.fillRect(col1-1,row1-1,3,3);
      g.setColor(Color.green);
      g.drawLine(col,row,col2,row2);
      g.fillRect(col2-1,row2-1,3,3);
    }
    private int calcCol (double x) {
      return (int) Math.round(PIXEL_WIDTH * ((x + max) / (twoMax)));
    }
    private int calcRow (double y) {
      return PIXEL_WIDTH - (int) Math.round(PIXEL_WIDTH * ((y + max) / (twoMax)));
    }
  }
  /****************************************************************************
  * PlanePanel2.
  ****************************************************************************/
  private class PlanePanel2 extends JPanel {
    private static final int PIXEL_WIDTH = 200;
    private double max;
    private double twoMax;
    private int zeroRow, zeroCol;
    private int col1 = 0, row1 = 0;
    private int col2 = 0, row2 = 0;
    private static final long serialVersionUID = 1L;
    public PlanePanel2 (double max) {
      this.max = max;
      twoMax = 2 * max;
      zeroCol = calcCol(0);
      zeroRow = calcRow(0);
      setMinimumSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
      setPreferredSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
      setMaximumSize(new Dimension(PIXEL_WIDTH,PIXEL_WIDTH));
    }
    public void update (double x1, double y1, double x2, double y2) {
      col1 = calcCol(x1);
      row1 = calcRow(y1);
      col2 = calcCol(x2);
      row2 = calcRow(y2);
      repaint();
    }
    public void paint (Graphics g) {
      g.setColor(Color.black);
      g.fillRect(0,0,PIXEL_WIDTH,PIXEL_WIDTH);
      g.setColor(Color.gray);
      g.drawLine(zeroCol,0,zeroCol,PIXEL_WIDTH-1);
      g.drawLine(0,zeroRow,PIXEL_WIDTH-1,zeroRow);
      g.setColor(Color.red);
      g.drawLine(zeroCol,zeroRow,col1,row1);
      g.fillRect(col1-1,row1-1,3,3);
      g.setColor(Color.green);
      g.drawLine(zeroCol,zeroRow,col2,row2);
      g.fillRect(col2-1,row2-1,3,3);
    }
    private int calcCol (double x) {
      return (int) Math.round(PIXEL_WIDTH * ((x + max) / (twoMax)));
    }
    private int calcRow (double y) {
      return PIXEL_WIDTH - (int) Math.round(PIXEL_WIDTH * ((y + max) / (twoMax)));
    }
  }
  /****************************************************************************
  * WorldMap.
  ****************************************************************************/
  private class WorldMap extends JPanel {
    private static final int PIXEL_HEIGHT = 200;
    private static final int PIXEL_WIDTH = 400;
    private int colLeft = 0, rowLeft = 0;
    private int col = 0, row = 0;
    private int colRight = 0, rowRight = 0;
    private static final long serialVersionUID = 1L;
    public WorldMap () {
      setMinimumSize(new Dimension(PIXEL_WIDTH,PIXEL_HEIGHT));
      setPreferredSize(new Dimension(PIXEL_WIDTH,PIXEL_HEIGHT));
      setMaximumSize(new Dimension(PIXEL_WIDTH,PIXEL_HEIGHT));
    }
    public void update (double latLeft, double lonLeft, double lat, double lon, double latRight, double lonRight) {
      // System.out.println(latLeft + " " + lat + " " + latRight);
      // System.out.println(lonLeft + " " + lon + " " + lonRight);
      colLeft = calcCol(lonLeft);
      rowLeft = calcRow(latLeft);
      col = calcCol(lon);
      row = calcRow(lat);
      colRight = calcCol(lonRight);
      rowRight = calcRow(latRight);
      // System.out.println(colLeft + " " + col + " " + colRight);
      // System.out.println(rowLeft + " " + row + " " + rowRight);
      repaint();
    }
    public void paint (Graphics g) {
      g.setColor(Color.black);
      g.fillRect(0,0,PIXEL_WIDTH,PIXEL_HEIGHT);
      g.setColor(Color.red);
      g.fillRect(colLeft-1,rowLeft-1,3,3);
      g.setColor(Color.white);
      g.fillRect(col-1,row-1,3,3);
      g.setColor(Color.green);
      g.fillRect(colRight-1,rowRight-1,3,3);
    }
    private int calcCol (double lon) {
      return (int) Math.round(PIXEL_WIDTH * (lon / 360.0));
    }
    private int calcRow (double lat) {
      return (int) Math.round(PIXEL_HEIGHT * ((90.0 - lat) / 180.0));
    }
  }
}
