/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
package gov.nasa.gsfc.aisb.simulcast.test;
import java.io.*;

public class ShpTest1 {
  private static final int POINT = 1;
  private static final int POLYLINE = 3;
  private static final int POLYGON = 5;
  private static final int POLYGON_M = 25;
  public static void main (String[] args) throws Exception {
    FileInputStream in = null;
    try {
      in = new FileInputStream(args[0]);
      readBigInt(in);
      readBigInt(in);
      readBigInt(in);
      readBigInt(in);
      readBigInt(in);
      readBigInt(in);
      readBigInt(in);
      readLittleInt(in);
      readLittleInt(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      while (true) {
	readBigInt(in);
	readBigInt(in);
	int shapeType = readLittleInt(in);
	switch (shapeType) {
	  case POINT: {
	    float lon = (float) readLittleDouble(in);
	    float lat = (float) readLittleDouble(in);
	    System.out.println("POINT, lat: " + lat + ", lon: " + lon);
	    break;
	  }
	  case POLYLINE:
	  case POLYGON:
	  case POLYGON_M: {
	    /*float xMin = (float)*/ readLittleDouble(in);
	    /*float yMin = (float)*/ readLittleDouble(in);
	    /*float xMax = (float)*/ readLittleDouble(in);
	    /*float yMax = (float)*/ readLittleDouble(in);
	    int numParts = readLittleInt(in);
	    int numPoints = readLittleInt(in);
	    int[] partOffsets = new int[numParts+1];
	    for (int i = 0; i < numParts; i++) {
	       partOffsets[i] = readLittleInt(in);
	    }
	    partOffsets[numParts] = numPoints;
	    for (int i = 0; i < numParts; i++) {
	       int count = partOffsets[i+1] - partOffsets[i];
	       for (int j = 0; j < count; j++) {
	          float lon = (float) readLittleDouble(in);
	          float lat = (float) readLittleDouble(in);
	          System.out.println("POLY, lat: " + lat + ", lon: " + lon);
	       }
	    }
	    if (shapeType == POLYGON_M) {
	      readLittleDouble(in);			// Mmin
	      readLittleDouble(in);			// Mmax
	      for (int i = 0; i < numPoints; i++) {
	         readLittleDouble(in);			// Marray value
	      }
	    }
	    break;
	  }
	  default:
	    throw new Exception("unknown shape type: " + shapeType);
	}
      }
    } catch (EOFException e) {
      // This is expected - all other exceptions are not.
    } finally {
      try {in.close();} catch (Exception e) {}
    }
  }
  /****************************************************************************
  * readBigInt.
  ****************************************************************************/
  private static int readBigInt (InputStream in) throws Exception {
    byte[] bytes = new byte[4];
    for (int i = 0; i < 4; i++) {
       int b = in.read();
       if (b == -1) throw new EOFException();
       bytes[i] = (byte) b;
    }
    DataInputStream dis = new DataInputStream(new ByteArrayInputStream(bytes));
    int value = dis.readInt();
    dis.close();
    return value;
  }
  /****************************************************************************
  * readLittleInt.
  ****************************************************************************/
  private static int readLittleInt (InputStream in) throws Exception {
    byte[] bytes = new byte[4];
    for (int i = 0; i < 4; i++) {
       int b = in.read();
       if (b == -1) throw new EOFException();
       bytes[3-i] = (byte) b;
    }
    DataInputStream dis = new DataInputStream(new ByteArrayInputStream(bytes));
    int value = dis.readInt();
    dis.close();
    return value;
  }
  /****************************************************************************
  * readLittleDouble.
  ****************************************************************************/
  private static double readLittleDouble (InputStream in) throws Exception {
    byte[] bytes = new byte[8];
    for (int i = 0; i < 8; i++) {
       int b = in.read();
       if (b == -1) throw new EOFException();
       bytes[7-i] = (byte) b;
    }
    DataInputStream dis = new DataInputStream(new ByteArrayInputStream(bytes));
    double value = dis.readDouble();
    dis.close();
    return value;
  }
}
