/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*  10-Feb-06, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.awt.*;
import javax.swing.*;

public class ArrowButton extends JButton {
  private boolean border;
  private int direction;
  private int size;
  private Color foregroundColor = Color.black;
  private Color backgroundColor;
  public static final int UP = 1;
  public static final int DOWN = 2;
  public static final int LEFT = 3;
  public static final int RIGHT = 4;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ArrowButton (constructor).
  ****************************************************************************/
  public ArrowButton (boolean border, int direction) {
    construct(border,direction,13);
  }
  public ArrowButton (boolean border, int direction, int size) {
    construct(border,direction,size);
  }
  private void construct (boolean border, int direction, int size) {
    this.border = border;
    this.direction = direction;
    this.size = size - (1 - (size % 2));
    backgroundColor = getBackground();
    setPreferredSize(new Dimension(size,size));
    setMinimumSize(getPreferredSize());
    setMaximumSize(getPreferredSize());
  }
  /****************************************************************************
  * getDirection.
  ****************************************************************************/
  public int getDirection () {
    return direction;
  }
  /****************************************************************************
  * setDirection.
  ****************************************************************************/
  public void setDirection (int direction) {
    this.direction = direction;
    repaint();
  }
  /****************************************************************************
  * setColor.
  ****************************************************************************/
  public void setColor (Color color) {
    foregroundColor = color;
    repaint();
  }
  /****************************************************************************
  * paint.
  ****************************************************************************/
  public void paint (Graphics g) {
    g.setColor(backgroundColor);
    g.fillRect(0,0,size,size);
    g.setColor(foregroundColor);
    if (border) {
      g.drawRect(0,0,size-1,size-1);
      int longLen = size - 4;
      int shortLen = (longLen + 1) / 2;
      int offset = (size - 1 - shortLen - 1) / 2;
      switch (direction) {
        case UP:
	  for (int x = 2, y = size - 2 - offset, w = longLen; w > 0; x++, y--, w -= 2) {
	     g.drawLine(x,y,x+w-1,y);
	  }
	  break;
        case DOWN:
	  for (int x = 2, y = 1 + offset, w = longLen; w > 0; x++, y++, w -= 2) {
	     g.drawLine(x,y,x+w-1,y);
	  }
	  break;
        case LEFT:
	  for (int x = size - 2 - offset, y = 2, h = longLen; h > 0; x--, y++, h -= 2) {
	     g.drawLine(x,y,x,y+h-1);
	  }
	  break;
        case RIGHT:
	  for (int x = 1 + offset, y = 2, h = longLen; h > 0; x++, y++, h -= 2) {
	     g.drawLine(x,y,x,y+h-1);
	  }
	  break;
      }
    } else {
      int longLen = size - 2;
      int shortLen = (longLen + 1) / 2;
      int offset = (size - shortLen) / 2;
      switch (direction) {
        case UP:
	  for (int x = 1, y = size - 2 - offset, w = longLen; w > 0; x++, y--, w -= 2) {
	     g.drawLine(x,y,x+w-1,y);
	  }
	  break;
        case DOWN:
	  for (int x = 1, y = 1 + offset, w = longLen; w > 0; x++, y++, w -= 2) {
	     g.drawLine(x,y,x+w-1,y);
	  }
	  break;
        case LEFT:
	  for (int x = size - 2 - offset, y = 1, h = longLen; h > 0; x--, y++, h -= 2) {
	     g.drawLine(x,y,x,y+h-1);
	  }
	  break;
        case RIGHT:
	  for (int x = 1 + offset, y = 1, h = longLen; h > 0; x++, y++, h -= 2) {
	     g.drawLine(x,y,x,y+h-1);
	  }
	  break;
      }
    }
  }
}
