/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*   6-Dec-04, J Love, GST	Original version.
*  10-Feb-06, J Love, GST	Clean up.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

public class ArrowPanel extends VPanel {
  private ArrowButton arrowButton;
  private JLabel titleLabel;
  private Component component;
  private Runnable rightClickRunnable = null;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ArrowPanel.
  ****************************************************************************/
  public ArrowPanel (boolean border, int direction, String text, int indent, Component component) {
    setBorder(new EmptyBorder(1,1,1,1));
    this.component = component;
    switch (direction) {
      case ArrowButton.DOWN: {
	component.setVisible(true);
	break;
      }
      case ArrowButton.RIGHT: {
        component.setVisible(false);
	break;
      }
    }
    arrowButton = new ArrowButton(border,direction);
    arrowButton.addMouseListener(new TitleLineClicked());
    titleLabel = new JLabel(text);
    titleLabel.addMouseListener(new TitleLineClicked());
    HPanel arrowPanel = new HPanel(arrowButton,GUtil.HSpace(5),titleLabel);
    arrowPanel.setAlignmentX(LEFT_ALIGNMENT);
    add(arrowPanel);
    HPanel componentPanel = new HPanel(GUtil.HSpace(indent),component);
    componentPanel.setAlignmentX(LEFT_ALIGNMENT);
    add(componentPanel);
  }
  /****************************************************************************
  * setRightClickRunnable.
  ****************************************************************************/
  public void setRightClickRunnable (Runnable rightClickRunnable) {
    this.rightClickRunnable = rightClickRunnable;
  }
  /****************************************************************************
  * getTitle.
  ****************************************************************************/
  public String getTitle () {
    return titleLabel.getText();
  }
  /****************************************************************************
  * setTitle.
  ****************************************************************************/
  public void setTitle (String text, Color color) {
    titleLabel.setText(text);
    setColor(color);
  }
  /****************************************************************************
  * setColor.
  ****************************************************************************/
  public void setColor (Color color) {
    arrowButton.setColor(color);
    titleLabel.setForeground(color);
  }
  /****************************************************************************
  * TitleLineClicked.
  ****************************************************************************/
  private class TitleLineClicked extends MouseAdapter {
    public void mouseClicked (MouseEvent me) {
      switch (me.getButton()) {
      	case MouseEvent.BUTTON1: {
	  doArrow();
	  break;
        }
      	case MouseEvent.BUTTON2: {
	  break;
        }
      	case MouseEvent.BUTTON3: {
      	  if (rightClickRunnable != null) {
	    rightClickRunnable.run();
      	  }
	  break;
        }
      }
    }
  }
  /****************************************************************************
  * doArrow.
  ****************************************************************************/
  private void doArrow () {
    switch (arrowButton.getDirection()) {
      case ArrowButton.DOWN: {
        component.setVisible(false);
        arrowButton.setDirection(ArrowButton.RIGHT);
	break;
      }
      case ArrowButton.RIGHT: {
        component.setVisible(true);
        arrowButton.setDirection(ArrowButton.DOWN);
	break;
      }
    }
    revalidate();
    repaint();
  }
}
