/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*   5-Aug-04, J Love, GST	Original version.
*  19-Dec-05, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.io.*;

public class DataBlock implements Block {
  private int blockCode;
  private byte[][][] data;
  /****************************************************************************
  * DataBlock.
  ****************************************************************************/
  public DataBlock (int blockCode, byte[][][] data) {
    this.blockCode = blockCode;
    this.data = data;
  }
  public DataBlock (int blockCode, DataInput in) throws Exception {
    this.blockCode = blockCode;
    read(in);
  }
  /****************************************************************************
  * read.
  ****************************************************************************/
  public void read (DataInput in) throws Exception {
    int nBands = in.readInt();
    if (data == null) {
      data = new byte[nBands][][];
    }
    for (int band = 0; band < nBands; band++) {
       int nRows = in.readInt();
       int nCols = in.readInt();
       if (data[band] == null) {
	 data[band] = new byte[nRows][nCols];
       }
    }
    for (int band = 0; band < nBands; band++) {
       for (int row = 0; row < data[band].length; row++) {
          in.readFully(data[band][row]);
       }
    }
  }
  /****************************************************************************
  * skip.
  ****************************************************************************/
  public static void skip (DataInput in) throws Exception {
    int nBytes = 0;
    int nBands = in.readInt();
    for (int band = 0; band < nBands; band++) {
       int nRows = in.readInt();
       int nCols = in.readInt();
       nBytes += nRows * nCols;
    }
    Util.skipBytes(in,nBytes);
  }
  /****************************************************************************
  * getData.
  ****************************************************************************/
  public byte[][][] getData () {
    return data;
  }
  /****************************************************************************
  * size.
  ****************************************************************************/
  public int size () {
    int size = 0;
    size += 4;
    size += 4;
    for (int band = 0; band < data.length; band++) {
       size += data[band].length * data[band][0].length;
    }
    return size;
  }
  /****************************************************************************
  * write.
  ****************************************************************************/
  public void write (DataOutput out) throws Exception {
    out.writeInt(blockCode);
    out.writeInt(data.length);
    for (int band = 0; band < data.length; band++) {
       out.writeInt(data[band].length);
       out.writeInt(data[band][0].length);
    }
    for (int band = 0; band < data.length; band++) {
       for (int row = 0; row < data[band].length; row++) {
       	  out.write(data[band][row]);
       }
    }
  }
}
