/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*  14-Jul-04, J Love, GST	Original version.
*  19-Dec-05, J Love, GST	Geolocation, etc.
*  11-Jan-07, J Love, GST	Support for fixing a corrupted pass file.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.io.*;

public class FooterBlock implements Block {
  private int endSolarCode;
  private long endedAt;
  private int nDayScans;
  private int nNightScans;
  private int nDayEvGroups;
  private int nDaySvCalGroups;
  private int nDaySdCalGroups;
  private int nDaySrcaCalGroups;
  private int nDayBbCalGroups;
  private int nNightEvGroups;
  private int nNightSvCalGroups;
  private int nNightSdCalGroups;
  private int nNightSrcaCalGroups;
  private int nNightBbCalGroups;
  private int nEng1Groups;
  private int nEng2Groups;
  /****************************************************************************
  * PassFooter.
  ****************************************************************************/
  public FooterBlock (int endSolarCode, long endedAt, int nDayScans,
  		     int nNightScans, int nDayEvGroups, int nDaySvCalGroups,
  		     int nDaySdCalGroups, int nDaySrcaCalGroups,
  		     int nDayBbCalGroups, int nNightEvGroups,
  		     int nNightSvCalGroups, int nNightSdCalGroups,
		     int nNightSrcaCalGroups, int nNightBbCalGroups,
		     int nEng1Groups, int nEng2Groups) {
    this.endSolarCode = endSolarCode;
    this.endedAt = endedAt;
    this.nDayScans = nDayScans;
    this.nNightScans = nNightScans;
    this.nDayEvGroups = nDayEvGroups;
    this.nDaySvCalGroups = nDaySvCalGroups;
    this.nDaySdCalGroups = nDaySdCalGroups;
    this.nDaySrcaCalGroups = nDaySrcaCalGroups;
    this.nDayBbCalGroups = nDayBbCalGroups;
    this.nNightEvGroups = nNightEvGroups;
    this.nNightSvCalGroups = nNightSvCalGroups;
    this.nNightSdCalGroups = nNightSdCalGroups;
    this.nNightSrcaCalGroups = nNightSrcaCalGroups;
    this.nNightBbCalGroups = nNightBbCalGroups;
    this.nEng1Groups = nEng1Groups;
    this.nEng2Groups = nEng2Groups;
  }
  public FooterBlock () {
    this.endSolarCode = Simulcast.UNKNOWN;
    this.endedAt = Simulcast.UNKNOWN;
    this.nDayScans = 0;
    this.nNightScans = 0;
    this.nDayEvGroups = 0;
    this.nDaySvCalGroups = 0;
    this.nDaySdCalGroups = 0;
    this.nDaySrcaCalGroups = 0;
    this.nDayBbCalGroups = 0;
    this.nNightEvGroups = 0;
    this.nNightSvCalGroups = 0;
    this.nNightSdCalGroups = 0;
    this.nNightSrcaCalGroups = 0;
    this.nNightBbCalGroups = 0;
    this.nEng1Groups = 0;
    this.nEng2Groups = 0;
  }
  public FooterBlock (DataInput in) throws Exception {
    read(in);
  }
  /****************************************************************************
  * read.
  ****************************************************************************/
  public void read (DataInput in) throws Exception {
    endSolarCode = in.readInt();
    endedAt = in.readLong();
    nDayScans = in.readInt();
    nNightScans = in.readInt();
    nDayEvGroups = in.readInt();
    nDaySvCalGroups = in.readInt();
    nDaySdCalGroups = in.readInt();
    nDaySrcaCalGroups = in.readInt();
    nDayBbCalGroups = in.readInt();
    nNightEvGroups = in.readInt();
    nNightSvCalGroups = in.readInt();
    nNightSdCalGroups = in.readInt();
    nNightSrcaCalGroups = in.readInt();
    nNightBbCalGroups = in.readInt();
    nEng1Groups = in.readInt();
    nEng2Groups = in.readInt();
  }
  /****************************************************************************
  * getEndSolarCode.
  ****************************************************************************/
  public int getEndSolarCode () {
    return endSolarCode;
  }
  /****************************************************************************
  * getEndedAt.
  ****************************************************************************/
  public long getEndedAt () {
    return endedAt;
  }
  /****************************************************************************
  * getDayScanCount.
  ****************************************************************************/
  public int getDayScanCount () {
    return nDayScans;
  }
  /****************************************************************************
  * getNightScanCount.
  ****************************************************************************/
  public int getNightScanCount () {
    return nNightScans;
  }
  /****************************************************************************
  * setEndSolarCode.
  ****************************************************************************/
  public void setEndSolarCode (int endSolarCode) {
    this.endSolarCode = endSolarCode;
  }
  /****************************************************************************
  * setEndedAt.
  ****************************************************************************/
  public void setEndedAt (long endedAt) {
    this.endedAt = endedAt;
  }
  /****************************************************************************
  * incrDayScanCount.
  ****************************************************************************/
  public void incrDayScanCount () {
    nDayScans++;
  }
  /****************************************************************************
  * incrNightScanCount.
  ****************************************************************************/
  public void incrNightScanCount () {
    nNightScans++;
  }
  /****************************************************************************
  * write.
  ****************************************************************************/
  public void write (DataOutput out) throws Exception {
    out.writeInt(Simulcast.FOOTER_BLOCK_CODE);
    out.writeInt(endSolarCode);
    out.writeLong(endedAt);
    out.writeInt(nDayScans);
    out.writeInt(nNightScans);
    out.writeInt(nDayEvGroups);
    out.writeInt(nDaySvCalGroups);
    out.writeInt(nDaySdCalGroups);
    out.writeInt(nDaySrcaCalGroups);
    out.writeInt(nDayBbCalGroups);
    out.writeInt(nNightEvGroups);
    out.writeInt(nNightSvCalGroups);
    out.writeInt(nNightSdCalGroups);
    out.writeInt(nNightSrcaCalGroups);
    out.writeInt(nNightBbCalGroups);
    out.writeInt(nEng1Groups);
    out.writeInt(nEng2Groups);
  }
}
