/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*  21-Nov-04, J Love, GST	Original version.
*   8-Feb-06, J Love, GST	Java 1.5.
*  26-Jul-06, J Love, GST	More button icons.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import javax.swing.*;
import java.awt.*;
import java.net.*;

public class GUtil {
  private static Color[] grays = null;
  // private static Color[] irColors = null;
  public static final Color DARK_GREEN = new Color(0,180,0);
  public static final ImageIcon playIcon = getImageIcon("images/play.gif");
  public static final ImageIcon[] playingIcons = new ImageIcon[] {
    getImageIcon("images/playing3/playing0.gif"),
    getImageIcon("images/playing3/playing1.gif"),
    getImageIcon("images/playing3/playing2.gif"),
    getImageIcon("images/playing3/playing3.gif"),
    getImageIcon("images/playing3/playing4.gif"),
    getImageIcon("images/playing3/playing5.gif"),
    getImageIcon("images/playing3/playing6.gif"),
    getImageIcon("images/playing3/playing7.gif")
  };
  public static final ImageIcon[] pausedIcons = new ImageIcon[] {
    getImageIcon("images/paused1/paused0.gif"),
    getImageIcon("images/paused1/paused0.gif"),
    getImageIcon("images/paused1/paused1.gif"),
    getImageIcon("images/paused1/paused1.gif"),
    getImageIcon("images/paused1/paused0.gif"),
    getImageIcon("images/paused1/paused0.gif"),
    getImageIcon("images/paused1/paused1.gif"),
    getImageIcon("images/paused1/paused1.gif")
  };
  public static final ImageIcon upIcon = getImageIcon("images/up.png");
  public static final ImageIcon downIcon = getImageIcon("images/down.png");
  public static final ImageIcon pauseIcon = getImageIcon("images/pause.gif");
  public static final ImageIcon stopIcon = getImageIcon("images/stop.gif");
  public static final ImageIcon eraseIcon = getImageIcon("images/erase.gif");
  public static final ImageIcon saveIcon = getImageIcon("images/save.png");
  public static final ImageIcon closeIcon = getImageIcon("images/close.png");
  public static final ImageIcon visibleIcon = getImageIcon("images/visible.png");
  public static final ImageIcon toMapIcon = getImageIcon("images/toMap.png");
  public static final ImageIcon toInstrumentIcon = getImageIcon("images/toInstrument.png");
  /****************************************************************************
  * getGrays.
  ****************************************************************************/
  public static Color[] getGrays () {
    if (grays == null) {
      grays = new Color[256];
      for (int value = 0; value < grays.length; value++) {
         grays[value] = new Color(value,value,value);
      }
    }
    return grays;
  }
  /****************************************************************************
  * buttonGroup.
  ****************************************************************************/
  public static ButtonGroup buttonGroup (AbstractButton b1, AbstractButton b2) {
    return buttonGroup(new AbstractButton[] {b1,b2});
  }
  public static ButtonGroup buttonGroup (AbstractButton b1, AbstractButton b2,
  					 AbstractButton b3) {
    return buttonGroup(new AbstractButton[] {b1,b2,b3});
  }
  public static ButtonGroup buttonGroup (AbstractButton b1, AbstractButton b2,
  					 AbstractButton b3, AbstractButton b4) {
    return buttonGroup(new AbstractButton[] {b1,b2,b3,b4});
  }
  public static ButtonGroup buttonGroup (AbstractButton b1, AbstractButton b2,
  					 AbstractButton b3, AbstractButton b4,
					 AbstractButton b5) {
    return buttonGroup(new AbstractButton[] {b1,b2,b3,b4,b5});
  }
  public static ButtonGroup buttonGroup(AbstractButton[] buttons) {
    ButtonGroup group = new ButtonGroup();
    for (int i = 0; i < buttons.length; i++) {
       group.add(buttons[i]);
    }
    return group;
  }
  /****************************************************************************
  * getImageIcon.
  ****************************************************************************/
  public static ImageIcon getImageIcon (String path) {
    try {
      Object o = new Object();
      Class c = o.getClass();
      URL url = c.getResource("/"+path);
      return new ImageIcon(url);
    } catch (Exception e) {
    	
    }
    try {
      return new ImageIcon(path);
    } catch (Exception e) {
    	
    }
    return new ImageIcon();
  }
  /****************************************************************************
  * SetCombo.
  ****************************************************************************/
  public static void setCombo (JComboBox combo, int index) {
    invokeAndWait(new SetCombo(combo,index));
  }
  private static class SetCombo implements Runnable {
    private JComboBox combo;
    private int index;
    public SetCombo (JComboBox combo, int index) {
      this.combo = combo;
      this.index = index;
    }
    public void run () {
      combo.setSelectedIndex(index);
    }
  }
  /****************************************************************************
  * invokeLater.
  ****************************************************************************/
  public static void invokeLater (Runnable runnable) {
    SwingUtilities.invokeLater(runnable);
  }
  /****************************************************************************
  * invokeAndWait.
  ****************************************************************************/
  public static void invokeAndWait (Runnable runnable) {
    if (SwingUtilities.isEventDispatchThread()) {
      runnable.run();
    } else {
      try {
        SwingUtilities.invokeAndWait(runnable);
      } catch (Exception e) {
    	
      }
    }
  }
  /****************************************************************************
  * append.
  ****************************************************************************/
  public static void append (JTextArea area, String text) {
    SwingUtilities.invokeLater(new AppendJTextArea(area,text));
  }
  private static class AppendJTextArea implements Runnable {
    private JTextArea area;
    private String text;
    public AppendJTextArea (JTextArea area, String text) {
      this.area = area;
      this.text = text;
    }
    public void run () {
      area.append(text);
      area.repaint();
    }
  }
  /****************************************************************************
  * HGlue.
  ****************************************************************************/
  public static Component HGlue () {
    return Box.createHorizontalGlue();
  }
  /****************************************************************************
  * HSpace.
  ****************************************************************************/
  public static Component HSpace (int width) {
    return Box.createRigidArea(new Dimension(width,1));
  }
  /****************************************************************************
  * VGlue.
  ****************************************************************************/
  public static Component VGlue () {
    return Box.createVerticalGlue();
  }
  /****************************************************************************
  * VSpace.
  ****************************************************************************/
  public static Component VSpace (int height) {
    return Box.createRigidArea(new Dimension(1,height));
  }
  /****************************************************************************
  * limitHeight.
  ****************************************************************************/
  public static void limitHeight (JComponent component) {
    component.setMaximumSize(new Dimension(component.getMaximumSize().width,
    					   component.getPreferredSize().height));
  }
  /****************************************************************************
  * configureWindow.
  ****************************************************************************/
  public static void configureWindow (Window window,
				      int width,
				      int height) {
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    int screenWidth = d.width;
    int screenHeight = d.height;
    window.setSize(width,height);
    window.setLocation((screenWidth - width) / 2,
		       (screenHeight - height) / 2);
  }
  public static void configureWindow (Window window,
				      double pctOfWidth,
				      double pctOfHeight) {
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    int screenWidth = d.width;
    int screenHeight = d.height;
    int windowWidth = (int) Math.round(screenWidth * pctOfWidth);
    int windowHeight = (int) Math.round(screenHeight * pctOfHeight);
    window.setSize(windowWidth,windowHeight);
    window.setLocation((screenWidth - windowWidth) / 2,
		       (screenHeight - windowHeight) / 2);
  }
  /****************************************************************************
  * centerWindow.
  ****************************************************************************/
  public static void centerWindow (Window window) {
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    int screenWidth = d.width;
    int screenHeight = d.height;
    int windowWidth = window.getWidth();
    int windowHeight = window.getHeight();
    window.setLocation((screenWidth - windowWidth) / 2,
		       (screenHeight - windowHeight) / 2);
  }
}
