/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*   8-Feb-06, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
// import gov.nasa.gsfc.drl.geo.*;
import java.io.*;

public class GeoBlock implements Block {
  // private double scanAngleAdjustment;
  // private double trackAngleAdjustment;
  // private Geo geo = null;
  // private double scLat;
  // private double scLon;
  // private double scAlt;
  private int nRows;
  private int nCols;
  private double[][] lats;
  private double[][] lons;
  // private static final int LAT = 0;
  // private static final int LON = 1;
  // private static final int ALT = 2;
  /****************************************************************************
  * GeoBlock.
  ****************************************************************************/
  public GeoBlock (/*double scanAngleAdjustment, double trackAngleAdjustment*/) {
    // this.scanAngleAdjustment = scanAngleAdjustment;
    // this.trackAngleAdjustment = trackAngleAdjustment;
    nRows = 10;
    nCols = 1354;
    lats = new double[nRows+1][nCols+1];
    lons = new double[nRows+1][nCols+1];
  }
  public GeoBlock (int nRows, int nCols) {
    this.nRows = nRows;
    this.nCols = nCols;
    lats = new double[nRows+1][nCols+1];
    lons = new double[nRows+1][nCols+1];
  }
  public GeoBlock (DataInput in) throws Exception {
    read(in);
  }
  /****************************************************************************
  * setOrbit.
  ****************************************************************************/
  /*
  public void setOrbit (String[] tle) throws Exception {
    geo = new Geo(tle);
  }
  */
  /****************************************************************************
  * read.
  ****************************************************************************/
  public void read (DataInput in) throws Exception {
    nRows = in.readInt();
    nCols = in.readInt();
    if (lats == null) {
      lats = new double[nRows+1][nCols+1];
    }
    if (lons == null) {
      lons = new double[nRows+1][nCols+1];
    }
    for (int row = 0; row <= nRows; row++) {
       for (int col = 0; col <= nCols; col++) {
       	  lats[row][col] = in.readDouble();
       	  lons[row][col] = in.readDouble();
       }
    }
  }
  /****************************************************************************
  * skip.
  ****************************************************************************/
  public static void skip (DataInput in) throws Exception {
    int nRows = in.readInt();
    int nCols = in.readInt();
    Util.skipBytes(in,((nRows+1)*(nCols+1))*(8+8));
  }
  /****************************************************************************
  * getRowCount.
  ****************************************************************************/
  public int getRowCount () {
    return nRows;
  }
  /****************************************************************************
  * getColCount.
  ****************************************************************************/
  public int getColCount () {
    return nCols;
  }
  /****************************************************************************
  * getLats.
  ****************************************************************************/
  public double[][] getLats () {
    return lats;
  }
  /****************************************************************************
  * getLons.
  ****************************************************************************/
  public double[][] getLons () {
    return lons;
  }
  /****************************************************************************
  * size.
  ****************************************************************************/
  public int size () {
    return 4 + 4 + 4 + (nRows * nCols * (8 + 8));
  }
  /****************************************************************************
  * write.
  ****************************************************************************/
  public void write (DataOutput out) throws Exception {
    out.writeInt(Simulcast.GEO_BLOCK_CODE);
    out.writeInt(nRows);
    out.writeInt(nCols);
    for (int row = 0; row <= nRows; row++) {
       for (int col = 0; col <= nCols; col++) {
       	  out.writeDouble(lats[row][col]);
       	  out.writeDouble(lons[row][col]);
       }
    }
  }
  /****************************************************************************
  * calc.
  ****************************************************************************/
  /*
  public void calc (long scanTime) throws Exception {
    Geo.Location loc = geo.calcLoc(scanTime);
    scLat = loc.getLat();
    scLon = loc.getLon();
    scAlt = loc.getAlt();
    geo.calcScan(scanTime,lats,lons);
  }
  */
}
