/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*  10-Nov-05, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.io.*;

public class GroundStation {
  private String id;
  private String name;
  private double lat;
  private double lon;
  /****************************************************************************
  * GroundStation.
  ****************************************************************************/
  public GroundStation (XmlParser parser) throws Exception {
    id = parser.getElementValue("id");
    name = parser.getElementValue("name");
    lat = Double.parseDouble(parser.getElementValue("lat"));
    lon = Double.parseDouble(parser.getElementValue("lon"));
  }
  public GroundStation (DataInput in) throws Exception {
    id = in.readUTF();
    name = in.readUTF();
    lat = in.readDouble();
    lon = in.readDouble();
  }
  /****************************************************************************
  * getId
  ****************************************************************************/
  public String getId () {
    return id;
  }
  /****************************************************************************
  * getName.
  ****************************************************************************/
  public String getName () {
    return name;
  }
  /****************************************************************************
  * getLat.
  ****************************************************************************/
  public double getLat () {
    return lat;
  }
  /****************************************************************************
  * getLon.
  ****************************************************************************/
  public double getLon () {
    return lon;
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public String toXml () {
    return "<groundStation>" +
    	   "<id>" + id + "</id>" +
	   "<name>" + name + "</name>" +
	   "<lat>" + Double.toString(lat) + "</lat>" +
	   "<lon>" + Double.toString(lon) + "</lon>" +
	   "</groundStation>";
  }
  /****************************************************************************
  * toString.
  ****************************************************************************/
  public String toString () {
    return name + " (" + Double.toString(lat) + "," + Double.toString(lon) + ")";
  }
  /****************************************************************************
  * write.
  ****************************************************************************/
  public void write (DataOutput out) throws Exception {
    out.writeUTF(id);
    out.writeUTF(name);
    out.writeDouble(lat);
    out.writeDouble(lon);
  }
}
