/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*  20-Jul-04, J Love, GST	Original version.
*  10-Feb-06, J Love, GST	Clean up.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import javax.swing.*;
import java.util.*;

public class LabeledField extends JPanel {
  private JTextField textField;
  private java.util.Timer timer = null;
  private String pendingText = null;
  private static final int UPDATE_GAP_MS = 1000;
  private static final long serialVersionUID = 1L;
  public LabeledField (String label) {
    setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
    textField = new JTextField();
    GUtil.limitHeight(textField);
    add(new MarginPanel(2,2,new HPanel(new JLabel(label),GUtil.HSpace(5),textField),2,2));
  }
  public synchronized void setText (String text) {
    if (timer == null) {
      GUtil.invokeAndWait(new SetText(text));
      timer = new java.util.Timer();
      timer.schedule(new TimeToUpdate(),UPDATE_GAP_MS);
    } else {
      pendingText = text;
    }
  }
  private class SetText implements Runnable {
    private String text;
    public SetText (String text) {
      this.text = text;
    }
    public void run () {
      textField.setText(text);
      textField.repaint();
    }
  }
  private class TimeToUpdate extends TimerTask {
    public void run () {
      update();
    }
  }
  private synchronized void update () {
    if (pendingText != null) {
      SwingUtilities.invokeLater(new SetText(pendingText));
      timer = new java.util.Timer();
      timer.schedule(new TimeToUpdate(),UPDATE_GAP_MS);
      pendingText = null;
    } else {
      timer = null;
    }
  }
}
