/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  26-Aug-05, J Love, GST	Original version.
*   3-Aug-06, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.awt.*;
import java.awt.image.*;

public class MapImage extends BufferedImage {
  private MapParms mapParms;
  // private int[] values = new int[3];
  // private Color[] grays = GUtil.getGrays();
  private Graphics g;
  /****************************************************************************
  * MapImage.
  ****************************************************************************/
  public MapImage (MapParms mapParms) {
    super(mapParms.getWidth(),mapParms.getHeight(),TYPE_INT_RGB);
    this.mapParms = mapParms;
    g = getGraphics();
    g.setClip(mapParms.getViewport());
    clear();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    g.setColor(Color.black);
    g.fillRect(0,0,mapParms.getWidth(),mapParms.getHeight());
  }
  /****************************************************************************
  * dataBlock.
  ****************************************************************************/
  public void dataBlock (int nGeoDataRows, int nGeoDataCols, int geoBinSize,
  			 double[][] geoLats, double[][] geoLons, Color[][] colors) {
    double[][] lats = new double[geoBinSize+1][geoBinSize+1];
    double[][] lons = new double[geoBinSize+1][geoBinSize+1];
    for (int geoRow = 0; geoRow < nGeoDataRows; geoRow++) {
       for (int geoCol = 0; geoCol < nGeoDataCols; geoCol++) {
          loadGeo(geoLats[geoRow][geoCol],geoLats[geoRow][geoCol+1],
          	  geoLats[geoRow+1][geoCol],geoLats[geoRow+1][geoCol+1],
		  geoBinSize,lats);
          loadGeo(geoLons[geoRow][geoCol],geoLons[geoRow][geoCol+1],
          	  geoLons[geoRow+1][geoCol],geoLons[geoRow+1][geoCol+1],
		  geoBinSize,lons);
          for (int rx = 0; rx < geoBinSize; rx++) {
             for (int cx = 0; cx < geoBinSize; cx++) {
		int row = (geoRow * geoBinSize) + rx;
		int col = (geoCol * geoBinSize) + cx;
		g.setColor(colors[row][col]);
		mapParms.fillPoly(g,lats[rx][cx],lons[rx][cx],
				    lats[rx][cx+1],lons[rx][cx+1],
				    lats[rx+1][cx+1],lons[rx+1][cx+1],
				    lats[rx+1][cx],lons[rx+1][cx]);
             }
          }
       }
    }
  }
  /****************************************************************************
  * loadGeo.
  ****************************************************************************/
  private void loadGeo (double geo00, double geo01, double geo10, double geo11,
          		int binSize, double[][] geos) {
    // Corners.
    geos[0][0] = geo00;
    geos[0][binSize] = geo01;
    geos[binSize][0] = geo10;
    geos[binSize][binSize] = geo11;
    if (binSize > 1) {
      // Top.
      double geoJump = (geos[0][binSize] - geos[0][0]) / binSize;
      for (int cx = 1; cx < binSize; cx++) {
         geos[0][cx] = geos[0][0] + (geoJump * cx);
      }
      // Bottom.
      geoJump = (geos[binSize][binSize] - geos[binSize][0]) / binSize;
      for (int cx = 1; cx < binSize; cx++) {
         geos[binSize][cx] = geos[binSize][0] + (geoJump * cx);
      }
      // Middle.
      for (int cx = 0; cx <= binSize; cx++) {
         geoJump = (geos[binSize][cx] - geos[0][cx]) / binSize;
         for (int rx = 1; rx < binSize; rx++) {
            geos[rx][cx] = geos[0][cx] + (geoJump * rx);
         }
      }
    }
  }
}
