/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
* 
*  History:
* 
*  19-Dec-04, J Love, GST	Original version.
*  24-Feb-05, J Love, GST	Fixed Classic Night bug.  IR display choices.
*  18-Oct-05, J Love, GST	Geolocation, etc.
*  26-Apr-07, Z Yin, GST	Added automatically display resolution choices.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.lang.reflect.Array;

public class Modis {
  public static final int APID = 64;
  public static final String TRUE_COLOR_RED_BAND = "1";
  public static final String TRUE_COLOR_GREEN_BAND = "4";
  public static final String TRUE_COLOR_BLUE_BAND = "3";
  public static final String INFRARED_BAND = "22";
  public static final int TRUE_COLOR_RED_BAND_INDEX = 0;
  public static final int TRUE_COLOR_GREEN_BAND_INDEX = 3;
  public static final int TRUE_COLOR_BLUE_BAND_INDEX = 2;
  public static final int INFRARED_BAND_INDEX = 23;
  public static String[] dayBandChoices = new String[] {
    "1","2","3","4","5","6","7","8","9","10","11","12","13L","13H","14L","14H",
    "15","16","17","18","19","20","21","22","23","24","25","26","27","28","29",
    "30","31","32","33","34","35","36"
  };
  public static final int[] dayBandIndices = new int[] {
    0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,
    27,28,29,30,31,32,33,34,35,36,37
  };
  public static String[] nightBandChoices = new String[] {
    "20","21","22","23","24","25","26","27","28","29","30","31","32","33",
    "34","35","36"
  };
  public static final int[] nightBandIndices = new int[] {
    21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37
  };
  public static final int[] reflectiveBandIndices = new int[] {
    0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,27
  };
  public static String[] emissiveBandChoices = new String[] {
    "20","21","22","23","24","25","27","28","29","30","31","32","33",
    "34","35","36"
  };
  public static final int[] emissiveBandIndices = new int[] {
    21,22,23,24,25,26,28,29,30,31,32,33,34,35,36,37
  };
  public static final String[] resolutionChoices = new String[] {
    "250 meter","500 meter","1 kilometer","2 kilometer","5 kilometer","10 kilometer","20 kilometer","30 kilometer","50 kilometer"
  };
  public static final String[] resolutionValues = new String[] {
    "250","500","1000","2000","5000","10000","20000","30000","50000"
  };
  public static final String[] allBandResolutionChoices = new String[] {
    "5 km","10 km","20 km","30 km","50 km"
  };
  public static final String[] allBandResolutionValues = new String[] {
    "5000","10000","20000","30000","50000"
  };
  public static final int DAY_BAND_COUNT = dayBandIndices.length;
  public static final int NIGHT_BAND_COUNT = nightBandIndices.length;
  public static final String[] bandNumbers = dayBandChoices;
  public static final String trueColorTip =
    "True-color composite (red component from Band " +
    bandNumbers[TRUE_COLOR_RED_BAND_INDEX] +
    ", green component from Band " +
    bandNumbers[TRUE_COLOR_GREEN_BAND_INDEX] +
    ", and blue component from Band " +
    bandNumbers[TRUE_COLOR_BLUE_BAND_INDEX] +
    ").";
  public static final String infraredTip =
    "Infrared from Band " + bandNumbers[INFRARED_BAND_INDEX] + ".";
  /****************************************************************************
  * resolutionIndex.
  ****************************************************************************/
  public static int resolutionIndex (String choice) {
    int i, index=0;
    for (i=0; i< Array.getLength(Modis.resolutionChoices); i++)
    {
      if (choice.equals(Modis.resolutionChoices[i]))
      {
        index=i;
      }
    }
    return index;
  }
  /****************************************************************************
  * isDayBand.
  ****************************************************************************/
  public static boolean isDayBand (int bandIndex) {
    return Util.oneOf(bandIndex,dayBandIndices);
  }
  /****************************************************************************
  * isNightBand.
  ****************************************************************************/
  public static boolean isNightBand (int bandIndex) {
    return Util.oneOf(bandIndex,nightBandIndices);
  }
  /****************************************************************************
  * isReflectiveBand.
  ****************************************************************************/
  public static boolean isReflectiveBand (int bandIndex) {
    return Util.oneOf(bandIndex,reflectiveBandIndices);
  }
  /****************************************************************************
  * isEmissiveBand.
  ****************************************************************************/
  public static boolean isEmissiveBand (int bandIndex) {
    return Util.oneOf(bandIndex,emissiveBandIndices);
  }
  /****************************************************************************
  * getReflectiveBandIndex.
  ****************************************************************************/
  public static int getReflectiveBandIndex (int bandIndex) {
    return Util.indexOf(bandIndex,reflectiveBandIndices);
  }
  /****************************************************************************
  * getEmissiveBandIndex.
  ****************************************************************************/
  public static int getEmissiveBandIndex (int bandIndex) {
    return Util.indexOf(bandIndex,emissiveBandIndices);
  }
  /****************************************************************************
  * Enhancements.
  ****************************************************************************/
  public static final int DAY_ENHANCEMENT_NONE = 0;
  public static final int DAY_ENHANCEMENT_1 = 1;
  public static final int DAY_ENHANCEMENT_2 = 2;
  public static final String[] dayEnhancementChoices = new String[] {
    "none              ","algorithm 1","algorithm 2"
  };
  public static final String[] dayEnhancementClassNames = new String[] {
    null,
    "gov.nasa.gsfc.aisb.simulcast.util.TrueColorEnhancement1",
    "gov.nasa.gsfc.aisb.simulcast.util.TrueColorEnhancement2"
  };
  public static final int NIGHT_ENHANCEMENT_NONE = 0;
  public static final String[] nightEnhancementChoices = new String[] {
    "none              "
  };
  public static final String[] nightEnhancementClassNames = new String[] {
    null
  };
}
