/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  19-Oct-05, J Love, GST	Original version.
*   2-May-06, J Love, GST	Satellite name/color.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import gov.nasa.gsfc.drl.geo.sgp4.*;
import java.util.*;
import java.awt.*;

public class OrbitHandler {
  private String satelliteName;
  private Color color;
  private LinkedList<OrbitLocation> track = new LinkedList<OrbitLocation>();
  /****************************************************************************
  * OrbitHandler.
  ****************************************************************************/
  public OrbitHandler (String satelliteName, Color color, Orbit orbit, long startTime, long interval) {
    this.satelliteName = satelliteName;
    this.color = color;
    long time = startTime;
    OrbitLocation prev = new OrbitLocation(orbit,time);
    track.addLast(prev);
    time += interval;
    OrbitLocation next = new OrbitLocation(orbit,time);
    track.addLast(next);
    time += interval;
    boolean descending = next.getLat() < prev.getLat();
    prev = next;
    while (true) {
      next = new OrbitLocation(orbit,time);
      if (descending) {
      	if (next.getLat() > prev.getLat()) {
      	  break;
      	}
      } else {
      	if (next.getLat() < prev.getLat()) {
      	  descending = true;
      	}
      }
      track.addLast(next);
      prev = next;
      time += interval;
    }
  }
  /****************************************************************************
  * getSatelliteName.
  ****************************************************************************/
  public String getSatelliteName () {
    return satelliteName;
  }
  /****************************************************************************
  * getColor.
  ****************************************************************************/
  public Color getColor () {
    return color;
  }
  /****************************************************************************
  * getTrack.
  ****************************************************************************/
  public LinkedList<OrbitLocation> getTrack () {
    return track;
  }
  /****************************************************************************
  * getEndTime.
  ****************************************************************************/
  public long getEndTime () {
    OrbitLocation loc = track.getLast();
    return loc.getTime();
  }
  /****************************************************************************
  * update.
  ****************************************************************************/
  public boolean update (long now) {
    Iterator<OrbitLocation> it = track.iterator();
    while (it.hasNext()) {
      OrbitLocation loc = it.next();
      if (loc.getTime() < now) {
      	it.remove();
      } else {
      	break;
      }
    }
    return (track.size() == 0);
  }
}
