/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  26-May-05, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import gov.nasa.gsfc.aisb.simulcast.client.MapLocation;
import gov.nasa.gsfc.drl.geo.sgp4.*;

public class OrbitLocation {
  private long time;
  private MapLocation mapLoc;
  /****************************************************************************
  * OrbitLocation.
  ****************************************************************************/
  public OrbitLocation (Orbit orbit, long time) {
    this.time = time;
    double tsince = (((time - orbit.getTleTime()) / 1000.0) / 60.0);
    ECI eci = orbit.getPosition(tsince);
    CoordGeo geo = eci.toGeo();
    double lat = Math.toDegrees(geo.m_Lat);
    double lon = Math.toDegrees(geo.m_Lon);
    if (lon > 180.0) {
      lon = lon - 360.0;
    }
    mapLoc = new MapLocation(lat,lon);
  }
  /****************************************************************************
  * getTime.
  ****************************************************************************/
  public long getTime () {
    return time;
  }
  /****************************************************************************
  * getLat.
  ****************************************************************************/
  public double getLat () {
    return mapLoc.getLat();
  }
  /****************************************************************************
  * getLon.
  ****************************************************************************/
  public double getLon () {
    return mapLoc.getLon();
  }
}
