/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*  17-Dec-04, J Love, GST	Original version.
*  10-Feb-06, J Love, GST	Appearance tweaking.
*  18-Oct-06, J Love, GST	Visual preferences saving.
*  10-Jan-07, J Love, GST	Don't display value label.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.text.*;
import javax.swing.*;
import javax.swing.event.*;

public class RealSlider extends HPanel {
  private JSlider slider;
  private JLabel valueLabel;
  private DecimalFormat dot2 = new DecimalFormat("0.00");
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * RealSlider.
  ****************************************************************************/
  public RealSlider (double min, double initial, double max, String tip) {
    slider = new JSlider((int)(min*100),(int)(max*100),(int)(initial*100));
    slider.setToolTipText(tip);
    slider.addChangeListener(new SliderChanged());
    valueLabel = new JLabel(dot2.format(initial));
    add(slider);
  }
  /****************************************************************************
  * getValue.
  ****************************************************************************/
  public double getValue () {
    return slider.getValue() / 100.0;
  }
  /****************************************************************************
  * getValueLabel.
  ****************************************************************************/
  public JLabel getValueLabel () {
    return valueLabel;
  }
  /****************************************************************************
  * setValue.
  ****************************************************************************/
  public void setValue (double value) {
    slider.setValue((int)Math.round(100*value));
  }
  /****************************************************************************
  * SliderChanged.
  ****************************************************************************/
  private class SliderChanged implements ChangeListener {
    public void stateChanged (ChangeEvent ce) {
      valueLabel.setText(dot2.format(slider.getValue()/100.0));
      valueLabel.repaint();
    }
  }
}
