/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*  19-Dec-05, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import gov.nasa.gsfc.drl.geo.*;
import java.io.*;

public class ScanBlock implements Block {
  private long scanTime;
  private int solarCode;
  private int scanNumber;
  private double scLat;
  private double scLon;
  private Geo.Extents extents;
  private boolean isNullScan;
  // private boolean geoBlockFollows;	// A GEO block.
  // private boolean bowedBlockFollows;	// A DATA block.
  // private boolean debowedBlockFollows;	// A DATA block.
  /****************************************************************************
  * ScanBlock.
  ****************************************************************************/
  public ScanBlock (/*boolean geoBlockFollows, boolean bowedBlockFollows, boolean debowedBlockFollows, */ Geo.Extents extents) {
    // this.geoBlockFollows = geoBlockFollows;
    // this.bowedBlockFollows = bowedBlockFollows;
    // this.debowedBlockFollows = debowedBlockFollows;
    this.extents = extents;
  }
  public ScanBlock (DataInput in) throws Exception {
    read(in);
  }
  /****************************************************************************
  * read.
  ****************************************************************************/
  public void read (DataInput in) throws Exception {
    scanTime = in.readLong();
    solarCode = in.readInt();
    scanNumber = in.readInt();
    scLat = in.readDouble();
    scLon = in.readDouble();
    if (extents == null) {
      extents = new Geo.Extents(in);
    } else {
      extents.read(in);
    }
    isNullScan = in.readBoolean();
    // geoBlockFollows = in.readBoolean();
    // bowedBlockFollows = in.readBoolean();
    // debowedBlockFollows = in.readBoolean();
  }
  /****************************************************************************
  * getScanTime.
  ****************************************************************************/
  public long getScanTime () {
    return scanTime;
  }
  /****************************************************************************
  * getSolarCode.
  ****************************************************************************/
  public int getSolarCode () {
    return solarCode;
  }
  /****************************************************************************
  * getScanNumber.
  ****************************************************************************/
  public int getScanNumber () {
    return scanNumber;
  }
  /****************************************************************************
  * getSpacecraftLat.
  ****************************************************************************/
  public double getSpacecraftLat () {
    return scLat;
  }
  /****************************************************************************
  * getSpacecraftLon.
  ****************************************************************************/
  public double getSpacecraftLon () {
    return scLon;
  }
  /****************************************************************************
  * getExtents.
  ****************************************************************************/
  public Geo.Extents getExtents () {
    return extents;
  }
  /****************************************************************************
  * isNullScan.
  ****************************************************************************/
  public boolean isNullScan () {
    return isNullScan;
  }
  /****************************************************************************
  * getGeoBlockFollows.
  ****************************************************************************/
  /*
  public boolean getGeoBlockFollows () {
    return geoBlockFollows;
  }
  */
  /****************************************************************************
  * getBowedBlockFollows.
  ****************************************************************************/
  /*
  public boolean getBowedBlockFollows () {
    return bowedBlockFollows;
  }
  */
  /****************************************************************************
  * getDebowedBlockFollows.
  ****************************************************************************/
  /*
  public boolean getDebowedBlockFollows () {
    return debowedBlockFollows;
  }
  */
  /****************************************************************************
  * set.
  ****************************************************************************/
  public void set (long scanTime, int solarCode, int scanNumber, double scLat, double scLon, boolean isNullScan) {
    this.scanTime = scanTime;
    this.solarCode = solarCode;
    this.scanNumber = scanNumber;
    this.scLat = scLat;
    this.scLon = scLon;
    this.isNullScan = isNullScan;
  }
  /****************************************************************************
  * size.
  ****************************************************************************/
  public int size () {
    return 4 + 8 + 4 + 4 + 8 + 8 + extents.size() + 1;
  }
  /****************************************************************************
  * write.
  ****************************************************************************/
  public void write (DataOutput out) throws Exception {
    out.writeInt(Simulcast.SCAN_BLOCK_CODE);
    out.writeLong(scanTime);
    out.writeInt(solarCode);
    out.writeInt(scanNumber);
    out.writeDouble(scLat);
    out.writeDouble(scLon);
    extents.write(out);
    out.writeBoolean(isNullScan);
    // out.writeBoolean(geoBlockFollows);
    // out.writeBoolean(bowedBlockFollows);
    // out.writeBoolean(debowedBlockFollows);
  }
}
