/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*   6-Dec-04, J Love, GST	Original version.
*   7-Aug-05, J Love, GST	Geolocation, etc.
*  10-Jan-07, J Love, GST	Tweaked appearance.  Dealt with IPv6 by not
*  				using HostPort anymore.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;

public class StationPassParameters implements PassParameters {
  public String passType;
  public GroundStation groundStation;
  public String routerInputAddr;
  public String routerHandlerClassName;
  public String routerOutputAddr;
  public String processorInputAddr;
  public String processorHandlerClassName;
  public String processorOutputAddr;
  public String serverInputAddr;
  /****************************************************************************
  * StationPassParameters.
  ****************************************************************************/
  public StationPassParameters (String passType, GroundStation groundStation,
  				String routerInputAddr, String routerHandlerClassName) {
    this.passType = passType;
    this.groundStation = groundStation;
    this.routerInputAddr = routerInputAddr;
    this.routerHandlerClassName = routerHandlerClassName;
    this.routerOutputAddr = null;
    this.processorInputAddr = null;
    this.processorHandlerClassName = null;
    this.processorOutputAddr = null;
    this.serverInputAddr = null;
  }
  public StationPassParameters (XmlParser parser) throws Exception {
    passType = parser.getElementValue("passType");
    groundStation = new GroundStation(parser.getElement("groundStation"));
    routerInputAddr = parser.getElementValue("routerInputAddress");
    routerHandlerClassName = parser.getElementValue("routerHandlerClassName");
    routerOutputAddr = parser.hasElement("routerOutputAddress") ?
    		       parser.getElementValue("routerOutputAddress") : null;
    processorInputAddr = parser.hasElement("processorInputAddress") ?
    			 parser.getElementValue("processorInputAddress") : null;
    processorHandlerClassName = parser.hasElement("processorHandlerClassName") ?
    				parser.getElementValue("processorHandlerClassName") : null;
    processorOutputAddr = parser.hasElement("processorOutputAddress") ?
    		          parser.getElementValue("processorOutputAddress") : null;
    serverInputAddr = parser.hasElement("serverInputAddress") ?
    		      parser.getElementValue("serverInputAddress") : null;
  }
  /****************************************************************************
  * getGroundStation.
  ****************************************************************************/
  public GroundStation getGroundStation () {
    return groundStation;
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public String toXml () {
    StringBuffer xml = new StringBuffer();
    xml.append("<stationPassParameters>");
    xml.append("<passType>" + passType + "</passType>");
    xml.append(groundStation.toXml());
    xml.append("<routerInputAddress>" + routerInputAddr + "</routerInputAddress>");
    xml.append("<routerHandlerClassName>" + routerHandlerClassName + "</routerHandlerClassName>");
    if (routerOutputAddr != null) {
      xml.append("<routerOutputAddress>" + routerOutputAddr + "</routerOutputAddress>");
    }
    if (processorInputAddr != null) {
      xml.append("<processorInputAddress>" + processorInputAddr + "</processorInputAddress>");
    }
    if (processorHandlerClassName != null) {
      xml.append("<processorHandlerClassName>" + processorHandlerClassName + "</processorHandlerClassName>");
    }
    if (processorOutputAddr != null) {
      xml.append("<processorOutputAddress>" + processorOutputAddr + "</processorOutputAddress>");
    }
    if (serverInputAddr != null) {
      xml.append("<serverInputAddress>" + serverInputAddr + "</serverInputAddress>");
    }
    xml.append("</stationPassParameters>");
    return xml.toString();
  }
  /****************************************************************************
  * toText.
  ****************************************************************************/
  public String toText () {
    StringBuffer text = new StringBuffer();
    text.append("passType: " + passType.toUpperCase());
    text.append(", groundStation: " + groundStation.toString());
    text.append(", routerInputAddress: " + routerInputAddr);
    text.append(", routerHandlerClassName: " + routerHandlerClassName);
    if (routerOutputAddr != null) {
      text.append(", routerOutputAddress: " + routerOutputAddr);
    }
    if (processorInputAddr != null) {
      text.append(", processorInputAddress: " + processorInputAddr);
    }
    if (processorHandlerClassName != null) {
      text.append(", processorHandlerClassName: " + processorHandlerClassName);
    }
    if (processorOutputAddr != null) {
      text.append(", processorOutputAddress: " + processorOutputAddr);
    }
    if (serverInputAddr != null) {
      text.append(", serverInputAddress: " + serverInputAddr);
    }
    return text.toString();
  }
  /****************************************************************************
  * toLines.
  * Assumed to be invoked only after all parameters have been set.
  ****************************************************************************/
  public String[] toLines () {
    return new String[] {
      "PassType: " + passType.toUpperCase(),
      "GroundStation: " + groundStation.toString(),
      "RouterInputAddress: " + routerInputAddr,
      "RouterHandlerClassName: " + routerHandlerClassName,
      "RouterOutputAddress: " + routerOutputAddr,
      "ProcessorInputAddress: " + processorInputAddr,
      "ProcessorHandlerClassName: " + processorHandlerClassName,
      "ProcessorOutputAddress: " + processorOutputAddr,
      "ServerInputAddress: " + serverInputAddr
    };
  }
}
