/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*   7-Jan-05, J Love, GST	Original version.
*   6-Aug-06, J Love, GST	Java 1.5 (generics).
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.util.*;
// import java.lang.reflect.*;

public class StringMap2<E> implements Iterable {
  private LinkedList<String> strings;
  private LinkedList<E> objects;
  /****************************************************************************
  * StringMap.
  ****************************************************************************/
  public StringMap2 () {
    strings = new LinkedList<String>();
    objects = new LinkedList<E>();
  }
  /****************************************************************************
  * putLast.
  ****************************************************************************/
  public void putLast (String string, E object) {
    put(string,object);
  }
  /****************************************************************************
  * putFirst.
  ****************************************************************************/
  public void putFirst (String string, E object) {
    if (!contains(string)) {
      strings.addFirst(string);
      objects.addFirst(object);
    } else {
      objects.set(indexOf(string),object);
    }
  }
  /****************************************************************************
  * put.
  ****************************************************************************/
  public void put (String string, E object) {
    if (!contains(string)) {
      strings.addLast(string);
      objects.addLast(object);
    } else {
      objects.set(indexOf(string),object);
    }
  }
  /****************************************************************************
  * contains.
  ****************************************************************************/
  public boolean contains (String string) {
    for (int i = 0; i < strings.size(); i++) {
       if (string.equals(strings.get(i))) {
       	 return true;
       }
    }
    return false;
  }
  /****************************************************************************
  * getLast.
  ****************************************************************************/
  public E getLast () {
    return objects.getLast();
  }
  /****************************************************************************
  * get.
  ****************************************************************************/
  public E get (int index) {
    return objects.get(index);
  }
  public E get (String string) {
    for (int i = 0; i < strings.size(); i++) {
       if (string.equals((String)strings.get(i))) {
       	 return objects.get(i);
       }
    }
    return null;
  }
  /****************************************************************************
  * getStrings.
  ****************************************************************************/
  public String[] getStrings () {
    return (String[]) strings.toArray(new String[strings.size()]);
  }
  /****************************************************************************
  * getObjects.
  ****************************************************************************/
  @SuppressWarnings("unchecked")
  public E[] getObjects (E[] e) {
    return (E[]) objects.toArray(e);
  }
  /****************************************************************************
  * indexOf.
  ****************************************************************************/
  public int indexOf (String string) {
    for (int i = 0; i < strings.size(); i++) {
       if (string.equals(strings.get(i))) {
       	 return i;
       }
    }
    return -1;
  }
  /****************************************************************************
  * iterator.
  ****************************************************************************/
  public Iterator<E> iterator () {
    return objects.iterator();
  }
  /****************************************************************************
  * remove.
  ****************************************************************************/
  public E remove (int index) {
    strings.remove(index);
    return objects.remove(index);
  }
  public E remove (String string) {
    for (int i = 0; i < strings.size(); i++) {
       if (string.equals(strings.get(i))) {
         E e = objects.get(i);
	 strings.remove(i);
	 objects.remove(i);
	 return e;
       }
    }
    return null;
  }
  /****************************************************************************
  * size.
  ****************************************************************************/
  public int size () {
    return strings.size();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    strings.clear();
    objects.clear();
  }
}
