/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*  12-Jan-07, J Love, GST	Original version.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;

public class TimeoutMonitor implements Runnable {
  private long timeoutInterval;
  private TimeoutMonitorListener listener;
  private boolean timedOut = false;
  private long inputAt = System.currentTimeMillis();
  private Thread thread = null;
  /****************************************************************************
  * InputMonitor.
  ****************************************************************************/
  public TimeoutMonitor (long timeoutInterval, TimeoutMonitorListener listener) {
    this.timeoutInterval = timeoutInterval;
    this.listener = listener;
  }
  /****************************************************************************
  * inputReceived.
  ****************************************************************************/
  public void inputReceived () {
    inputAt = System.currentTimeMillis();
  }
  /****************************************************************************
  * isTimedOut.
  ****************************************************************************/
  public boolean isTimedOut () {
    return timedOut;
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start () {
    if (timeoutInterval > 0) {
      thread = Util.startThread(this,true);
    }
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    try {
      check();
    } catch (Exception e) {
	
    }
  }
  private void check () throws Exception {
    while (true) {
      if (System.currentTimeMillis() - inputAt > timeoutInterval) {
	timedOut = true;
	listener.timedOut();
	return;
      }
      Thread.sleep(1000);	// Don't use Util.sleep().  It catches the
				// InterruptedException...
    }
  }
  /****************************************************************************
  * stop.
  ****************************************************************************/
  public void stop () {
    if (thread != null) {
      thread.interrupt();
    }
  }
}
