/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*   6-Dec-04, J Love, GST	Original version.
*  11-Nov-05, J Love, GST	Upstream server handling changes.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;

public class UpstreamPassParameters implements PassParameters {
  public HostPort upstreamServerAddr;
  public String passName;
  /****************************************************************************
  * UpstreamPassParameters.
  ****************************************************************************/
  public UpstreamPassParameters (HostPort upstreamServerAddr, String passName) {
    this.upstreamServerAddr = upstreamServerAddr;
    this.passName = passName;
  }
  public UpstreamPassParameters (XmlParser parser) throws Exception {
    upstreamServerAddr = new HostPort(parser.getElementValue("upstreamServerAddress"));
    passName = parser.getElementValue("passName");
  }
  /****************************************************************************
  * getPassName.
  ****************************************************************************/
  public String getPassName () {
    return passName;
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public String toXml () {
    StringBuffer xml = new StringBuffer();
    xml.append("<upstreamPassParameters>");
    xml.append("<upstreamServerAddress>" + upstreamServerAddr.toString() + "</upstreamServerAddress>");
    xml.append("<passName>" + passName + "</passName>");
    xml.append("</upstreamPassParameters>");
    return xml.toString();
  }
  /****************************************************************************
  * toText.
  ****************************************************************************/
  public String toText () {
    StringBuffer text = new StringBuffer();
    text.append("upstreamServerAddress: " + upstreamServerAddr.toString());
    text.append(", passName: " + passName);
    return text.toString();
  }
  /****************************************************************************
  * toLines.
  * Assumed to be invoked only after all parameters have been set.
  ****************************************************************************/
  public String[] toLines () {
    return new String[] {
      "PassName: " + passName + " (upstream)",
      "UpstreamServerAddress: " + upstreamServerAddr.toString()
    };
  }
}
