/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
* 
*  History:
* 
*   8-Sep-04, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;

public class Viirs {
  public static final int APID = 0;
  public static final int TRUE_COLOR_RED_BAND_INDEX = 17;
  public static final int TRUE_COLOR_GREEN_BAND_INDEX = 3;
  public static final int TRUE_COLOR_BLUE_BAND_INDEX = 2;
  public static final int INFRARED_BAND_INDEX = 20;
  public static String[] dayBandChoices = new String[] {
    "M1","M2","M3","M4","M5","M6","M7","M8","M9","M10","M11","M12","M13","M14",
    "M15","M16","DNB","I1","I2","I3","I4","I5"
  };
  public static final int[] dayBandIndices = new int[] {
    0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21
  };
  public static String[] nightBandChoices = new String[] {
    "M12","M13","M14","M15","M16","I4","I5"
  };
  public static final int[] nightBandIndices = new int[] {
    11,12,13,14,15,20,21
  };
  public static final int[] reflectiveBandIndices = new int[] {
    0,1,2,3,4,5,6,7,8,9,10,16,17,18,19
  };
  public static final int[] emissiveBandIndices = new int[] {
    11,12,13,14,15,20,21
  };
  public static final String[] resolutionChoices = new String[] {
    "350 meter","700 meter","1.4 kilometer","2.8 kilometer","4.2 kilometer",
    "8.4 kilometer","16.8 kilometer","33.6 kilometer"
  };
  public static final String[] resolutionValues = new String[] {
    "350","700","1400","2800","4200","8400","16800","33600"
  };
  public static final int DAY_BAND_COUNT = dayBandIndices.length;
  public static final int NIGHT_BAND_COUNT = nightBandIndices.length;
  /****************************************************************************
  * isDayBand.
  ****************************************************************************/
  public static boolean isDayBand (int bandIndex) {
    return Util.oneOf(bandIndex,dayBandIndices);
  }
  /****************************************************************************
  * isNightBand.
  ****************************************************************************/
  public static boolean isNightBand (int bandIndex) {
    return Util.oneOf(bandIndex,nightBandIndices);
  }
  /****************************************************************************
  * isReflectiveBand.
  ****************************************************************************/
  public static boolean isReflectiveBand (int bandIndex) {
    return Util.oneOf(bandIndex,reflectiveBandIndices);
  }
  /****************************************************************************
  * isEmissiveBand.
  ****************************************************************************/
  public static boolean isEmissiveBand (int bandIndex) {
    return Util.oneOf(bandIndex,emissiveBandIndices);
  }
  /****************************************************************************
  * getReflectiveBandIndex.
  ****************************************************************************/
  public static int getReflectiveBandIndex (int bandIndex) {
    return Util.indexOf(bandIndex,reflectiveBandIndices);
  }
  /****************************************************************************
  * getEmissiveBandIndex.
  ****************************************************************************/
  public static int getEmissiveBandIndex (int bandIndex) {
    return Util.indexOf(bandIndex,emissiveBandIndices);
  }
}
