/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  13-Jul-06, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import java.awt.*;
import java.awt.image.*;
import java.util.*;

public class AscendingInstrumentView extends InstrumentView {
  private int width;
  private int imageType;
  private int nRows = 0;
  private LinkedList<Image> images = new LinkedList<Image>();
  private BufferedImage tailImage = null;
  private static final int NUM_SUBIMAGE_ROWS = 100;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * AscendingImageHandler.
  ****************************************************************************/
  public AscendingInstrumentView (int width, boolean isColor, String tip) {
    this.width = width;
    this.imageType = (isColor ? BufferedImage.TYPE_INT_RGB :
    				BufferedImage.TYPE_BYTE_GRAY);
    Dimension imageDims = new Dimension(width,nRows);
    setMinimumSize(imageDims);
    setPreferredSize(imageDims);
    setMaximumSize(imageDims);
    setToolTipText(tip);
  }
  /****************************************************************************
  * getBufferedImage.
  * Returning a TYPE_BYTE_GRAY BufferedImage doesn't paint properly in a
  * JPanel so always return a TYPE_INT_RGB.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    BufferedImage bi = new BufferedImage(width,nRows,BufferedImage.TYPE_INT_RGB);
    Graphics g = bi.getGraphics();
    paint(g);
    return bi;
  }
  /****************************************************************************
  * addRow.
  ****************************************************************************/
  public void addRow (Color[] colors) {
      if (nRows % NUM_SUBIMAGE_ROWS == 0) {
      	if (tailImage != null) {
      	  images.addLast(tailImage);
      	}
	tailImage = new BufferedImage(width,NUM_SUBIMAGE_ROWS,imageType);
      }
      Graphics g = tailImage.getGraphics();
      int row = (NUM_SUBIMAGE_ROWS - 1) - (nRows % NUM_SUBIMAGE_ROWS);
      for (int col = 0; col < width; col++) {
         g.setColor(colors[col]);
         g.fillRect((width-1)-col,row,1,1);
      }
      nRows++;
      Dimension imageDims = new Dimension(width,nRows);
      setMinimumSize(imageDims);
      setPreferredSize(imageDims);
      setMaximumSize(imageDims);
      revalidate();
      repaint();
  }
  /****************************************************************************
  * paint.
  ****************************************************************************/
  public void paint (Graphics g) {
    int row = nRows - 1;
    Iterator it = images.iterator();
    while (it.hasNext()) {
      BufferedImage image = (BufferedImage) it.next();
      g.drawImage(image,0,row - (NUM_SUBIMAGE_ROWS - 1),width,row + 1,0,0,width,NUM_SUBIMAGE_ROWS,null);
      row -= NUM_SUBIMAGE_ROWS;
    }
    int nPartialRows = row + 1;
    g.drawImage(tailImage,0,0,width,nPartialRows,
    		0,NUM_SUBIMAGE_ROWS - nPartialRows,width,NUM_SUBIMAGE_ROWS,null);
  }
}
