/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  27-Dec-04, J Love, GST	Original version.
*   8-Feb-06, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import java.awt.*;
import java.awt.image.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

public class AscendingLimitedInstrumentView extends InstrumentView {
  private int width;
  private int imageType;
  private LinkedList<Image> images = new LinkedList<Image>();
  private int maxRows;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * AscendingLimitedImageHandler.
  ****************************************************************************/
  public AscendingLimitedInstrumentView (int width, boolean isColor,
  					String tip, JViewport viewport) {
    this.width = width;
    this.imageType = (isColor ? BufferedImage.TYPE_INT_RGB :
    				BufferedImage.TYPE_BYTE_GRAY);
    Dimension imageDims = new Dimension(width,0);
    setMinimumSize(imageDims);
    setPreferredSize(imageDims);
    setMaximumSize(imageDims);
    setToolTipText(tip);
    maxRows = viewport.getHeight();
    viewport.addChangeListener(new ViewportChanged());
  }
  /****************************************************************************
  * getBufferedImage.
  * Returning a TYPE_BYTE_GRAY BufferedImage doesn't paint properly in a
  * JPanel so always return a TYPE_INT_RGB.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    BufferedImage bi = new BufferedImage(width,images.size(),BufferedImage.TYPE_INT_RGB);
    Graphics g = bi.getGraphics();
    paint(g);
    return bi;
  }
  /****************************************************************************
  * addRow.
  ****************************************************************************/
  public void addRow (Color[] colors) {
      Image image = new BufferedImage(width,1,imageType);
      Graphics g = image.getGraphics();
      for (int col = 0; col < width; col++) {
         g.setColor(colors[col]);
         g.fillRect((width-1)-col,0,1,1);
      }
      images.addLast(image);
      while (images.size() > maxRows) {
      	images.removeFirst();
      }
      Dimension imageDims = new Dimension(width,images.size());
      setMinimumSize(imageDims);
      setPreferredSize(imageDims);
      setMaximumSize(imageDims);
      revalidate();
      repaint();
  }
  /****************************************************************************
  * paint.
  ****************************************************************************/
  public void paint (Graphics g) {
    int row = images.size() - 1;
    Iterator it = images.iterator();
    while (it.hasNext()) {
      BufferedImage image = (BufferedImage) it.next();
      g.drawImage(image,0,row,width,row+1,0,0,width,1,null);
      row--;
    }
  }
  /****************************************************************************
  * ViewportChanged.
  ****************************************************************************/
  private class ViewportChanged implements ChangeListener {
    public void stateChanged (ChangeEvent ce) {
      JViewport viewport = (JViewport) ce.getSource();
      maxRows = viewport.getHeight();
      while (images.size() > maxRows) {
      	images.removeFirst();
      }
      Dimension imageDims = new Dimension(width,images.size());
      setMinimumSize(imageDims);
      setPreferredSize(imageDims);
      setMaximumSize(imageDims);
      revalidate();
      repaint();
    }
  }
}
