/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  18-Oct-06, J Love, GST	Original version.
*   2-Jan-07, J Love, GST	Handle passes being removed from server.
*  30-Jan-07, J Love, GST	Tweaked layout.
*  25-Apr-07, Z Yin, GST	Make reolution change intelligently.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import javax.swing.*;

public class InstrumentLayout extends VPanel implements Layout {
  private StringMap scrollers = new StringMap();
  private VPanel scrollerPanel;
  private PlayerManager playerManager;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * InstrumentLayout.
  ****************************************************************************/
  public InstrumentLayout () {
    scrollerPanel = new VPanel(new BlackPanel());
    playerManager = new PlayerManager(this);
    add(scrollerPanel/*,BorderLayout.CENTER*/);
    add(GUtil.VSpace(5));
    add(playerManager);
  }
  /****************************************************************************
  * loadPass.
  * Note that loop-play is considered a type of auto-play.
  ****************************************************************************/
  public void loadPass (String passName, int platformCode, int instrumentCode,
			int directionCode, int startSolarCode, boolean autoPlay) {
    Player player = playerManager.getPlayer(passName);
    if (player == null) {
      // System.out.println("\n create instrument player.");
      player = new Player(passName,platformCode,instrumentCode,directionCode,startSolarCode,this,playerManager);
      playerManager.addPlayer(player);
      InstrumentScroller scroller = new InstrumentScroller();
      scrollers.put(passName,scroller);
      loadScroller(scroller);
    } else {
      playerManager.setSelectedPlayer(passName);
    }
    if (autoPlay) {
      player.play();
    }
  }
  /****************************************************************************
  * doHeader.
  ****************************************************************************/
  public void doHeader (String passName, HeaderBlock headerBlock) throws Exception {
    InstrumentScroller scroller = (InstrumentScroller) scrollers.get(passName);
    scroller.setView(headerBlock.getDirectionCode(),headerBlock.getCoarsestBlockColCount());
  }
  /****************************************************************************
  * doScan.
  ****************************************************************************/
  public void doScan (String passName, GeoBlock geoBlock, Color[][][] colors) throws Exception {
    InstrumentScroller scroller = (InstrumentScroller) scrollers.get(passName);
    scroller.addRows(colors[0]);
  }
  /****************************************************************************
  * doFooter.
  ****************************************************************************/
  public void doFooter (String passName) {
    
  }
  /****************************************************************************
  * playerSelected.
  ****************************************************************************/
  public void playerSelected (String passName) {
    InstrumentScroller scroller = (InstrumentScroller) scrollers.get(passName);
    loadScroller(scroller);
  }
  /****************************************************************************
  * playerRemoved.
  ****************************************************************************/
  public void playerRemoved (String removedPassName, String selectedPassName) {
    scrollers.remove(removedPassName);
    if (selectedPassName != null) {
      loadScroller((InstrumentScroller)scrollers.get(selectedPassName));
    } else {
      loadScroller(null);
    }
  }
  /****************************************************************************
  * getVisualSelector.
  ****************************************************************************/
  public VisualSelector getVisualSelector () {
    return playerManager.getVisualSelector();
  }
  /****************************************************************************
  * getPlayerManager.
  ****************************************************************************/
  public PlayerManager getPlayerManager () {
    return playerManager;
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    playerManager.closeAll();
  }
  /****************************************************************************
  * loadScroller.
  ****************************************************************************/
  private void loadScroller (InstrumentScroller scroller) {
    scrollerPanel.removeAll();
    if (scroller != null) {
      scrollerPanel.add(scroller);
    } else {
      scrollerPanel.add(new BlackPanel());
    }
    revalidate();
    repaint();
  }
  /****************************************************************************
  * passRemoved.
  ****************************************************************************/
  public void passRemoved (String name) {
    playerManager.passRemoved(name);
  }
  /****************************************************************************
  * BlackPanel.
  ****************************************************************************/
  private class BlackPanel extends JPanel {
    private static final long serialVersionUID = 1L;
    public BlackPanel () {
      setPreferredSize(getMaximumSize());
    }
    public void paint (Graphics g) {
      g.setColor(Color.black);
      g.fillRect(0,0,getWidth(),getHeight());
    }
  }
}
