/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  17-Oct-06, J Love, GST	Original version.
*  30-Jan-07, J Love, GST	Tweaked layout.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import javax.swing.*;

public class InstrumentScroller extends VPanel {
  private InstrumentView view = null;
  private JScrollPane scrollPane;
  private JScrollBar vScrollBar;
  private JScrollBar hScrollBar;
  private int directionCode;
  private boolean limited = Viewer.optionsScreen.limitViewCheckBox.isSelected();
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * InstrumentScroller.
  ****************************************************************************/
  public InstrumentScroller () {
    scrollPane = new JScrollPane((limited ? JScrollPane.VERTICAL_SCROLLBAR_NEVER : JScrollPane.VERTICAL_SCROLLBAR_ALWAYS),
    				 JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    scrollPane.setPreferredSize(scrollPane.getMaximumSize());
    vScrollBar = scrollPane.getVerticalScrollBar();
    hScrollBar = scrollPane.getHorizontalScrollBar();
    add(scrollPane);
  }
  /****************************************************************************
  * setView.
  ****************************************************************************/
  public void setView (int newDirectionCode, int width) {
    GUtil.invokeAndWait(new SetView(newDirectionCode,width));
    GUtil.invokeAndWait(new DoCenter());
  }
  private class SetView implements Runnable {
    private int newDirectionCode;
    private int width;
    public SetView (int newDirectionCode, int width) {
      this.newDirectionCode = newDirectionCode;
      this.width = width;
    }
    public void run () {
      directionCode = newDirectionCode;
      switch (directionCode) {
        case Simulcast.ASCENDING_PASS_CODE: {
	  if (limited) {
	    view = new AscendingLimitedInstrumentView(width,true,null,scrollPane.getViewport());
	  } else {
	    view = new AscendingInstrumentView(width,true,null);
	  }
	  break;
        }
        case Simulcast.DESCENDING_PASS_CODE: {
	  if (limited) {
	    view = new DescendingLimitedInstrumentView(width,true,null,scrollPane.getViewport());
	  } else {
	    view = new DescendingInstrumentView(width,true,null);
	  }
	  break;
        }
      }
      scrollPane.setViewportView(new MarginPanel(0,-1,view,-1,-1));
      revalidate();
      repaint();
    }
  }
  /****************************************************************************
  * addRows.
  ****************************************************************************/
  public void addRows (Color[][] colors) {
    GUtil.invokeAndWait(new AddRows(colors));
    if (Viewer.optionsScreen.autoScrollCheckBox.isSelected()) {
      if (!limited) {
	GUtil.invokeAndWait(new DoScroll());
      }
    }
  }
  private class AddRows implements Runnable {
    private Color[][] colors;
    public AddRows (Color[][] colors) {
      this.colors = colors;
    }
    public void run () {
      for (int row = 0; row < colors.length; row++) {
         view.addRow(colors[row]);
      }
      revalidate();
      repaint();
    }
  }
  /****************************************************************************
  * DoScroll.
  ****************************************************************************/
  private class DoScroll implements Runnable {
    public void run () {
      switch (directionCode) {
        case Simulcast.ASCENDING_PASS_CODE: {
          vScrollBar.setValue(vScrollBar.getMinimum());
          break;
        }
        case Simulcast.DESCENDING_PASS_CODE: {
          vScrollBar.setValue(vScrollBar.getMaximum());
          break;
        }
      }
      revalidate();
      repaint();
    }
  }
  /****************************************************************************
  * DoCenter.
  ****************************************************************************/
  private class DoCenter implements Runnable {
    public void run () {
      hScrollBar.setValue(((hScrollBar.getMaximum() - hScrollBar.getVisibleAmount()) - hScrollBar.getMinimum()) / 2);
      revalidate();
      repaint();
    }
  }
}
