/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  18-Oct-06, J Love, GST	Original version.
*   2-Jan-07, J Love, GST	Handle passes being removed from server.
*   7-Feb-07, J Love, GST	Tweaked layout.
*  15-Mar-07, Z Yin,  GST       Added getPlayerManager for displaying image 
*                               resolution choices equal or larger
*                               then data resolution.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;

public class MapLayout extends VPanel implements Layout {
  protected MapHandler map;
  private PlayerManager playerManager;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * MapLayout.
  ****************************************************************************/
  public MapLayout (int projection) {
    setLayout(new BorderLayout());
    map = new MapHandler(projection);
    playerManager = new PlayerManager(this);
    add(new VPanel(map,GUtil.VSpace(5),playerManager));
  }
  /****************************************************************************
  * setTLEs.
  ****************************************************************************/
  public void setTLEs (String[] aquaTLE, String[] terraTLE) throws Exception {
    map.setTLEs(aquaTLE,terraTLE);
  }
  /****************************************************************************
  * setGroundStations.
  ****************************************************************************/
  public void setGroundStations (GroundStations groundStations) {
    map.setGroundStations(groundStations);
  }
  /****************************************************************************
  * getAreaFilter.
  ****************************************************************************/
  public String getAreaFilter () {
    return map.getAreaFilter();
  }
  /****************************************************************************
  * getPlayerManager.
  ****************************************************************************/
  public PlayerManager getPlayerManager () {
    return playerManager;
  }
  /****************************************************************************
  * optionChanged.
  ****************************************************************************/
  public void optionChanged () {
    map.repaint();
  }
  /****************************************************************************
  * loadPass.
  * Note that loop-play is considered a type of auto-play.
  ****************************************************************************/
  public void loadPass (String passName, int platformCode, int instrumentCode,
			int directionCode, int startSolarCode, boolean autoPlay) {
    Player player = playerManager.getPlayer(passName);
    if (player == null) {
      // System.out.println("\n create map player.");
      player = new Player(passName,platformCode,instrumentCode,directionCode,startSolarCode,this,playerManager);
      playerManager.addPlayer(player);
      map.addView(passName);
    } else {
      playerManager.setSelectedPlayer(passName);
    }
    if (autoPlay) {
      if (Viewer.optionsScreen.eraseOnAutoPlayCheckBox.isSelected()) {
	map.clear();
      }
      player.play();
    }
  }
  /****************************************************************************
  * doHeader.
  ****************************************************************************/
  public void doHeader (String passName, HeaderBlock headerBlock) throws Exception {
    map.doHeader(passName,headerBlock);
  }
  /****************************************************************************
  * doScan.
  ****************************************************************************/
  public void doScan (String passName, GeoBlock geoBlock, Color[][][] colors) throws Exception {
    map.doScan(passName,geoBlock,colors[0]);
  }
  /****************************************************************************
  * doFooter.
  ****************************************************************************/
  public void doFooter (String passName) {
    map.doFooter(passName);
  }
  /****************************************************************************
  * playerSelected.
  ****************************************************************************/
  public void playerSelected (String passName) {
    map.passSelected(passName);
  }
  /****************************************************************************
  * playerRemoved.
  ****************************************************************************/
  public void playerRemoved (String removedPassName, String selectedPassName) {
    map.removeView(removedPassName);
    if (selectedPassName != null) {
      playerSelected(selectedPassName);
    }
  }
  /****************************************************************************
  * getVisualSelector.
  ****************************************************************************/
  public VisualSelector getVisualSelector () {
    return playerManager.getVisualSelector();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    playerManager.closeAll();
  }
  /****************************************************************************
  * passRemoved.
  ****************************************************************************/
  public void passRemoved (String name) {
    playerManager.passRemoved(name);
  }
}
