/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  29-Sep-06, J Love, GST	Original version.
*  10-Feb-07, J Love, GST	Start auto-play on the event thread.
*  26-Feb-07, J Love, GST	More tool tips.
*  15-Mar-07, Z Yin,  GST       Display image resolution choices equal or larger
*                               then data resolution.
*   2-Apr-07, Z Yin,  GST       Display more meaningful message on pass manager. 
*                               Change red message color to black.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.lang.reflect.Array;
import java.io.*;
import java.net.Socket;
import javax.swing.JComponent;

public class PassHandler extends VPanel {
  private int i;
  private int index;
  private int defaultIndex;
  private int ResolutionNumber = Array.getLength(Modis.resolutionChoices);
  private int resolution;
  private int BiggestInResolution;
  private int SmallestIndex;
  private Visual visual;
  private HPanel visualPanel;
  private int infraredBandIndex;
  private int grayBandIndex;
  private int redBandIndex;
  private int greenBandIndex;
  private int blueBandIndex;
  private String passName;
  private String stationId;
  private long startedAt = Simulcast.UNKNOWN;
  private long endedAt = Simulcast.UNKNOWN;
  private int platformCode = Simulcast.UNKNOWN;
  private int instrumentCode = Simulcast.UNKNOWN;
  private int startSolarCode = Simulcast.UNKNOWN;
  private int directionCode = Simulcast.UNKNOWN;
  private CustomLabel nameField;
  private CustomLabel groundStationField;
  private CustomLabel platformField;
  private CustomLabel instrumentField;
  private CustomLabel startedField;
  private CustomLabel endedField;
  private CustomLabel directionField;
  private CustomLabel solarField;
  private CustomLabel durationField;
  private CustomLabel scansField;
  private CustomLabel errorField;
  private JCheckBox selectCheckBox;
  private JFrame detailsFrame = null;
  private VPanel detailsPanel = new VPanel();
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PassHandler.
  ****************************************************************************/
  public PassHandler (XmlParser parser) throws Exception {
    setBorder(new EmptyBorder(5,5,0,5));
    selectCheckBox = new JCheckBox("");
    selectCheckBox.setToolTipText("If checked, the pass is added to the next selected layout (Map or Instrument).");
    selectCheckBox.setMargin(new Insets(1,0,0,0));
    selectCheckBox.setAlignmentY(TOP_ALIGNMENT);
    passName = parser.getElementValue("name");
    stationId = passName.split("\\.")[0];
    nameField = new CustomLabel(passName,160);
    nameField.setToolTipText("Name of the pass.");
    groundStationField = new CustomLabel(" ",160);
    groundStationField.setToolTipText("Ground station at which the pass was received.");
    platformField = new CustomLabel(" ",70);
    platformField.setToolTipText("Satellite.");
    instrumentField = new CustomLabel(" ",70);
    instrumentField.setToolTipText("Instrument.");
    startedField = new CustomLabel(" ",200);
    startedField.setToolTipText("Start date/time of the pass.");
    endedField = new CustomLabel(" ",200);
    endedField.setToolTipText("Stop date/time of the pass.");
    directionField = new CustomLabel(" ",100);
    directionField.setToolTipText("Direction of the pass.");
    solarField = new CustomLabel(" ",100);
    solarField.setToolTipText("Instrument mode (day or night).");
    durationField = new CustomLabel(" ",90);
    durationField.setToolTipText("Duration of the pass (minutes:seconds).");
    scansField = new CustomLabel(" ",90);
    scansField.setToolTipText("Number of scans.");
    errorField = new CustomLabel("");
    errorField.setToolTipText("Error during pass (click Details).");
    errorField.setAlignmentX(LEFT_ALIGNMENT);
    HPanel p1 = new HPanel(new VPanel(nameField,groundStationField),GUtil.HSpace(15),new VPanel(platformField,instrumentField),GUtil.HSpace(15),new VPanel(startedField,endedField),GUtil.HSpace(15),new VPanel(directionField,solarField),GUtil.HSpace(15),new VPanel(durationField,scansField));
    p1.setAlignmentX(LEFT_ALIGNMENT);
    VPanel p2 = new VPanel(p1,errorField);
    p2.setAlignmentY(TOP_ALIGNMENT);
    JButton mapButton = new JButton("Map");
    mapButton.setToolTipText("Switches to the Map Layout and loads this pass.");
    mapButton.setMargin(new Insets(0,0,0,0));
    mapButton.addActionListener(new MapButtonClicked());
    JButton instrumentButton = new JButton("Instrument");
    instrumentButton.setToolTipText("Switches to the Instrument Layout and loads this pass.");
    instrumentButton.setMargin(new Insets(0,0,0,0));
    instrumentButton.addActionListener(new InstrumentButtonClicked());
    JButton detailsButton = new JButton("Details");
    detailsButton.setToolTipText("Displays a window containing details about the pass.");
    detailsButton.setMargin(new Insets(0,0,0,0));
    detailsButton.addActionListener(new DetailsButtonClicked());
    HPanel p3 = new HPanel(new MarginPanel(3,0,mapButton,3,0),GUtil.HSpace(10),new MarginPanel(3,0,instrumentButton,3,0),GUtil.HSpace(10),new MarginPanel(3,0,detailsButton,3,0));
    p3.setAlignmentY(TOP_ALIGNMENT);
    HPanel p0 = new HPanel(GUtil.HSpace(3),selectCheckBox,GUtil.HSpace(5),p2,GUtil.HGlue(),p3,GUtil.HSpace(5));
    p0.setBorder(new LineBorder(Color.black,1));
    add(p0);
    set(parser);
  }
  /****************************************************************************
  * passDetails.
  ****************************************************************************/
  public void passDetails (XmlParser parser) throws Exception {
    set(parser);
  }
  /****************************************************************************
  * passUpdated.
  ****************************************************************************/
  public void passUpdated (XmlParser parser) throws Exception {
    set(parser);
    // Util.startThread(new DoAutoPlay());
    SwingUtilities.invokeLater(new DoAutoPlay());
  }
  private class DoAutoPlay implements Runnable {
    public void run () {
      if (Viewer.optionsScreen.instrumentViewAutoPlayCheckBox.isSelected()) {
      	// Util.sleep(1.0);
        loadInstrument(true,true);
      }
      if (Viewer.optionsScreen.mapViewAutoPlayCheckBox.isSelected()) {
      	// Util.sleep(1.0);
        loadMap(true,true);
      }
    }
  }
  /****************************************************************************
  * passEnded.
  ****************************************************************************/
  public void passEnded (XmlParser parser) throws Exception {
    set(parser);
  }
  /****************************************************************************
  * resolutionIndex.
  ****************************************************************************/
  private int resolutionIndex (int BiggestInResolution)
  {
    switch (BiggestInResolution)
    {
      case 250:
        SmallestIndex=0;
        break;
      case 500 :
        SmallestIndex=1;
        break;
      case 1000:
        SmallestIndex=2;
        break;
      case 2000:
        SmallestIndex=3;
        break;
      case 5000:
        SmallestIndex=4;
        break;
      case 10000:
        SmallestIndex=5;
        break;
      case 20000:
        SmallestIndex=6;
        break;
      case 30000:
        SmallestIndex=7;
        break;
      case 50000:
        SmallestIndex=8;
        break;
    }
    return SmallestIndex;
  }
  /****************************************************************************
  * VisualBand.
  ****************************************************************************/
  public void visualBand (XmlParser parser) throws Exception 
  {
    resolution = 0; 
    BiggestInResolution = 0; 
    SmallestIndex = 0;

    if (Viewer.mapLayoutButton.isSelected())
    {
      visualPanel=Viewer.mapLayout.getVisualSelector().getVisualPanel();
      visual=Viewer.mapLayout.getVisualSelector().getVisual();
    }
    if (Viewer.instrumentLayoutButton.isSelected())
    {
      visualPanel=Viewer.instrumentLayout.getVisualSelector().getVisualPanel();
      visual=Viewer.instrumentLayout.getVisualSelector().getVisual();
    }

    visualPanel.removeAll();

    if (parser.hasElement("more"))
    {
      XmlParser[] parsers = parser.getElement("more").getElement("bands").getElements("band");

      if (visual instanceof ModisAutoDayNightVisual)
      {
        ModisAutoDayNightVisual DV = (ModisAutoDayNightVisual)visual;
        for (i = 0; i < parsers.length; i++)
        {
          index=Integer.parseInt(parsers[i].getElementValue("bandIndex"));
          resolution=Integer.parseInt(parsers[i].getElementValue("bandResolution"));
          if (index==0 || index==3 || index==2 || index == 23)
          {
            BiggestInResolution = BiggestInResolution < resolution ? resolution : BiggestInResolution;
          }
        }
        SmallestIndex=resolutionIndex(BiggestInResolution);
        defaultIndex=DV.getResolutionCombo().getSelectedIndex();
        DV.getResolutionCombo().removeAllItems();
        for (i=0; i<ResolutionNumber; i++)
        {
          if (i>=SmallestIndex)
          {
            DV.getResolutionCombo().addItem(Modis.resolutionChoices[i]);
          }
        }
        if (SmallestIndex >= defaultIndex)
        {
          DV.getResolutionCombo().setSelection(Modis.resolutionChoices[SmallestIndex]);
        }
        else
        {
          DV.getResolutionCombo().setSelection(Modis.resolutionChoices[defaultIndex-SmallestIndex]);
        }
      }
      if (visual instanceof ModisTrueColorVisual)
      {
        ModisTrueColorVisual TV = (ModisTrueColorVisual)visual;
        for (i = 0; i < parsers.length; i++)
        {
          index=Integer.parseInt(parsers[i].getElementValue("bandIndex"));
          resolution=Integer.parseInt(parsers[i].getElementValue("bandResolution"));
          if (index==0 || index==3 || index==2)
          {
            BiggestInResolution = BiggestInResolution < resolution ? resolution : BiggestInResolution;
          }
        }
        SmallestIndex=resolutionIndex(BiggestInResolution);
        defaultIndex=TV.getResolutionCombo().getSelectedIndex();
        TV.getResolutionCombo().removeAllItems();
        for (i=0; i<ResolutionNumber; i++)
        {
          if (i>=SmallestIndex)
          {
            TV.getResolutionCombo().addItem(Modis.resolutionChoices[i]);
          }
        }
        if (SmallestIndex >= defaultIndex)
        {
          TV.getResolutionCombo().setSelection(Modis.resolutionChoices[SmallestIndex]);
        }
        else
        {
          TV.getResolutionCombo().setSelection(Modis.resolutionChoices[defaultIndex-SmallestIndex]);
        }
      }
      if (visual instanceof ModisInfraredVisual)
      {
        ModisInfraredVisual IV = (ModisInfraredVisual)visual;
        infraredBandIndex=IV.getBandCombo().getSelectedIndex();
        for (i = 0; i < parsers.length; i++)
        {
          index=Integer.parseInt(parsers[i].getElementValue("bandIndex"));
          resolution=Integer.parseInt(parsers[i].getElementValue("bandResolution"));
          // infrared start at 20.
          if (index == infraredBandIndex + 19) 
          {
            BiggestInResolution = BiggestInResolution < resolution ? resolution : BiggestInResolution;
          }
        }
        SmallestIndex=resolutionIndex(BiggestInResolution);
        defaultIndex=IV.getResolutionCombo().getSelectedIndex();
        IV.getResolutionCombo().removeAllItems();
        for (i=0; i<ResolutionNumber; i++)
        {
          if (i>=SmallestIndex)
          {
            IV.getResolutionCombo().addItem(Modis.resolutionChoices[i]);
          }
        }
        if (SmallestIndex >= defaultIndex)
        {
          IV.getResolutionCombo().setSelection(Modis.resolutionChoices[SmallestIndex]);
        }
        else
        {
          IV.getResolutionCombo().setSelection(Modis.resolutionChoices[defaultIndex-SmallestIndex]);
        }
      }
      if (visual instanceof ModisGrayScaleVisual)
      {
        ModisGrayScaleVisual GV = (ModisGrayScaleVisual)visual;
        grayBandIndex=GV.getBandCombo().getSelectedIndex();
        for (i = 0; i < parsers.length; i++)
        {
          index=Integer.parseInt(parsers[i].getElementValue("bandIndex"));
          resolution=Integer.parseInt(parsers[i].getElementValue("bandResolution"));
          if (index==grayBandIndex)
          {
            BiggestInResolution = BiggestInResolution < resolution ? resolution : BiggestInResolution;
          }
        }
        SmallestIndex=resolutionIndex(BiggestInResolution);
        defaultIndex=GV.getResolutionCombo().getSelectedIndex();
        GV.getResolutionCombo().removeAllItems();
        for (i=0; i<ResolutionNumber; i++)
        {
          if (i>=SmallestIndex)
          {
            GV.getResolutionCombo().addItem(Modis.resolutionChoices[i]);
          }
        }
        if (SmallestIndex >= defaultIndex)
        {
          GV.getResolutionCombo().setSelection(Modis.resolutionChoices[SmallestIndex]);
        }
        else
        {
          GV.getResolutionCombo().setSelection(Modis.resolutionChoices[defaultIndex-SmallestIndex]);
        }
      }
      if (visual instanceof ModisFalseColorVisual)
      {
        ModisFalseColorVisual FV = (ModisFalseColorVisual)visual;
        redBandIndex=FV.getRedBandCombo().getSelectedIndex();
        greenBandIndex=FV.getGreenBandCombo().getSelectedIndex();
        blueBandIndex=FV.getBlueBandCombo().getSelectedIndex();
        for (i = 0; i < parsers.length; i++)
        {
          index=Integer.parseInt(parsers[i].getElementValue("bandIndex"));
          resolution=Integer.parseInt(parsers[i].getElementValue("bandResolution"));
          if (index==redBandIndex || index==greenBandIndex || index==blueBandIndex)
          {
            BiggestInResolution = BiggestInResolution < resolution ? resolution : BiggestInResolution;
          }
        }
        SmallestIndex=resolutionIndex(BiggestInResolution);
        defaultIndex=FV.getResolutionCombo().getSelectedIndex();
        FV.getResolutionCombo().removeAllItems();
        for (i=0; i<ResolutionNumber; i++)
        {
          if (i>=SmallestIndex)
          {
            FV.getResolutionCombo().addItem(Modis.resolutionChoices[i]);
          }
        }
        if (SmallestIndex >= defaultIndex)
        {
          FV.getResolutionCombo().setSelection(Modis.resolutionChoices[SmallestIndex]);
        }
        else
        {
          FV.getResolutionCombo().setSelection(Modis.resolutionChoices[defaultIndex-SmallestIndex]);
        }
      }

      visualPanel.add((Component)visual);
      visualPanel.revalidate();
      visualPanel.repaint();
    }
  }
  /****************************************************************************
  * InfraredBand.
  ****************************************************************************/
  public void infraredBand (XmlParser parser) throws Exception 
  {
    resolution = 0; 
    BiggestInResolution = 0; 
    SmallestIndex = 0;

    if (Viewer.mapLayoutButton.isSelected())
    {
      visualPanel=Viewer.mapLayout.getVisualSelector().getVisualPanel();
      visual=Viewer.mapLayout.getVisualSelector().getVisual();
    }
    if (Viewer.instrumentLayoutButton.isSelected())
    {
      visualPanel=Viewer.instrumentLayout.getVisualSelector().getVisualPanel();
      visual=Viewer.instrumentLayout.getVisualSelector().getVisual();
    }
 
    visualPanel.removeAll();

    if (parser.hasElement("more"))
    {
      XmlParser[] parsers = parser.getElement("more").getElement("bands").getElements("band");

      ModisInfraredVisual IV = (ModisInfraredVisual)visual;
      infraredBandIndex=IV.getBandCombo().getSelectedIndex();
      for (i = 0; i < parsers.length; i++)
      {
        index=Integer.parseInt(parsers[i].getElementValue("bandIndex"));
        resolution=Integer.parseInt(parsers[i].getElementValue("bandResolution"));
        if (index==infraredBandIndex + 19)
        {
          BiggestInResolution = resolution;
        }
      }

      SmallestIndex=resolutionIndex(BiggestInResolution);
      defaultIndex=IV.getResolutionCombo().getSelectedIndex();

      IV.getResolutionCombo().removeAllItems();
      for (i=0; i<ResolutionNumber; i++)
      {
        if (i>=SmallestIndex)
        {
          IV.getResolutionCombo().addItem(Modis.resolutionChoices[i]);
        }
      }
      if (SmallestIndex >= defaultIndex)
      {
        IV.getResolutionCombo().setSelection(Modis.resolutionChoices[SmallestIndex]);
      }
      else
      {
        IV.getResolutionCombo().setSelection(Modis.resolutionChoices[defaultIndex-SmallestIndex]);
      }
    }
    
    visualPanel.add((Component)visual);
    visualPanel.revalidate();
    visualPanel.repaint();
  }
  /****************************************************************************
  * GrayBand.
  ****************************************************************************/
  public void grayBand (XmlParser parser) throws Exception 
  {
    resolution = 0; 
    BiggestInResolution = 0; 
    SmallestIndex = 0;

    if (Viewer.mapLayoutButton.isSelected())
    {
      visualPanel=Viewer.mapLayout.getVisualSelector().getVisualPanel();
      visual=Viewer.mapLayout.getVisualSelector().getVisual();
    }
    if (Viewer.instrumentLayoutButton.isSelected())
    {
      visualPanel=Viewer.instrumentLayout.getVisualSelector().getVisualPanel();
      visual=Viewer.instrumentLayout.getVisualSelector().getVisual();
    }

    visualPanel.removeAll();

    if (parser.hasElement("more"))
    {
      XmlParser[] parsers = parser.getElement("more").getElement("bands").getElements("band");

      ModisGrayScaleVisual GV = (ModisGrayScaleVisual)visual;
      grayBandIndex=GV.getBandCombo().getSelectedIndex();

      for (i = 0; i < parsers.length; i++)
      {
        index=Integer.parseInt(parsers[i].getElementValue("bandIndex"));
        resolution=Integer.parseInt(parsers[i].getElementValue("bandResolution"));
        if (index==grayBandIndex)
        {
          BiggestInResolution = resolution;
        }
      }

      SmallestIndex=resolutionIndex(BiggestInResolution);
      defaultIndex=GV.getResolutionCombo().getSelectedIndex();

      GV.getResolutionCombo().removeAllItems();
      for (i=0; i<ResolutionNumber; i++)
      {
        if (i>=SmallestIndex)
        {
          GV.getResolutionCombo().addItem(Modis.resolutionChoices[i]);
        }
      }
      if (SmallestIndex >= defaultIndex)
      {
        GV.getResolutionCombo().setSelection(Modis.resolutionChoices[SmallestIndex]);
      }
      else
      {
        GV.getResolutionCombo().setSelection(Modis.resolutionChoices[defaultIndex-SmallestIndex]);
      }
    }
    
    visualPanel.add((Component)visual);
    visualPanel.revalidate();
    visualPanel.repaint();
  }
  /****************************************************************************
  * ColorBand.
  ****************************************************************************/
  public void colorBand (XmlParser parser) throws Exception 
  {
    resolution = 0; 
    BiggestInResolution = 0; 
    SmallestIndex = 0;

    if (Viewer.mapLayoutButton.isSelected())
    {
      visualPanel=Viewer.mapLayout.getVisualSelector().getVisualPanel();
      visual=Viewer.mapLayout.getVisualSelector().getVisual();
    }
    if (Viewer.instrumentLayoutButton.isSelected())
    {
      visualPanel=Viewer.instrumentLayout.getVisualSelector().getVisualPanel();
      visual=Viewer.instrumentLayout.getVisualSelector().getVisual();
    }

    visualPanel.removeAll();

    if (parser.hasElement("more"))
    {
      XmlParser[] parsers = parser.getElement("more").getElement("bands").getElements("band");

      ModisFalseColorVisual FV = (ModisFalseColorVisual)visual;
      redBandIndex=FV.getRedBandCombo().getSelectedIndex();
      greenBandIndex=FV.getGreenBandCombo().getSelectedIndex();
      blueBandIndex=FV.getBlueBandCombo().getSelectedIndex();

      for (i = 0; i < parsers.length; i++)
      {
        index=Integer.parseInt(parsers[i].getElementValue("bandIndex"));
        resolution=Integer.parseInt(parsers[i].getElementValue("bandResolution"));
        if (index==redBandIndex || index==greenBandIndex || index==blueBandIndex)
        {
          BiggestInResolution = BiggestInResolution < resolution ? resolution : BiggestInResolution;
        }
      }

      SmallestIndex=resolutionIndex(BiggestInResolution);
      defaultIndex=FV.getResolutionCombo().getSelectedIndex();

      FV.getResolutionCombo().removeAllItems();
      for (i=0; i<ResolutionNumber; i++)
      {
        if (i>=SmallestIndex)
        {
          FV.getResolutionCombo().addItem(Modis.resolutionChoices[i]);
        }
      }
      if (SmallestIndex >= defaultIndex)
      {
        FV.getResolutionCombo().setSelection(Modis.resolutionChoices[SmallestIndex]);
      }
      else
      {
        FV.getResolutionCombo().setSelection(Modis.resolutionChoices[defaultIndex-SmallestIndex]);
      }
    }
    
    visualPanel.add((Component)visual);
    visualPanel.revalidate();
    visualPanel.repaint();
  }
  /****************************************************************************
  * set.
  ****************************************************************************/
  private void set (XmlParser parser) throws Exception {
    detailsPanel.removeAll();
    detailsPanel.add(new HPanel(new CustomLabel("PassName: "+parser.getElementValue("name")),GUtil.HGlue()));
    if (parser.hasElement("startedAt")) {
      startedAt = Long.parseLong(parser.getElementValue("startedAt"));
      platformCode = Integer.parseInt(parser.getElementValue("platformCode"));
      instrumentCode = Integer.parseInt(parser.getElementValue("instrumentCode"));
      startSolarCode = Integer.parseInt(parser.getElementValue("startSolarCode"));
      directionCode = Integer.parseInt(parser.getElementValue("directionCode"));
      GroundStation groundStation = new GroundStation(parser.getElement("groundStation"));
      groundStationField.update(groundStation.getName());
      platformField.update(Simulcast.getPlatformName(platformCode));
      instrumentField.update(Simulcast.getInstrumentName(instrumentCode));
      startedField.update(encodeDateTime(startedAt));
      solarField.update(getPassSolar(startSolarCode,Simulcast.UNKNOWN));
      directionField.update(Simulcast.getPassDirection(directionCode));
      
      detailsPanel.add(new HPanel(new CustomLabel("StartedAt: "+encodeDateTime(Long.parseLong(parser.getElementValue("startedAt")))),GUtil.HGlue()));
      detailsPanel.add(new HPanel(new CustomLabel("Satellite: "+Simulcast.getPlatformName(Integer.parseInt(parser.getElementValue("platformCode")))),GUtil.HGlue()));
      detailsPanel.add(new HPanel(new CustomLabel("Instrument: "+Simulcast.getInstrumentName(Integer.parseInt(parser.getElementValue("instrumentCode")))),GUtil.HGlue()));
      detailsPanel.add(new HPanel(new CustomLabel("StartSolar: "+Simulcast.getPassSolar(Integer.parseInt(parser.getElementValue("startSolarCode")))),GUtil.HGlue()));
      detailsPanel.add(new HPanel(new CustomLabel("Direction: "+Simulcast.getPassDirection(Integer.parseInt(parser.getElementValue("directionCode")))),GUtil.HGlue()));
      detailsPanel.add(new HPanel(new CustomLabel("GroundStation: "+groundStation.getName()),GUtil.HGlue()));
    }
    if (parser.hasElement("endedAt")) {
      endedAt = Long.parseLong(parser.getElementValue("endedAt"));
      endedField.update(encodeDateTime(endedAt));
      solarField.update(getPassSolar(startSolarCode,Integer.parseInt(parser.getElementValue("endSolarCode"))));
      durationField.update(encodeDuration(startedAt,endedAt));
      scansField.update(encodeScanCounts(startSolarCode,Integer.parseInt(parser.getElementValue("dayScanCount")),Integer.parseInt(parser.getElementValue("nightScanCount"))));

      detailsPanel.add(new HPanel(new CustomLabel("EndedAt: "+encodeDateTime(Long.parseLong(parser.getElementValue("endedAt")))),GUtil.HGlue()));
      detailsPanel.add(new HPanel(new CustomLabel("EndSolar: "+Simulcast.getPassSolar(Integer.parseInt(parser.getElementValue("endSolarCode")))),GUtil.HGlue()));
      detailsPanel.add(new HPanel(new CustomLabel("DayScans: "+Integer.parseInt(parser.getElementValue("dayScanCount"))),GUtil.HGlue()));
      detailsPanel.add(new HPanel(new CustomLabel("NightScans: "+Integer.parseInt(parser.getElementValue("nightScanCount"))),GUtil.HGlue()));
    }
    if (parser.hasElement("more")) {
      setMore(parser.getElement("more"));
    }
    if (parser.hasElement("error")) {
      // errorField.update("ERROR..."+parser.getElementValue("error"),Color.red);
      if (parser.getElementValue("error").equals("no scans"))
      {
        errorField.update("No scans.",Color.black);
        detailsPanel.add(new HPanel(new CustomLabel("No scans were acquired during this pass.",Color.black),GUtil.HGlue()));
      }
      else if (parser.getElementValue("error").startsWith("timeout while"))
      {
        errorField.update("Incomplete scans.",Color.black);
        detailsPanel.add(new HPanel(new CustomLabel("Network or upstream server problem occured during acquisition of this pass.",Color.black),GUtil.HGlue()));
      }
      else if (parser.getElementValue("error").startsWith("java.net.SocketException"))
      {
        errorField.update("Incomplete scans.",Color.black);
        detailsPanel.add(new HPanel(new CustomLabel("Network or upstream server problem occured during acquisition of this pass.",Color.black),GUtil.HGlue()));
      }
      else if (parser.getElementValue("error").startsWith("java.io.EOFException"))
      {
        errorField.update("Interrupted scans.",Color.black);
        detailsPanel.add(new HPanel(new CustomLabel("Data or Network problem occured during acquisition of this pass.",Color.black),GUtil.HGlue()));
      }
      else
      {
        errorField.update("ERROR",Color.red);
        detailsPanel.add(new HPanel(new CustomLabel("ERROR..."+parser.getElementValue("error"),Color.black),GUtil.HGlue()));
      }
    }
    else
    {
	//Need to update error message to none after fix.
        errorField.update("",Color.black);
    }
    detailsPanel.revalidate();
    detailsPanel.repaint();
  }
  private void setMore (XmlParser parser) throws Exception {
    if (parser.hasElement("geolocated")) {
      detailsPanel.add(new HPanel(new CustomLabel("Geolocation: "+parser.getElementValue("geoResolution")+"m, "+parser.getElementValue("geoBlockRowCount")+"x"+parser.getElementValue("geoBlockColCount")),GUtil.HGlue()));
    } else {
      detailsPanel.add(new HPanel(new CustomLabel("Geolocation: "),new CustomLabel("NONE",Color.RED),GUtil.HGlue()));
    }
    if (!parser.hasElement("debowed")) {
      detailsPanel.add(new HPanel(new CustomLabel("Debowing: "),new CustomLabel("NONE",Color.RED),GUtil.HGlue()));
    }
    detailsPanel.add(new HPanel(new CustomLabel("CoarsestBand: "+parser.getElementValue("coarsestResolution")+"m, "+parser.getElementValue("coarsestBlockRowCount")+"x"+parser.getElementValue("coarsestBlockColCount")),GUtil.HGlue()));
    XmlParser[] parsers = parser.getElement("bands").getElements("band");
    HPanel h1 = new HPanel();
    for (int b = 0; b < parsers.length; b++) {
       h1.add(new HPanel(GUtil.HSpace(10),new CustomLabel(Modis.dayBandChoices[Integer.parseInt(parsers[b].getElementValue("bandIndex"))]+"/"+parsers[b].getElementValue("bandResolution")+"m/"+parsers[b].getElementValue("blockRowCount")+"x"+parsers[b].getElementValue("blockColCount"))));
    }
    detailsPanel.add(new HPanel(new CustomLabel("Bands:"),h1,GUtil.HGlue()));
  }
  /****************************************************************************
  * getPassName.
  ****************************************************************************/
  public String getPassName () {
    return passName;
  }
  /****************************************************************************
  * getStationId.
  ****************************************************************************/
  public String getStationId () {
    return stationId;
  }
  /****************************************************************************
  * hasHeader.
  ****************************************************************************/
  public boolean hasHeader () {
    return (startedAt != Simulcast.UNKNOWN);
  }
  /****************************************************************************
  * hasFooter.
  ****************************************************************************/
  public boolean hasFooter () {
    return (endedAt != Simulcast.UNKNOWN);
  }
  /****************************************************************************
  * getStartedAt.
  ****************************************************************************/
  public long getStartedAt () {
    return startedAt;
  }
  /****************************************************************************
  * getSatelliteCode.
  ****************************************************************************/
  public long getSatelliteCode () {
    return platformCode;
  }
  /****************************************************************************
  * getStartSolarCode.
  ****************************************************************************/
  public long getStartSolarCode () {
    return startSolarCode;
  }
  /****************************************************************************
  * getPassSolar.
  ****************************************************************************/
  private String getPassSolar (int startSolarCode, int endSolarCode) {
    StringBuffer passSolar = new StringBuffer(Simulcast.getPassSolar(startSolarCode));
    if (endSolarCode != Simulcast.UNKNOWN && endSolarCode != startSolarCode) {
      passSolar.append("/" + Simulcast.getPassSolar(endSolarCode));
    }
    return passSolar.toString();
  }
  /****************************************************************************
  * encodeDateTime.
  ****************************************************************************/
  private String encodeDateTime (long at) {
    if (at != Simulcast.UNKNOWN) {
      return Util.encodeDateTime(at);
    }
    return " ";
  }
  /****************************************************************************
  * encodeDuration.
  ****************************************************************************/
  private String encodeDuration (long startedAt, long endedAt) {
    if (endedAt != Simulcast.UNKNOWN) {
      int duration = (int) (endedAt - startedAt);
      return Util.encodeDuration(duration);
    }
    return " ";
  }
  /****************************************************************************
  * encodeScanCounts
  ****************************************************************************/
  private String encodeScanCounts (int startSolarCode, int nDayScans, int nNightScans) {
    if (nDayScans > 0) {
      if (nNightScans > 0) {
	if (startSolarCode == Simulcast.DAY_PASS_CODE) {
	  return Integer.toString(nDayScans) + "/" + Integer.toString(nNightScans);
	} else {
	  return Integer.toString(nNightScans) + "/" + Integer.toString(nDayScans);
	}
      } else {
        return Integer.toString(nDayScans);
      }
    } else {
      if (nNightScans > 0) {
        return Integer.toString(nNightScans);
      } else {
        return " ";
      }
    } 
  }
  /****************************************************************************
  * MapButtonClicked.
  ****************************************************************************/
  private class MapButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      loadMap(((ae.getModifiers() & ActionEvent.CTRL_MASK) == 0),false);
      // Send request to server to get data band resolution.
      mapBand();
    }
  }
  /****************************************************************************
  * MapBand.
  ****************************************************************************/
  private void mapBand () 
  {
    //Get pass name.
    String passName=null;
    try
    {
      passName=Viewer.mapLayout.getPlayerManager().getPlayerCombo().getSelection();
    }
    catch (NullPointerException e)
    {
      // System.out.println("\n No passName at start of viewer");
      return;
    }

    // Send message to server to get BiggestInResolution from bands.

    Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
    "<mapInstrumentBand>" + "<name>" + passName + "</name>" +
    "</mapInstrumentBand>" + "</simulcastMessage>");
  }
  /****************************************************************************
  * InstrumentButtonClicked.
  ****************************************************************************/
  private class InstrumentButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      loadInstrument(((ae.getModifiers() & ActionEvent.CTRL_MASK) == 0),false);
      // Send request to server to get data band resolution.
      instrumentBand();
    }
  }
  /****************************************************************************
  * InstrumentBand.
  ****************************************************************************/
  private void instrumentBand () 
  {
    //Get pass name.
    String passName=null;
    try
    {
      passName=Viewer.instrumentLayout.getPlayerManager().getPlayerCombo().getSelection();
    }
    catch (NullPointerException e)
    {
      // System.out.println("\n No passName at start of viewer");
      return;
    }

    // Send message to server to get BiggestInResolution from bands.
    Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
    "<mapInstrumentBand>" + "<name>" + passName + "</name>" +
    "</mapInstrumentBand>" + "</simulcastMessage>");
  }
  /****************************************************************************
  * DetailsButtonClicked.
  ****************************************************************************/
  private class DetailsButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      if (detailsFrame != null) {
	detailsFrame.dispose();
      }
      detailsFrame = new JFrame();
      detailsFrame.setTitle("Simulcast Viewer - Details for "+passName);
      detailsFrame.getContentPane().setLayout(new BorderLayout());
      detailsPanel = new VPanel(new MarginPanel(-1,-1,new CustomLabel("Loading..."),-1,-1));
      detailsFrame.getContentPane().add(new JScrollPane(detailsPanel),BorderLayout.CENTER);
      detailsFrame.pack();
      detailsFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
      GUtil.configureWindow(detailsFrame,0.60,0.60);
      detailsFrame.setVisible(true);
      Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
         "<sendPassDetails>" + "<passName>" + passName + "</passName>" +
         "</sendPassDetails>" + "</simulcastMessage>");
    }
  }
  /****************************************************************************
  * loopLoad.
  ****************************************************************************/
  public void loopLoad (Layout layout) {
    layout.loadPass(passName,platformCode,instrumentCode,directionCode,startSolarCode,true);
  }
  /****************************************************************************
  * loadIfSelected.
  ****************************************************************************/
  public void loadIfSelected (Layout layout) {
    if (selectCheckBox.isSelected()) {
      if (startedAt != Simulcast.UNKNOWN) {
	layout.loadPass(passName,platformCode,instrumentCode,directionCode,startSolarCode,false);
      }
      selectCheckBox.setSelected(false);
    }
  }
  /****************************************************************************
  * loadMap.
  ****************************************************************************/
  public boolean loadMap (boolean switchTo, boolean autoPlay) {
    if (startedAt != Simulcast.UNKNOWN) {
      if (switchTo) {
	Viewer.switchToMapLayout();
      }
      Viewer.mapLayout.loadPass(passName,platformCode,instrumentCode,directionCode,startSolarCode,autoPlay);
      return true;
    }
    return false;
  }
  /****************************************************************************
  * loadInstrument.
  ****************************************************************************/
  public boolean loadInstrument (boolean switchTo, boolean autoPlay) {
    if (startedAt != Simulcast.UNKNOWN) {
      if (switchTo) {
	Viewer.switchToInstrumentLayout();
      }
      Viewer.instrumentLayout.loadPass(passName,platformCode,instrumentCode,directionCode,startSolarCode,autoPlay);
      return true;
    }
    return false;
  }
}
