/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  17-Dec-04, J Love, GST	Original version.
*  26-Aug-05, J Love, GST	Added initial settings.
*   8-Feb-06, J Love, GST	Appearance tweaking.
*  18-Oct-06, J Love, GST	Visual preferences saving.
*  10-Jan-07, J Love, GST	Place values labels.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;

public class PictureControls extends HPanel {
  private IntegerSlider brightnessSlider;
  private RealSlider contrastSlider;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PictureControls.
  ****************************************************************************/
  public PictureControls (double initialContrast, int initialBrightness) {
    brightnessSlider = new IntegerSlider(-99,initialBrightness,99,"Brightness setting (added to values that have been scaled 0 to 255).  Applied after contrast adjustment.");
    contrastSlider = new RealSlider(0.0,initialContrast,9.99,"Contrast setting (values scaled 0 to 255 are multiplied by this setting).  Applied before brightness adjustment.");
    VPanel v1 = new VPanel(TOP_ALIGNMENT,new HPanel(/*CENTER_ALIGNMENT,*/GUtil.HSpace(8),new CustomLabel("Contrast"),GUtil.HGlue(),contrastSlider.getValueLabel(),GUtil.HSpace(9)),GUtil.VSpace(4),/*new HPanel(CENTER_ALIGNMENT,*/contrastSlider/*)*/);
    VPanel v2 = new VPanel(TOP_ALIGNMENT,new HPanel(/*CENTER_ALIGNMENT,*/GUtil.HSpace(8),new CustomLabel("Brightness"),GUtil.HGlue(),brightnessSlider.getValueLabel(),GUtil.HSpace(9)),GUtil.VSpace(4),/*new HPanel(CENTER_ALIGNMENT,*/brightnessSlider/*)*/);
    add(new HPanel(v1,GUtil.HSpace(15),v2));
  }
  /****************************************************************************
  * getContrast.
  ****************************************************************************/
  public double getContrast () {
    return contrastSlider.getValue();
  }
  /****************************************************************************
  * getBrightness.
  ****************************************************************************/
  public int getBrightness () {
    return brightnessSlider.getValue();
  }
  /****************************************************************************
  * setContrast.
  ****************************************************************************/
  public void setContrast (double value) {
    contrastSlider.setValue(value);
  }
  /****************************************************************************
  * setBrightness.
  ****************************************************************************/
  public void setBrightness (int value) {
    brightnessSlider.setValue(value);
  }
  /****************************************************************************
  * adjustValue.
  ****************************************************************************/
  public int adjustValue (int value) {
    return Util.adjustValue(value,contrastSlider.getValue(),brightnessSlider.getValue());
  }
}
