/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  18-Oct-06, J Love, GST	Original version.
*  15-Mar-07, Z Yin,  GST       Displaying image resolution choices equal or larger
*                               then data resolution.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;

public class ServerListener extends HPanel implements ServerHandlerListener {
  private CustomLabel status;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ServerListener.
  ****************************************************************************/
  public ServerListener () {
    status = new CustomLabel("Not Connected.",Color.BLACK);
    add(status);
    add(GUtil.HGlue());
  }
  /****************************************************************************
  * info.
  ****************************************************************************/
  public void info (String text) {
    GUtil.invokeAndWait(new UpdateLabel(status,text,Color.BLACK));
  }
  /****************************************************************************
  * error.
  ****************************************************************************/
  public void error (String text, boolean wasConnected) {
    GUtil.invokeAndWait(new UpdateLabel(status,text,Color.RED));
    if (wasConnected) {
      GUtil.invokeAndWait(new DoDisconnected());
    }
  }
  /****************************************************************************
  * connected.
  ****************************************************************************/
  public void connected (String hostPort) {
    String text = "Connected to server " + hostPort + ".";
    GUtil.invokeAndWait(new UpdateLabel(status,text,Color.BLACK));
    Viewer.frame.setTitle("Simulcast Viewer - " + hostPort);
    Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" + "<clientMode/>" + "</simulcastMessage>");
  }
  /****************************************************************************
  * received.
  ****************************************************************************/
  public void received (String msg) {
    // System.out.println("ServerListener.received/msg: " + msg);
    GUtil.invokeAndWait(new DoMessage(msg));
  }
  private class DoMessage implements Runnable {
    private String msg;
    public DoMessage (String msg) {
      this.msg = msg;
    }
    public void run () {
      try {
	doMessage(msg);
      } catch (Exception e) {
      	e.printStackTrace();
      }
    }
    private void doMessage (String msg) throws Exception {
      // System.out.println("ServerListener.DoMessage.doMessage, msg: " + msg);
      XmlParser parser = new XmlParser(msg,"simulcastMessage");
      if (parser.hasElement("serverReady")) {

        return;
      }
      if (parser.hasElement("passListing")) {
        Viewer.passManager.handlePassListing(parser.getElement("passListing"));
        return;
      }
      if (parser.hasElement("passDetails")) {
        Viewer.passManager.handlePassDetails(parser.getElement("passDetails"));
        return;
      }
      if (parser.hasElement("passStarted")) {
        Viewer.passManager.handlePassStarted(parser.getElement("passStarted").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("passUpdated")) {
        Viewer.passManager.handlePassUpdated(parser.getElement("passUpdated").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("passEnded")) {
        Viewer.passManager.handlePassEnded(parser.getElement("passEnded").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("passRemoved")) {
        Viewer.passManager.handlePassRemoved(parser.getElement("passRemoved"));
        return;
      }
      if (parser.hasElement("clientId")) {
        Viewer.clientId = parser.getElementValue("clientId");
        Viewer.serverSimulcastVersion = parser.getElementValue("simulcastVersion");
        return;
      }
      if (parser.hasElement("mostRecentTLEs")) {
        String[] aquaTLE = new String[] {
          parser.getElement("mostRecentTLEs").getElement("aqua").getElement("tle").getElementValue("line1"),
          parser.getElement("mostRecentTLEs").getElement("aqua").getElement("tle").getElementValue("line2")
        };
        String[] terraTLE = new String[] {
          parser.getElement("mostRecentTLEs").getElement("terra").getElement("tle").getElementValue("line1"),
          parser.getElement("mostRecentTLEs").getElement("terra").getElement("tle").getElementValue("line2")
        };
        Viewer.mapLayout.setTLEs(aquaTLE,terraTLE);
        return;
      }
      if (parser.hasElement("groundStations")) {
      	Viewer.mapLayout.setGroundStations(new GroundStations(parser.getElement("groundStations")));
      	return;
      }
      if (parser.hasElement("visualBand")) {
        Viewer.passManager.handleVisualBand(parser.getElement("visualBand").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("infraredBand")) {
        Viewer.passManager.handleInfraredBand(parser.getElement("infraredBand").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("grayBand")) {
        Viewer.passManager.handleGrayBand(parser.getElement("grayBand").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("redBand")) {
        Viewer.passManager.handleColorBand(parser.getElement("redBand").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("greenBand")) {
        Viewer.passManager.handleColorBand(parser.getElement("greenBand").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("blueBand")) {
        Viewer.passManager.handleColorBand(parser.getElement("blueBand").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("mapInstrumentBand")) {
        Viewer.passManager.handleVisualBand(parser.getElement("mapInstrumentBand").getElement("passDetails"));
        return;
      }
      throw new Exception("unknown message from server");
    }
  }
  /****************************************************************************
  * disconnected.
  ****************************************************************************/
  public void disconnected () {
    GUtil.invokeAndWait(new UpdateLabel(status,"Not Connected.",Color.BLACK));
    GUtil.invokeAndWait(new DoDisconnected());
  }
  /****************************************************************************
  * DoDisconnected.
  ****************************************************************************/
  private class DoDisconnected implements Runnable {
    public void run () {
      Viewer.frame.setTitle("Simulcast Viewer");
      Viewer.serverSimulcastVersion = null;
      Viewer.passManager.clear();
      Viewer.mapLayout.clear();
      Viewer.instrumentLayout.clear();
    }
  }
}
