/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  18-Oct-06, J Love, GST	Original version.
*  10-Jan-07, J Love, GST	Appearance tweaking.
*  26-Feb-07, J Love, GST	More tool tips.
*  15-Mar-07, Z Yin,  GST       Displaying image resolution choices equal or larger
*                               then data resolution.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.Socket;

public class VisualSelector extends HPanel {
  private CustomCombo visualCombo;
  private HPanel visualPanel;
  private Visual[] visuals;
  private Visual visual;

  public static final int MODIS_AUTO_DAY_NIGHT_VISUAL = 0;
  public static final int MODIS_TRUE_COLOR_VISUAL = 1;
  public static final int MODIS_INFRARED_VISUAL = 2;
  public static final int MODIS_GRAY_SCALE_VISUAL = 3;
  public static final int MODIS_FALSE_COLOR_VISUAL = 4;
  public static final String[] visualChoices = new String[] {
    "MODIS Auto Day/Night","MODIS True Color","MODIS Infrared","MODIS Gray Scale","MODIS False Color"
  };
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * VisualSelector.
  ****************************************************************************/
  public VisualSelector () {
    this(new Visual[] {new ModisAutoDayNightVisual(),new ModisTrueColorVisual(),new ModisInfraredVisual(),new ModisGrayScaleVisual(),new ModisFalseColorVisual()},0);
  }
  private VisualSelector (Visual[] visuals, int index) {
    this.visuals = visuals;
    visual = visuals[index];
    visualCombo = new CustomCombo(visualChoices,index);
    visualCombo.setToolTipText("Selects the visual to be used.");
    visualCombo.addActionListener(new VisualSelected());
    visualPanel = new HPanel((Component)visual);
    add(new HPanel(new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(3),new CustomLabel("Visual")),new HPanel(LEFT_ALIGNMENT,visualCombo)),GUtil.HSpace(10),visualPanel));
  }
  /****************************************************************************
  * setEnabled.
  ****************************************************************************/
  public void setEnabled (boolean enabled) {
    visual.setControls(enabled);
    visualCombo.setEnabled(enabled);
  }
  /****************************************************************************
  * getVisual.
  ****************************************************************************/
  public Visual getVisual () {
    return visual;
  }
  /****************************************************************************
  * getVisual.
  ****************************************************************************/
  public HPanel getVisualPanel () {
    return visualPanel;
  }
  /****************************************************************************
  * newInstance.
  ****************************************************************************/
  public VisualSelector newInstance () {
    // System.out.println("\n Create all visual.");
    return new VisualSelector(new Visual[] {
      visuals[MODIS_AUTO_DAY_NIGHT_VISUAL].newInstance(),
      visuals[MODIS_TRUE_COLOR_VISUAL].newInstance(),
      visuals[MODIS_INFRARED_VISUAL].newInstance(),
      visuals[MODIS_GRAY_SCALE_VISUAL].newInstance(),
      visuals[MODIS_FALSE_COLOR_VISUAL].newInstance()
    },visualCombo.getSelectedIndex());
  }
  /****************************************************************************
  * VisualSelected.
  ****************************************************************************/
  private class VisualSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      visualPanel.removeAll();
      visual = visuals[visualCombo.getSelectedIndex()];
      visualPanel.add((Component)visual);
      visualPanel.revalidate();
      visualPanel.repaint();

      try
      {
        if (Viewer.mapLayoutButton.isSelected())
        {
          String mapPassName=null;
          mapPassName=Viewer.mapLayout.getPlayerManager().getPlayerCombo().getSelection();

          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<visualBand>" + "<name>" + mapPassName + "</name>" +
          "</visualBand>" + "</simulcastMessage>");
        }

        if (Viewer.instrumentLayoutButton.isSelected())
        {
          String instrumentPassName=null;
          instrumentPassName=Viewer.instrumentLayout.getPlayerManager().getPlayerCombo().getSelection();

          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<visualBand>" + "<name>" + instrumentPassName + "</name>" +
          "</visualBand>" + "</simulcastMessage>");
        }
      }
      catch (NullPointerException me)
      {
        // System.out.println("\n No mapPassName.");
      }
    }
  }
  /****************************************************************************
  * set.
  ****************************************************************************/
  public void set (XmlParser parser) throws Exception {
    visualCombo.setSelection(parser.getAttributeValue("default"));
    for (XmlParser p : parser.getElements("visual")) {
       visuals[Util.indexOf(p.getAttributeValue("name"),VisualSelector.visualChoices)].set(p);
    }
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public void toXml (PrintWriter out) throws Exception {
    out.println("  <visuals default=\"" + visualCombo.getSelection() + "\">");
    visuals[MODIS_AUTO_DAY_NIGHT_VISUAL].toXml(out);
    visuals[MODIS_TRUE_COLOR_VISUAL].toXml(out);
    visuals[MODIS_INFRARED_VISUAL].toXml(out);
    visuals[MODIS_GRAY_SCALE_VISUAL].toXml(out);
    visuals[MODIS_FALSE_COLOR_VISUAL].toXml(out);
    out.println("  </visuals>");
  }
}
